#include <ROL_AbsoluteValue.hpp>
#include <ROL_Arcsine.hpp>
#include <ROL_BPOE.hpp>
#include <ROL_BatchManager.hpp>
#include <ROL_Beta.hpp>
#include <ROL_BoundConstraint.hpp>
#include <ROL_CVaR.hpp>
#include <ROL_Cauchy.hpp>
#include <ROL_ChebyshevSpectral.hpp>
#include <ROL_Chi2Divergence.hpp>
#include <ROL_CoherentEntropicRisk.hpp>
#include <ROL_Constraint.hpp>
#include <ROL_ConstraintFromObjective.hpp>
#include <ROL_ConvexCombinationRiskMeasure.hpp>
#include <ROL_DeviationMeasureFactory.hpp>
#include <ROL_DeviationMeasureInfo.hpp>
#include <ROL_Dirac.hpp>
#include <ROL_Distribution.hpp>
#include <ROL_DistributionFactory.hpp>
#include <ROL_Elementwise_Function.hpp>
#include <ROL_Elementwise_Reduce.hpp>
#include <ROL_EntropicRisk.hpp>
#include <ROL_ErrorMeasureFactory.hpp>
#include <ROL_ErrorMeasureInfo.hpp>
#include <ROL_ExpectationQuad.hpp>
#include <ROL_ExpectationQuadDeviation.hpp>
#include <ROL_ExpectationQuadError.hpp>
#include <ROL_ExpectationQuadRegret.hpp>
#include <ROL_ExpectationQuadRisk.hpp>
#include <ROL_Exponential.hpp>
#include <ROL_FDivergence.hpp>
#include <ROL_Fejer2Quadrature.hpp>
#include <ROL_Gamma.hpp>
#include <ROL_GaussChebyshev1Quadrature.hpp>
#include <ROL_GaussChebyshev2Quadrature.hpp>
#include <ROL_GaussChebyshev3Quadrature.hpp>
#include <ROL_GaussLegendreQuadrature.hpp>
#include <ROL_Gaussian.hpp>
#include <ROL_GenMoreauYosidaCVaR.hpp>
#include <ROL_HMCR.hpp>
#include <ROL_KLDivergence.hpp>
#include <ROL_Kumaraswamy.hpp>
#include <ROL_Laplace.hpp>
#include <ROL_LogExponentialQuadrangle.hpp>
#include <ROL_LogQuantileQuadrangle.hpp>
#include <ROL_Logistic.hpp>
#include <ROL_MeanDeviation.hpp>
#include <ROL_MeanDeviationFromTarget.hpp>
#include <ROL_MeanSemiDeviation.hpp>
#include <ROL_MeanSemiDeviationFromTarget.hpp>
#include <ROL_MeanValueObjective.hpp>
#include <ROL_MeanVariance.hpp>
#include <ROL_MeanVarianceFromTarget.hpp>
#include <ROL_MeanVarianceQuadrangle.hpp>
#include <ROL_MixedCVaR.hpp>
#include <ROL_MoreauYosidaCVaR.hpp>
#include <ROL_Objective.hpp>
#include <ROL_Parabolic.hpp>
#include <ROL_PlusFunction.hpp>
#include <ROL_PolyhedralProjection.hpp>
#include <ROL_PositiveFunction.hpp>
#include <ROL_ProbabilityFactory.hpp>
#include <ROL_ProbabilityInfo.hpp>
#include <ROL_Problem.hpp>
#include <ROL_Quadrature1D.hpp>
#include <ROL_QuantileQuadrangle.hpp>
#include <ROL_QuantileRadius.hpp>
#include <ROL_RaisedCosine.hpp>
#include <ROL_RandVarFunctional.hpp>
#include <ROL_RandVarFunctionalFactory.hpp>
#include <ROL_RandVarFunctionalInfo.hpp>
#include <ROL_RegretMeasureFactory.hpp>
#include <ROL_RegretMeasureInfo.hpp>
#include <ROL_RiskBoundConstraint.hpp>
#include <ROL_RiskLessConstraint.hpp>
#include <ROL_RiskLessObjective.hpp>
#include <ROL_RiskMeasureFactory.hpp>
#include <ROL_RiskMeasureInfo.hpp>
#include <ROL_RiskNeutralObjective.hpp>
#include <ROL_RiskVector.hpp>
#include <ROL_SampleGenerator.hpp>
#include <ROL_ScalarController.hpp>
#include <ROL_SecondOrderCVaR.hpp>
#include <ROL_Smale.hpp>
#include <ROL_SmoothedPOE.hpp>
#include <ROL_SmoothedWorstCaseQuadrangle.hpp>
#include <ROL_SpectralRisk.hpp>
#include <ROL_StdBoundConstraint.hpp>
#include <ROL_StdVector.hpp>
#include <ROL_StochasticConstraint.hpp>
#include <ROL_StochasticObjective.hpp>
#include <ROL_Triangle.hpp>
#include <ROL_TruncatedExponential.hpp>
#include <ROL_TruncatedGaussian.hpp>
#include <ROL_TruncatedMeanQuadrangle.hpp>
#include <ROL_Types.hpp>
#include <ROL_Uniform.hpp>
#include <ROL_UpdateType.hpp>
#include <ROL_Vector.hpp>
#include <ROL_VectorController.hpp>
#include <Teuchos_ENull.hpp>
#include <Teuchos_FilteredIterator.hpp>
#include <Teuchos_ParameterEntry.hpp>
#include <Teuchos_ParameterEntryValidator.hpp>
#include <Teuchos_ParameterList.hpp>
#include <Teuchos_ParameterListModifier.hpp>
#include <Teuchos_PtrDecl.hpp>
#include <Teuchos_RCPDecl.hpp>
#include <Teuchos_RCPNode.hpp>
#include <Teuchos_StringIndexedOrderedValueObjectContainer.hpp>
#include <Teuchos_any.hpp>
#include <cwchar>
#include <deque>
#include <ios>
#include <iterator>
#include <locale>
#include <memory>
#include <ostream>
#include <sstream> // __str__
#include <streambuf>
#include <string>
#include <vector>

#include <functional>
#include <pybind11/pybind11.h>
#include <string>
#include <Teuchos_RCP.hpp>


#ifndef BINDER_PYBIND11_TYPE_CASTER
	#define BINDER_PYBIND11_TYPE_CASTER
	PYBIND11_DECLARE_HOLDER_TYPE(T, Teuchos::RCP<T>)
	PYBIND11_DECLARE_HOLDER_TYPE(T, T*)
	PYBIND11_MAKE_OPAQUE(Teuchos::RCP<void>)
#endif

// ROL::SampleGenerator file:ROL_SampleGenerator.hpp line:54
struct PyCallBack_ROL_SampleGenerator_double_t : public ROL::SampleGenerator<double> {
	using ROL::SampleGenerator<double>::SampleGenerator;

	void update(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SampleGenerator<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SampleGenerator::update(a0);
	}
	int start() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SampleGenerator<double> *>(this), "start");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<int>::value) {
				static pybind11::detail::override_caster_t<int> caster;
				return pybind11::detail::cast_ref<int>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<int>(std::move(o));
		}
		return SampleGenerator::start();
	}
	double computeError(class std::vector<double, class std::allocator<double> > & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SampleGenerator<double> *>(this), "computeError");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SampleGenerator::computeError(a0);
	}
	void refine() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SampleGenerator<double> *>(this), "refine");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SampleGenerator::refine();
	}
	void setSamples(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SampleGenerator<double> *>(this), "setSamples");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SampleGenerator::setSamples(a0);
	}
	int numGlobalSamples() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SampleGenerator<double> *>(this), "numGlobalSamples");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<int>::value) {
				static pybind11::detail::override_caster_t<int> caster;
				return pybind11::detail::cast_ref<int>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<int>(std::move(o));
		}
		return SampleGenerator::numGlobalSamples();
	}
	int numMySamples() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SampleGenerator<double> *>(this), "numMySamples");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<int>::value) {
				static pybind11::detail::override_caster_t<int> caster;
				return pybind11::detail::cast_ref<int>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<int>(std::move(o));
		}
		return SampleGenerator::numMySamples();
	}
	using _binder_ret_0 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_0 getMyPoint(const int a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SampleGenerator<double> *>(this), "getMyPoint");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_0>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_0> caster;
				return pybind11::detail::cast_ref<_binder_ret_0>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_0>(std::move(o));
		}
		return SampleGenerator::getMyPoint(a0);
	}
	double getMyWeight(const int a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SampleGenerator<double> *>(this), "getMyWeight");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SampleGenerator::getMyWeight(a0);
	}
};

// ROL::MeanValueObjective file:ROL_MeanValueObjective.hpp line:53
struct PyCallBack_ROL_MeanValueObjective_double_t : public ROL::MeanValueObjective<double> {
	using ROL::MeanValueObjective<double>::MeanValueObjective;

	void update(const class ROL::Vector<double> & a0, enum ROL::UpdateType a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanValueObjective::update(a0, a1, a2);
	}
	void update(const class ROL::Vector<double> & a0, bool a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanValueObjective::update(a0, a1, a2);
	}
	double value(const class ROL::Vector<double> & a0, double & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "value");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MeanValueObjective::value(a0, a1);
	}
	void gradient(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "gradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanValueObjective::gradient(a0, a1, a2);
	}
	void hessVec(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "hessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanValueObjective::hessVec(a0, a1, a2, a3);
	}
	void precond(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "precond");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanValueObjective::precond(a0, a1, a2, a3);
	}
	double dirDeriv(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "dirDeriv");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Objective::dirDeriv(a0, a1, a2);
	}
	void invHessVec(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "invHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::invHessVec(a0, a1, a2, a3);
	}
	void prox(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "prox");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::prox(a0, a1, a2, a3);
	}
	using _binder_ret_0 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_0 checkHessSym(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool a3, std::ostream & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "checkHessSym");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_0>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_0> caster;
				return pybind11::detail::cast_ref<_binder_ret_0>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_0>(std::move(o));
		}
		return Objective::checkHessSym(a0, a1, a2, a3, a4);
	}
	using _binder_ret_1 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_1 checkHessSym(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const bool a4, std::ostream & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "checkHessSym");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_1>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_1> caster;
				return pybind11::detail::cast_ref<_binder_ret_1>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_1>(std::move(o));
		}
		return Objective::checkHessSym(a0, a1, a2, a3, a4, a5);
	}
	void setParameter(const class std::vector<double, class std::allocator<double> > & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanValueObjective<double> *>(this), "setParameter");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::setParameter(a0);
	}
};

// ROL::RiskNeutralObjective file:ROL_RiskNeutralObjective.hpp line:56
struct PyCallBack_ROL_RiskNeutralObjective_double_t : public ROL::RiskNeutralObjective<double> {
	using ROL::RiskNeutralObjective<double>::RiskNeutralObjective;

	void update(const class ROL::Vector<double> & a0, enum ROL::UpdateType a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskNeutralObjective::update(a0, a1, a2);
	}
	void update(const class ROL::Vector<double> & a0, bool a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskNeutralObjective::update(a0, a1, a2);
	}
	double value(const class ROL::Vector<double> & a0, double & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "value");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RiskNeutralObjective::value(a0, a1);
	}
	void gradient(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "gradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskNeutralObjective::gradient(a0, a1, a2);
	}
	void hessVec(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "hessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskNeutralObjective::hessVec(a0, a1, a2, a3);
	}
	void precond(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "precond");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskNeutralObjective::precond(a0, a1, a2, a3);
	}
	double dirDeriv(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "dirDeriv");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Objective::dirDeriv(a0, a1, a2);
	}
	void invHessVec(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "invHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::invHessVec(a0, a1, a2, a3);
	}
	void prox(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "prox");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::prox(a0, a1, a2, a3);
	}
	using _binder_ret_0 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_0 checkHessSym(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool a3, std::ostream & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "checkHessSym");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_0>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_0> caster;
				return pybind11::detail::cast_ref<_binder_ret_0>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_0>(std::move(o));
		}
		return Objective::checkHessSym(a0, a1, a2, a3, a4);
	}
	using _binder_ret_1 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_1 checkHessSym(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const bool a4, std::ostream & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "checkHessSym");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_1>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_1> caster;
				return pybind11::detail::cast_ref<_binder_ret_1>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_1>(std::move(o));
		}
		return Objective::checkHessSym(a0, a1, a2, a3, a4, a5);
	}
	void setParameter(const class std::vector<double, class std::allocator<double> > & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskNeutralObjective<double> *>(this), "setParameter");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::setParameter(a0);
	}
};

// ROL::RandVarFunctional file:ROL_RandVarFunctional.hpp line:81
struct PyCallBack_ROL_RandVarFunctional_double_t : public ROL::RandVarFunctional<double> {
	using ROL::RandVarFunctional<double>::RandVarFunctional;

	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RandVarFunctional<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
};

// ROL::StdVector file:ROL_StdVector.hpp line:61
struct PyCallBack_ROL_StdVector_double_double_t : public ROL::StdVector<double,double> {
	using ROL::StdVector<double,double>::StdVector;

	void set(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "set");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdVector::set(a0);
	}
	void plus(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "plus");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdVector::plus(a0);
	}
	void axpy(const double a0, const class ROL::Vector<double> & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "axpy");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdVector::axpy(a0, a1);
	}
	void scale(const double a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "scale");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdVector::scale(a0);
	}
	double dot(const class ROL::Vector<double> & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "dot");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return StdVector::dot(a0);
	}
	double norm() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "norm");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return StdVector::norm();
	}
	class Teuchos::RCP<class ROL::Vector<double> > clone() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "clone");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<class Teuchos::RCP<class ROL::Vector<double> >>::value) {
				static pybind11::detail::override_caster_t<class Teuchos::RCP<class ROL::Vector<double> >> caster;
				return pybind11::detail::cast_ref<class Teuchos::RCP<class ROL::Vector<double> >>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<class Teuchos::RCP<class ROL::Vector<double> >>(std::move(o));
		}
		return StdVector::clone();
	}
	class Teuchos::RCP<class ROL::Vector<double> > basis(const int a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "basis");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<class Teuchos::RCP<class ROL::Vector<double> >>::value) {
				static pybind11::detail::override_caster_t<class Teuchos::RCP<class ROL::Vector<double> >> caster;
				return pybind11::detail::cast_ref<class Teuchos::RCP<class ROL::Vector<double> >>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<class Teuchos::RCP<class ROL::Vector<double> >>(std::move(o));
		}
		return StdVector::basis(a0);
	}
	int dimension() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "dimension");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<int>::value) {
				static pybind11::detail::override_caster_t<int> caster;
				return pybind11::detail::cast_ref<int>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<int>(std::move(o));
		}
		return StdVector::dimension();
	}
	void applyUnary(const class ROL::Elementwise::UnaryFunction<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "applyUnary");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdVector::applyUnary(a0);
	}
	void applyBinary(const class ROL::Elementwise::BinaryFunction<double> & a0, const class ROL::Vector<double> & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "applyBinary");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdVector::applyBinary(a0, a1);
	}
	double reduce(const class ROL::Elementwise::ReductionOp<double> & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "reduce");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return StdVector::reduce(a0);
	}
	void setScalar(const double a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "setScalar");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdVector::setScalar(a0);
	}
	void randomize(const double a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "randomize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdVector::randomize(a0, a1);
	}
	void print(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "print");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdVector::print(a0);
	}
	void zero() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "zero");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Vector::zero();
	}
	const class ROL::Vector<double> & dual() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "dual");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<const class ROL::Vector<double> &>::value) {
				static pybind11::detail::override_caster_t<const class ROL::Vector<double> &> caster;
				return pybind11::detail::cast_ref<const class ROL::Vector<double> &>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<const class ROL::Vector<double> &>(std::move(o));
		}
		return Vector::dual();
	}
	double apply(const class ROL::Vector<double> & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "apply");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Vector::apply(a0);
	}
	using _binder_ret_0 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_0 checkVector(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const bool a2, std::ostream & a3) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdVector<double,double> *>(this), "checkVector");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_0>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_0> caster;
				return pybind11::detail::cast_ref<_binder_ret_0>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_0>(std::move(o));
		}
		return Vector::checkVector(a0, a1, a2, a3);
	}
};

// ROL::RiskVector file:ROL_RiskVector.hpp line:54
struct PyCallBack_ROL_RiskVector_double_t : public ROL::RiskVector<double> {
	using ROL::RiskVector<double>::RiskVector;

	void set(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "set");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskVector::set(a0);
	}
	void plus(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "plus");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskVector::plus(a0);
	}
	void scale(const double a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "scale");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskVector::scale(a0);
	}
	void axpy(const double a0, const class ROL::Vector<double> & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "axpy");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskVector::axpy(a0, a1);
	}
	double dot(const class ROL::Vector<double> & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "dot");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RiskVector::dot(a0);
	}
	double norm() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "norm");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RiskVector::norm();
	}
	class Teuchos::RCP<class ROL::Vector<double> > clone() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "clone");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<class Teuchos::RCP<class ROL::Vector<double> >>::value) {
				static pybind11::detail::override_caster_t<class Teuchos::RCP<class ROL::Vector<double> >> caster;
				return pybind11::detail::cast_ref<class Teuchos::RCP<class ROL::Vector<double> >>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<class Teuchos::RCP<class ROL::Vector<double> >>(std::move(o));
		}
		return RiskVector::clone();
	}
	const class ROL::Vector<double> & dual() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "dual");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<const class ROL::Vector<double> &>::value) {
				static pybind11::detail::override_caster_t<const class ROL::Vector<double> &> caster;
				return pybind11::detail::cast_ref<const class ROL::Vector<double> &>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<const class ROL::Vector<double> &>(std::move(o));
		}
		return RiskVector::dual();
	}
	class Teuchos::RCP<class ROL::Vector<double> > basis(const int a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "basis");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<class Teuchos::RCP<class ROL::Vector<double> >>::value) {
				static pybind11::detail::override_caster_t<class Teuchos::RCP<class ROL::Vector<double> >> caster;
				return pybind11::detail::cast_ref<class Teuchos::RCP<class ROL::Vector<double> >>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<class Teuchos::RCP<class ROL::Vector<double> >>(std::move(o));
		}
		return RiskVector::basis(a0);
	}
	void applyUnary(const class ROL::Elementwise::UnaryFunction<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "applyUnary");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskVector::applyUnary(a0);
	}
	void applyBinary(const class ROL::Elementwise::BinaryFunction<double> & a0, const class ROL::Vector<double> & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "applyBinary");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskVector::applyBinary(a0, a1);
	}
	double reduce(const class ROL::Elementwise::ReductionOp<double> & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "reduce");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RiskVector::reduce(a0);
	}
	void setScalar(const double a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "setScalar");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskVector::setScalar(a0);
	}
	void randomize(const double a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "randomize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskVector::randomize(a0, a1);
	}
	int dimension() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "dimension");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<int>::value) {
				static pybind11::detail::override_caster_t<int> caster;
				return pybind11::detail::cast_ref<int>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<int>(std::move(o));
		}
		return RiskVector::dimension();
	}
	void zero() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "zero");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Vector::zero();
	}
	double apply(const class ROL::Vector<double> & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "apply");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Vector::apply(a0);
	}
	void print(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "print");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Vector::print(a0);
	}
	using _binder_ret_0 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_0 checkVector(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const bool a2, std::ostream & a3) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskVector<double> *>(this), "checkVector");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_0>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_0> caster;
				return pybind11::detail::cast_ref<_binder_ret_0>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_0>(std::move(o));
		}
		return Vector::checkVector(a0, a1, a2, a3);
	}
};

// ROL::PositiveFunction file:ROL_PositiveFunction.hpp line:50
struct PyCallBack_ROL_PositiveFunction_double_t : public ROL::PositiveFunction<double> {
	using ROL::PositiveFunction<double>::PositiveFunction;

	double evaluate(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::PositiveFunction<double> *>(this), "evaluate");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		pybind11::pybind11_fail("Tried to call pure virtual function \"PositiveFunction::evaluate\"");
	}
};

// ROL::Distribution file:ROL_Distribution.hpp line:55
struct PyCallBack_ROL_Distribution_double_t : public ROL::Distribution<double> {
	using ROL::Distribution<double>::Distribution;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Distribution<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Distribution::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Distribution<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Distribution::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Distribution<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Distribution::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Distribution<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Distribution::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Distribution<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Distribution::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Distribution<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Distribution::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Distribution<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Distribution::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Distribution<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Distribution::test(a0);
	}
};

// ROL::Dirac file:ROL_Dirac.hpp line:53
struct PyCallBack_ROL_Dirac_double_t : public ROL::Dirac<double> {
	using ROL::Dirac<double>::Dirac;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Dirac<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Dirac::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Dirac<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Dirac::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Dirac<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Dirac::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Dirac<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Dirac::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Dirac<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Dirac::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Dirac<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Dirac::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Dirac<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Dirac::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Dirac<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Dirac::test(a0);
	}
};

// ROL::Gaussian file:ROL_Gaussian.hpp line:53
struct PyCallBack_ROL_Gaussian_double_t : public ROL::Gaussian<double> {
	using ROL::Gaussian<double>::Gaussian;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gaussian<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gaussian::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gaussian<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gaussian::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gaussian<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gaussian::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gaussian<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gaussian::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gaussian<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gaussian::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gaussian<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gaussian::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gaussian<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gaussian::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gaussian<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Gaussian::test(a0);
	}
};

// ROL::TruncatedGaussian file:ROL_TruncatedGaussian.hpp line:55
struct PyCallBack_ROL_TruncatedGaussian_double_t : public ROL::TruncatedGaussian<double> {
	using ROL::TruncatedGaussian<double>::TruncatedGaussian;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedGaussian<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedGaussian::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedGaussian<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedGaussian::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedGaussian<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedGaussian::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedGaussian<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedGaussian::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedGaussian<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedGaussian::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedGaussian<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedGaussian::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedGaussian<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedGaussian::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedGaussian<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return TruncatedGaussian::test(a0);
	}
};

// ROL::Uniform file:ROL_Uniform.hpp line:53
struct PyCallBack_ROL_Uniform_double_t : public ROL::Uniform<double> {
	using ROL::Uniform<double>::Uniform;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Uniform<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Uniform::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Uniform<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Uniform::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Uniform<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Uniform::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Uniform<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Uniform::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Uniform<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Uniform::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Uniform<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Uniform::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Uniform<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Uniform::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Uniform<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Uniform::test(a0);
	}
};

// ROL::Logistic file:ROL_Logistic.hpp line:53
struct PyCallBack_ROL_Logistic_double_t : public ROL::Logistic<double> {
	using ROL::Logistic<double>::Logistic;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Logistic<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Logistic::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Logistic<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Logistic::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Logistic<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Logistic::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Logistic<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Logistic::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Logistic<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Logistic::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Logistic<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Logistic::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Logistic<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Logistic::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Logistic<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Logistic::test(a0);
	}
};

// ROL::Triangle file:ROL_Triangle.hpp line:53
struct PyCallBack_ROL_Triangle_double_t : public ROL::Triangle<double> {
	using ROL::Triangle<double>::Triangle;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Triangle<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Triangle::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Triangle<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Triangle::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Triangle<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Triangle::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Triangle<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Triangle::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Triangle<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Triangle::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Triangle<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Triangle::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Triangle<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Triangle::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Triangle<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Triangle::test(a0);
	}
};

// ROL::Parabolic file:ROL_Parabolic.hpp line:53
struct PyCallBack_ROL_Parabolic_double_t : public ROL::Parabolic<double> {
	using ROL::Parabolic<double>::Parabolic;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Parabolic<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Parabolic::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Parabolic<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Parabolic::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Parabolic<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Parabolic::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Parabolic<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Parabolic::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Parabolic<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Parabolic::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Parabolic<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Parabolic::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Parabolic<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Parabolic::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Parabolic<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Parabolic::test(a0);
	}
};

// ROL::RaisedCosine file:ROL_RaisedCosine.hpp line:53
struct PyCallBack_ROL_RaisedCosine_double_t : public ROL::RaisedCosine<double> {
	using ROL::RaisedCosine<double>::RaisedCosine;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RaisedCosine<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RaisedCosine::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RaisedCosine<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RaisedCosine::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RaisedCosine<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RaisedCosine::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RaisedCosine<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RaisedCosine::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RaisedCosine<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RaisedCosine::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RaisedCosine<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RaisedCosine::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RaisedCosine<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RaisedCosine::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RaisedCosine<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RaisedCosine::test(a0);
	}
};

// ROL::Laplace file:ROL_Laplace.hpp line:53
struct PyCallBack_ROL_Laplace_double_t : public ROL::Laplace<double> {
	using ROL::Laplace<double>::Laplace;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Laplace<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Laplace::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Laplace<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Laplace::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Laplace<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Laplace::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Laplace<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Laplace::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Laplace<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Laplace::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Laplace<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Laplace::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Laplace<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Laplace::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Laplace<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Laplace::test(a0);
	}
};

// ROL::Cauchy file:ROL_Cauchy.hpp line:53
struct PyCallBack_ROL_Cauchy_double_t : public ROL::Cauchy<double> {
	using ROL::Cauchy<double>::Cauchy;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Cauchy<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Cauchy::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Cauchy<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Cauchy::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Cauchy<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Cauchy::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Cauchy<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Cauchy::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Cauchy<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Cauchy::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Cauchy<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Cauchy::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Cauchy<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Cauchy::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Cauchy<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Cauchy::test(a0);
	}
};

// ROL::Smale file:ROL_Smale.hpp line:53
struct PyCallBack_ROL_Smale_double_t : public ROL::Smale<double> {
	using ROL::Smale<double>::Smale;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Smale<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Smale::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Smale<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Smale::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Smale<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Smale::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Smale<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Smale::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Smale<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Smale::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Smale<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Smale::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Smale<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Smale::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Smale<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Smale::test(a0);
	}
};

// ROL::Arcsine file:ROL_Arcsine.hpp line:53
struct PyCallBack_ROL_Arcsine_double_t : public ROL::Arcsine<double> {
	using ROL::Arcsine<double>::Arcsine;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Arcsine<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Arcsine::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Arcsine<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Arcsine::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Arcsine<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Arcsine::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Arcsine<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Arcsine::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Arcsine<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Arcsine::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Arcsine<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Arcsine::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Arcsine<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Arcsine::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Arcsine<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Arcsine::test(a0);
	}
};

// ROL::Kumaraswamy file:ROL_Kumaraswamy.hpp line:55
struct PyCallBack_ROL_Kumaraswamy_double_t : public ROL::Kumaraswamy<double> {
	using ROL::Kumaraswamy<double>::Kumaraswamy;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Kumaraswamy<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Kumaraswamy::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Kumaraswamy<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Kumaraswamy::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Kumaraswamy<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Kumaraswamy::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Kumaraswamy<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Kumaraswamy::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Kumaraswamy<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Kumaraswamy::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Kumaraswamy<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Kumaraswamy::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Kumaraswamy<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Kumaraswamy::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Kumaraswamy<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Kumaraswamy::test(a0);
	}
};

// ROL::Exponential file:ROL_Exponential.hpp line:53
struct PyCallBack_ROL_Exponential_double_t : public ROL::Exponential<double> {
	using ROL::Exponential<double>::Exponential;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Exponential<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Exponential::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Exponential<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Exponential::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Exponential<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Exponential::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Exponential<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Exponential::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Exponential<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Exponential::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Exponential<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Exponential::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Exponential<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Exponential::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Exponential<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Exponential::test(a0);
	}
};

// ROL::TruncatedExponential file:ROL_TruncatedExponential.hpp line:53
struct PyCallBack_ROL_TruncatedExponential_double_t : public ROL::TruncatedExponential<double> {
	using ROL::TruncatedExponential<double>::TruncatedExponential;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedExponential<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedExponential::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedExponential<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedExponential::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedExponential<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedExponential::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedExponential<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedExponential::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedExponential<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedExponential::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedExponential<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedExponential::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedExponential<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedExponential::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedExponential<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return TruncatedExponential::test(a0);
	}
};

// ROL::Gamma file:ROL_Gamma.hpp line:55
struct PyCallBack_ROL_Gamma_double_t : public ROL::Gamma<double> {
	using ROL::Gamma<double>::Gamma;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gamma<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gamma::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gamma<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gamma::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gamma<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gamma::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gamma<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gamma::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gamma<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gamma::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gamma<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gamma::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gamma<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Gamma::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Gamma<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Gamma::test(a0);
	}
};

// ROL::Beta file:ROL_Beta.hpp line:55
struct PyCallBack_ROL_Beta_double_t : public ROL::Beta<double> {
	using ROL::Beta<double>::Beta;

	double evaluatePDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Beta<double> *>(this), "evaluatePDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Beta::evaluatePDF(a0);
	}
	double evaluateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Beta<double> *>(this), "evaluateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Beta::evaluateCDF(a0);
	}
	double integrateCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Beta<double> *>(this), "integrateCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Beta::integrateCDF(a0);
	}
	double invertCDF(const double a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Beta<double> *>(this), "invertCDF");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Beta::invertCDF(a0);
	}
	double moment(const unsigned long a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Beta<double> *>(this), "moment");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Beta::moment(a0);
	}
	double lowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Beta<double> *>(this), "lowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Beta::lowerBound();
	}
	double upperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Beta<double> *>(this), "upperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Beta::upperBound();
	}
	void test(std::ostream & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Beta<double> *>(this), "test");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Beta::test(a0);
	}
};

// ROL::PlusFunction file:ROL_PlusFunction.hpp line:55
struct PyCallBack_ROL_PlusFunction_double_t : public ROL::PlusFunction<double> {
	using ROL::PlusFunction<double>::PlusFunction;

	double evaluate(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::PlusFunction<double> *>(this), "evaluate");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return PlusFunction::evaluate(a0, a1);
	}
};

// ROL::CVaR file:ROL_CVaR.hpp line:78
struct PyCallBack_ROL_CVaR_double_t : public ROL::CVaR<double> {
	using ROL::CVaR<double>::CVaR;

	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CVaR::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CVaR::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CVaR::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return CVaR::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CVaR::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CVaR::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CVaR<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::MixedCVaR file:ROL_MixedCVaR.hpp line:88
struct PyCallBack_ROL_MixedCVaR_double_t : public ROL::MixedCVaR<double> {
	using ROL::MixedCVaR<double>::MixedCVaR;

	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MixedCVaR::initialize(a0);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MixedCVaR::computeStatistic(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MixedCVaR::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MixedCVaR::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MixedCVaR::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MixedCVaR::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MixedCVaR::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MixedCVaR::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MixedCVaR<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
};

// ROL::SpectralRisk file:ROL_SpectralRisk.hpp line:89
struct PyCallBack_ROL_SpectralRisk_double_t : public ROL::SpectralRisk<double> {
	using ROL::SpectralRisk<double>::SpectralRisk;

	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::setHessVecStorage(a0, a1);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::setSample(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::initialize(a0);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SpectralRisk::computeStatistic(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SpectralRisk::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SpectralRisk<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
};

// ROL::GaussLegendreQuadrature file:ROL_GaussLegendreQuadrature.hpp line:52
struct PyCallBack_ROL_GaussLegendreQuadrature_double_t : public ROL::GaussLegendreQuadrature<double> {
	using ROL::GaussLegendreQuadrature<double>::GaussLegendreQuadrature;

};

// ROL::Fejer2Quadrature file:ROL_Fejer2Quadrature.hpp line:52
struct PyCallBack_ROL_Fejer2Quadrature_double_t : public ROL::Fejer2Quadrature<double> {
	using ROL::Fejer2Quadrature<double>::Fejer2Quadrature;

};

// ROL::SecondOrderCVaR file:ROL_SecondOrderCVaR.hpp line:85
struct PyCallBack_ROL_SecondOrderCVaR_double_t : public ROL::SecondOrderCVaR<double> {
	using ROL::SecondOrderCVaR<double>::SecondOrderCVaR;

	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::setHessVecStorage(a0, a1);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::setSample(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::initialize(a0);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SpectralRisk::computeStatistic(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SpectralRisk::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SecondOrderCVaR<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
};

// ROL::GaussChebyshev1Quadrature file:ROL_GaussChebyshev1Quadrature.hpp line:52
struct PyCallBack_ROL_GaussChebyshev1Quadrature_double_t : public ROL::GaussChebyshev1Quadrature<double> {
	using ROL::GaussChebyshev1Quadrature<double>::GaussChebyshev1Quadrature;

};

// ROL::GaussChebyshev2Quadrature file:ROL_GaussChebyshev2Quadrature.hpp line:53
struct PyCallBack_ROL_GaussChebyshev2Quadrature_double_t : public ROL::GaussChebyshev2Quadrature<double> {
	using ROL::GaussChebyshev2Quadrature<double>::GaussChebyshev2Quadrature;

};

// ROL::GaussChebyshev3Quadrature file:ROL_GaussChebyshev3Quadrature.hpp line:53
struct PyCallBack_ROL_GaussChebyshev3Quadrature_double_t : public ROL::GaussChebyshev3Quadrature<double> {
	using ROL::GaussChebyshev3Quadrature<double>::GaussChebyshev3Quadrature;

};

// ROL::ChebyshevSpectral file:ROL_ChebyshevSpectral.hpp line:97
struct PyCallBack_ROL_ChebyshevSpectral_double_t : public ROL::ChebyshevSpectral<double> {
	using ROL::ChebyshevSpectral<double>::ChebyshevSpectral;

	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::setHessVecStorage(a0, a1);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::setSample(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::initialize(a0);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SpectralRisk::computeStatistic(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SpectralRisk::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ChebyshevSpectral<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SpectralRisk::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
};

// ROL::QuantileRadius file:ROL_QuantileRadius.hpp line:55
struct PyCallBack_ROL_QuantileRadius_double_t : public ROL::QuantileRadius<double> {
	using ROL::QuantileRadius<double>::QuantileRadius;

	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return QuantileRadius::initialize(a0);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return QuantileRadius::computeStatistic(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return QuantileRadius::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return QuantileRadius::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return QuantileRadius::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return QuantileRadius::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return QuantileRadius::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return QuantileRadius::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileRadius<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
};

// ROL::HMCR file:ROL_HMCR.hpp line:75
struct PyCallBack_ROL_HMCR_double_t : public ROL::HMCR<double> {
	using ROL::HMCR<double>::HMCR;

	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return HMCR::initialize(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return HMCR::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return HMCR::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return HMCR::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return HMCR::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return HMCR::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return HMCR::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::HMCR<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::EntropicRisk file:ROL_EntropicRisk.hpp line:66
struct PyCallBack_ROL_EntropicRisk_double_t : public ROL::EntropicRisk<double> {
	using ROL::EntropicRisk<double>::EntropicRisk;

	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return EntropicRisk::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return EntropicRisk::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return EntropicRisk::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return EntropicRisk::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return EntropicRisk::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return EntropicRisk::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::EntropicRisk<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::CoherentEntropicRisk file:ROL_CoherentEntropicRisk.hpp line:67
struct PyCallBack_ROL_CoherentEntropicRisk_double_t : public ROL::CoherentEntropicRisk<double> {
	using ROL::CoherentEntropicRisk<double>::CoherentEntropicRisk;

	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CoherentEntropicRisk::initialize(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CoherentEntropicRisk::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return CoherentEntropicRisk::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CoherentEntropicRisk::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CoherentEntropicRisk::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CoherentEntropicRisk::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return CoherentEntropicRisk::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::CoherentEntropicRisk<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::AbsoluteValue file:ROL_AbsoluteValue.hpp line:62
struct PyCallBack_ROL_AbsoluteValue_double_t : public ROL::AbsoluteValue<double> {
	using ROL::AbsoluteValue<double>::AbsoluteValue;

	double evaluate(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::AbsoluteValue<double> *>(this), "evaluate");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return AbsoluteValue::evaluate(a0, a1);
	}
};

// ROL::MeanDeviationFromTarget file:ROL_MeanDeviationFromTarget.hpp line:76
struct PyCallBack_ROL_MeanDeviationFromTarget_double_t : public ROL::MeanDeviationFromTarget<double> {
	using ROL::MeanDeviationFromTarget<double>::MeanDeviationFromTarget;

	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviationFromTarget::initialize(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviationFromTarget::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MeanDeviationFromTarget::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviationFromTarget::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviationFromTarget::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviationFromTarget::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviationFromTarget::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviationFromTarget<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::MeanDeviation file:ROL_MeanDeviation.hpp line:80
struct PyCallBack_ROL_MeanDeviation_double_t : public ROL::MeanDeviation<double> {
	using ROL::MeanDeviation<double>::MeanDeviation;

	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviation::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviation::setHessVecStorage(a0, a1);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviation::initialize(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviation::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviation::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviation::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MeanDeviation::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviation::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanDeviation::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanDeviation<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::MeanVarianceFromTarget file:ROL_MeanVarianceFromTarget.hpp line:77
struct PyCallBack_ROL_MeanVarianceFromTarget_double_t : public ROL::MeanVarianceFromTarget<double> {
	using ROL::MeanVarianceFromTarget<double>::MeanVarianceFromTarget;

	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanVarianceFromTarget::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanVarianceFromTarget::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanVarianceFromTarget::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceFromTarget<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
};

// ROL::MeanVariance file:ROL_MeanVariance.hpp line:78
struct PyCallBack_ROL_MeanVariance_double_t : public ROL::MeanVariance<double> {
	using ROL::MeanVariance<double>::MeanVariance;

	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanVariance::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanVariance::setHessVecStorage(a0, a1);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanVariance::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MeanVariance::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanVariance::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanVariance::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanVariance::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanVariance::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVariance<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::MeanSemiDeviation file:ROL_MeanSemiDeviation.hpp line:72
struct PyCallBack_ROL_MeanSemiDeviation_double_t : public ROL::MeanSemiDeviation<double> {
	using ROL::MeanSemiDeviation<double>::MeanSemiDeviation;

	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviation::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviation::setHessVecStorage(a0, a1);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviation::initialize(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviation::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MeanSemiDeviation::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviation::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviation::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviation::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviation::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviation<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::MeanSemiDeviationFromTarget file:ROL_MeanSemiDeviationFromTarget.hpp line:72
struct PyCallBack_ROL_MeanSemiDeviationFromTarget_double_t : public ROL::MeanSemiDeviationFromTarget<double> {
	using ROL::MeanSemiDeviationFromTarget<double>::MeanSemiDeviationFromTarget;

	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviationFromTarget::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MeanSemiDeviationFromTarget::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviationFromTarget::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviationFromTarget::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviationFromTarget::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MeanSemiDeviationFromTarget::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanSemiDeviationFromTarget<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::ExpectationQuad file:ROL_ExpectationQuad.hpp line:87
struct PyCallBack_ROL_ExpectationQuad_double_t : public ROL::ExpectationQuad<double> {
	using ROL::ExpectationQuad<double>::ExpectationQuad;

	double regret(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuad<double> *>(this), "regret");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		pybind11::pybind11_fail("Tried to call pure virtual function \"ExpectationQuad::regret\"");
	}
	double error(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuad<double> *>(this), "error");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return ExpectationQuad::error(a0, a1);
	}
	void check() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuad<double> *>(this), "check");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuad::check();
	}
};

// ROL::ExpectationQuadRisk file:ROL_ExpectationQuadRisk.hpp line:89
struct PyCallBack_ROL_ExpectationQuadRisk_double_t : public ROL::ExpectationQuadRisk<double> {
	using ROL::ExpectationQuadRisk<double>::ExpectationQuadRisk;

	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadRisk::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadRisk::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadRisk::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return ExpectationQuadRisk::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadRisk::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadRisk::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRisk<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::LogQuantileQuadrangle file:ROL_LogQuantileQuadrangle.hpp line:76
struct PyCallBack_ROL_LogQuantileQuadrangle_double_t : public ROL::LogQuantileQuadrangle<double> {
	using ROL::LogQuantileQuadrangle<double>::LogQuantileQuadrangle;

	double error(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::LogQuantileQuadrangle<double> *>(this), "error");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return LogQuantileQuadrangle::error(a0, a1);
	}
	double regret(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::LogQuantileQuadrangle<double> *>(this), "regret");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return LogQuantileQuadrangle::regret(a0, a1);
	}
	void check() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::LogQuantileQuadrangle<double> *>(this), "check");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return LogQuantileQuadrangle::check();
	}
};

// ROL::SmoothedWorstCaseQuadrangle file:ROL_SmoothedWorstCaseQuadrangle.hpp line:89
struct PyCallBack_ROL_SmoothedWorstCaseQuadrangle_double_t : public ROL::SmoothedWorstCaseQuadrangle<double> {
	using ROL::SmoothedWorstCaseQuadrangle<double>::SmoothedWorstCaseQuadrangle;

	double error(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedWorstCaseQuadrangle<double> *>(this), "error");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SmoothedWorstCaseQuadrangle::error(a0, a1);
	}
	double regret(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedWorstCaseQuadrangle<double> *>(this), "regret");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SmoothedWorstCaseQuadrangle::regret(a0, a1);
	}
	void check() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedWorstCaseQuadrangle<double> *>(this), "check");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SmoothedWorstCaseQuadrangle::check();
	}
};

// ROL::TruncatedMeanQuadrangle file:ROL_TruncatedMeanQuadrangle.hpp line:52
struct PyCallBack_ROL_TruncatedMeanQuadrangle_double_t : public ROL::TruncatedMeanQuadrangle<double> {
	using ROL::TruncatedMeanQuadrangle<double>::TruncatedMeanQuadrangle;

	double error(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedMeanQuadrangle<double> *>(this), "error");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedMeanQuadrangle::error(a0, a1);
	}
	double regret(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedMeanQuadrangle<double> *>(this), "regret");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return TruncatedMeanQuadrangle::regret(a0, a1);
	}
	void check() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::TruncatedMeanQuadrangle<double> *>(this), "check");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return TruncatedMeanQuadrangle::check();
	}
};

// ROL::MoreauYosidaCVaR file:ROL_MoreauYosidaCVaR.hpp line:105
struct PyCallBack_ROL_MoreauYosidaCVaR_double_t : public ROL::MoreauYosidaCVaR<double> {
	using ROL::MoreauYosidaCVaR<double>::MoreauYosidaCVaR;

	double error(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MoreauYosidaCVaR<double> *>(this), "error");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MoreauYosidaCVaR::error(a0, a1);
	}
	double regret(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MoreauYosidaCVaR<double> *>(this), "regret");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MoreauYosidaCVaR::regret(a0, a1);
	}
	void check() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MoreauYosidaCVaR<double> *>(this), "check");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return MoreauYosidaCVaR::check();
	}
};

// ROL::GenMoreauYosidaCVaR file:ROL_GenMoreauYosidaCVaR.hpp line:105
struct PyCallBack_ROL_GenMoreauYosidaCVaR_double_t : public ROL::GenMoreauYosidaCVaR<double> {
	using ROL::GenMoreauYosidaCVaR<double>::GenMoreauYosidaCVaR;

	double error(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::GenMoreauYosidaCVaR<double> *>(this), "error");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return GenMoreauYosidaCVaR::error(a0, a1);
	}
	double regret(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::GenMoreauYosidaCVaR<double> *>(this), "regret");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return GenMoreauYosidaCVaR::regret(a0, a1);
	}
	void check() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::GenMoreauYosidaCVaR<double> *>(this), "check");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return GenMoreauYosidaCVaR::check();
	}
};

// ROL::LogExponentialQuadrangle file:ROL_LogExponentialQuadrangle.hpp line:81
struct PyCallBack_ROL_LogExponentialQuadrangle_double_t : public ROL::LogExponentialQuadrangle<double> {
	using ROL::LogExponentialQuadrangle<double>::LogExponentialQuadrangle;

	double error(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::LogExponentialQuadrangle<double> *>(this), "error");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return LogExponentialQuadrangle::error(a0, a1);
	}
	double regret(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::LogExponentialQuadrangle<double> *>(this), "regret");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return LogExponentialQuadrangle::regret(a0, a1);
	}
	void check() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::LogExponentialQuadrangle<double> *>(this), "check");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuad::check();
	}
};

// ROL::MeanVarianceQuadrangle file:ROL_MeanVarianceQuadrangle.hpp line:78
struct PyCallBack_ROL_MeanVarianceQuadrangle_double_t : public ROL::MeanVarianceQuadrangle<double> {
	using ROL::MeanVarianceQuadrangle<double>::MeanVarianceQuadrangle;

	double error(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceQuadrangle<double> *>(this), "error");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MeanVarianceQuadrangle::error(a0, a1);
	}
	double regret(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceQuadrangle<double> *>(this), "regret");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return MeanVarianceQuadrangle::regret(a0, a1);
	}
	void check() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::MeanVarianceQuadrangle<double> *>(this), "check");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuad::check();
	}
};

// ROL::FDivergence file:ROL_FDivergence.hpp line:87
struct PyCallBack_ROL_FDivergence_double_t : public ROL::FDivergence<double> {
	using ROL::FDivergence<double>::FDivergence;

	double Fprimal(double a0, int a1) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "Fprimal");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		pybind11::pybind11_fail("Tried to call pure virtual function \"FDivergence::Fprimal\"");
	}
	double Fdual(double a0, int a1) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "Fdual");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		pybind11::pybind11_fail("Tried to call pure virtual function \"FDivergence::Fdual\"");
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::initialize(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return FDivergence::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::FDivergence<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::Chi2Divergence file:ROL_Chi2Divergence.hpp line:73
struct PyCallBack_ROL_Chi2Divergence_double_t : public ROL::Chi2Divergence<double> {
	using ROL::Chi2Divergence<double>::Chi2Divergence;

	double Fprimal(double a0, int a1) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "Fprimal");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Chi2Divergence::Fprimal(a0, a1);
	}
	double Fdual(double a0, int a1) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "Fdual");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Chi2Divergence::Fdual(a0, a1);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::initialize(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return FDivergence::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return FDivergence::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Chi2Divergence<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::KLDivergence file:ROL_KLDivergence.hpp line:81
struct PyCallBack_ROL_KLDivergence_double_t : public ROL::KLDivergence<double> {
	using ROL::KLDivergence<double>::KLDivergence;

	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return KLDivergence::initialize(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return KLDivergence::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return KLDivergence::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return KLDivergence::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return KLDivergence::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return KLDivergence::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return KLDivergence::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::KLDivergence<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::ExpectationQuadDeviation file:ROL_ExpectationQuadDeviation.hpp line:89
struct PyCallBack_ROL_ExpectationQuadDeviation_double_t : public ROL::ExpectationQuadDeviation<double> {
	using ROL::ExpectationQuadDeviation<double>::ExpectationQuadDeviation;

	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadDeviation::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadDeviation::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadDeviation::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return ExpectationQuadDeviation::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadDeviation::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadDeviation::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadDeviation<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::QuantileQuadrangle file:ROL_QuantileQuadrangle.hpp line:97
struct PyCallBack_ROL_QuantileQuadrangle_double_t : public ROL::QuantileQuadrangle<double> {
	using ROL::QuantileQuadrangle<double>::QuantileQuadrangle;

	double error(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileQuadrangle<double> *>(this), "error");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return QuantileQuadrangle::error(a0, a1);
	}
	double regret(double a0, int a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileQuadrangle<double> *>(this), "regret");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return QuantileQuadrangle::regret(a0, a1);
	}
	void check() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::QuantileQuadrangle<double> *>(this), "check");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return QuantileQuadrangle::check();
	}
};

// ROL::ExpectationQuadError file:ROL_ExpectationQuadError.hpp line:89
struct PyCallBack_ROL_ExpectationQuadError_double_t : public ROL::ExpectationQuadError<double> {
	using ROL::ExpectationQuadError<double>::ExpectationQuadError;

	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadError::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadError::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadError::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return ExpectationQuadError::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadError::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadError::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadError<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::ExpectationQuadRegret file:ROL_ExpectationQuadRegret.hpp line:89
struct PyCallBack_ROL_ExpectationQuadRegret_double_t : public ROL::ExpectationQuadRegret<double> {
	using ROL::ExpectationQuadRegret<double>::ExpectationQuadRegret;

	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadRegret::updateValue(a0, a1, a2, a3);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadRegret::updateGradient(a0, a1, a2, a3);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadRegret::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return ExpectationQuadRegret::getValue(a0, a1, a2);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadRegret::getGradient(a0, a1, a2, a3, a4);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ExpectationQuadRegret::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ExpectationQuadRegret<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::BPOE file:ROL_BPOE.hpp line:66
struct PyCallBack_ROL_BPOE_double_t : public ROL::BPOE<double> {
	using ROL::BPOE<double>::BPOE;

	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return BPOE::initialize(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return BPOE::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return BPOE::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return BPOE::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return BPOE::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return BPOE::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return BPOE::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::BPOE<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::SmoothedPOE file:ROL_SmoothedPOE.hpp line:66
struct PyCallBack_ROL_SmoothedPOE_double_t : public ROL::SmoothedPOE<double> {
	using ROL::SmoothedPOE<double>::SmoothedPOE;

	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SmoothedPOE::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return SmoothedPOE::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SmoothedPOE::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SmoothedPOE::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SmoothedPOE::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return SmoothedPOE::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::initialize(a0);
	}
	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setSample(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::SmoothedPOE<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::ConvexCombinationRiskMeasure file:ROL_ConvexCombinationRiskMeasure.hpp line:68
struct PyCallBack_ROL_ConvexCombinationRiskMeasure_double_t : public ROL::ConvexCombinationRiskMeasure<double> {
	using ROL::ConvexCombinationRiskMeasure<double>::ConvexCombinationRiskMeasure;

	void setSample(const class std::vector<double, class std::allocator<double> > & a0, const double a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "setSample");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConvexCombinationRiskMeasure::setSample(a0, a1);
	}
	void resetStorage(bool a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConvexCombinationRiskMeasure::resetStorage(a0);
	}
	void resetStorage(enum ROL::UpdateType a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "resetStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConvexCombinationRiskMeasure::resetStorage(a0);
	}
	void initialize(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "initialize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConvexCombinationRiskMeasure::initialize(a0);
	}
	void updateValue(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "updateValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConvexCombinationRiskMeasure::updateValue(a0, a1, a2, a3);
	}
	double getValue(const class ROL::Vector<double> & a0, const class std::vector<double, class std::allocator<double> > & a1, class ROL::SampleGenerator<double> & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "getValue");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return ConvexCombinationRiskMeasure::getValue(a0, a1, a2);
	}
	void updateGradient(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "updateGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConvexCombinationRiskMeasure::updateGradient(a0, a1, a2, a3);
	}
	void getGradient(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, class ROL::SampleGenerator<double> & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "getGradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConvexCombinationRiskMeasure::getGradient(a0, a1, a2, a3, a4);
	}
	void updateHessVec(class ROL::Objective<double> & a0, const class ROL::Vector<double> & a1, const class std::vector<double, class std::allocator<double> > & a2, const class ROL::Vector<double> & a3, const class std::vector<double, class std::allocator<double> > & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "updateHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConvexCombinationRiskMeasure::updateHessVec(a0, a1, a2, a3, a4, a5);
	}
	void getHessVec(class ROL::Vector<double> & a0, class std::vector<double, class std::allocator<double> > & a1, const class ROL::Vector<double> & a2, const class std::vector<double, class std::allocator<double> > & a3, const class ROL::Vector<double> & a4, const class std::vector<double, class std::allocator<double> > & a5, class ROL::SampleGenerator<double> & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "getHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConvexCombinationRiskMeasure::getHessVec(a0, a1, a2, a3, a4, a5, a6);
	}
	void setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "setStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setStorage(a0, a1);
	}
	void setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > & a0, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "setHessVecStorage");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RandVarFunctional::setHessVecStorage(a0, a1);
	}
	double computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > & a0) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConvexCombinationRiskMeasure<double> *>(this), "computeStatistic");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RandVarFunctional::computeStatistic(a0);
	}
};

// ROL::StochasticObjective file:ROL_StochasticObjective.hpp line:56
struct PyCallBack_ROL_StochasticObjective_double_t : public ROL::StochasticObjective<double> {
	using ROL::StochasticObjective<double>::StochasticObjective;

	void update(const class ROL::Vector<double> & a0, enum ROL::UpdateType a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticObjective::update(a0, a1, a2);
	}
	void update(const class ROL::Vector<double> & a0, bool a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticObjective::update(a0, a1, a2);
	}
	double value(const class ROL::Vector<double> & a0, double & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "value");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return StochasticObjective::value(a0, a1);
	}
	void gradient(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "gradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticObjective::gradient(a0, a1, a2);
	}
	void hessVec(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "hessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticObjective::hessVec(a0, a1, a2, a3);
	}
	void precond(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "precond");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticObjective::precond(a0, a1, a2, a3);
	}
	double dirDeriv(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "dirDeriv");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Objective::dirDeriv(a0, a1, a2);
	}
	void invHessVec(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "invHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::invHessVec(a0, a1, a2, a3);
	}
	void prox(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "prox");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::prox(a0, a1, a2, a3);
	}
	using _binder_ret_0 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_0 checkHessSym(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool a3, std::ostream & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "checkHessSym");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_0>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_0> caster;
				return pybind11::detail::cast_ref<_binder_ret_0>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_0>(std::move(o));
		}
		return Objective::checkHessSym(a0, a1, a2, a3, a4);
	}
	using _binder_ret_1 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_1 checkHessSym(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const bool a4, std::ostream & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "checkHessSym");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_1>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_1> caster;
				return pybind11::detail::cast_ref<_binder_ret_1>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_1>(std::move(o));
		}
		return Objective::checkHessSym(a0, a1, a2, a3, a4, a5);
	}
	void setParameter(const class std::vector<double, class std::allocator<double> > & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticObjective<double> *>(this), "setParameter");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::setParameter(a0);
	}
};

// ROL::ConstraintFromObjective file:ROL_ConstraintFromObjective.hpp line:64
struct PyCallBack_ROL_ConstraintFromObjective_double_t : public ROL::ConstraintFromObjective<double> {
	using ROL::ConstraintFromObjective<double>::ConstraintFromObjective;

	void setParameter(const class std::vector<double, class std::allocator<double> > & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "setParameter");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConstraintFromObjective::setParameter(a0);
	}
	void update(const class ROL::Vector<double> & a0, enum ROL::UpdateType a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConstraintFromObjective::update(a0, a1, a2);
	}
	void update(const class ROL::Vector<double> & a0, bool a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConstraintFromObjective::update(a0, a1, a2);
	}
	void value(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "value");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConstraintFromObjective::value(a0, a1, a2);
	}
	void applyJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "applyJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConstraintFromObjective::applyJacobian(a0, a1, a2, a3);
	}
	void applyAdjointJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "applyAdjointJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConstraintFromObjective::applyAdjointJacobian(a0, a1, a2, a3);
	}
	void applyAdjointHessian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, double & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "applyAdjointHessian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return ConstraintFromObjective::applyAdjointHessian(a0, a1, a2, a3, a4);
	}
	void applyAdjointJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, double & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "applyAdjointJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Constraint::applyAdjointJacobian(a0, a1, a2, a3, a4);
	}
	using _binder_ret_0 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_0 solveAugmentedSystem(class ROL::Vector<double> & a0, class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "solveAugmentedSystem");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_0>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_0> caster;
				return pybind11::detail::cast_ref<_binder_ret_0>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_0>(std::move(o));
		}
		return Constraint::solveAugmentedSystem(a0, a1, a2, a3, a4, a5);
	}
	void applyPreconditioner(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, double & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "applyPreconditioner");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Constraint::applyPreconditioner(a0, a1, a2, a3, a4);
	}
	double checkAdjointConsistencyJacobian(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool a3, std::ostream & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "checkAdjointConsistencyJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Constraint::checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4);
	}
	double checkAdjointConsistencyJacobian(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4, const bool a5, std::ostream & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::ConstraintFromObjective<double> *>(this), "checkAdjointConsistencyJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Constraint::checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4, a5, a6);
	}
};

// ROL::StochasticConstraint file:ROL_StochasticConstraint.hpp line:54
struct PyCallBack_ROL_StochasticConstraint_double_t : public ROL::StochasticConstraint<double> {
	using ROL::StochasticConstraint<double>::StochasticConstraint;

	void update(const class ROL::Vector<double> & a0, enum ROL::UpdateType a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticConstraint::update(a0, a1, a2);
	}
	void update(const class ROL::Vector<double> & a0, bool a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticConstraint::update(a0, a1, a2);
	}
	void value(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "value");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticConstraint::value(a0, a1, a2);
	}
	void applyJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "applyJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticConstraint::applyJacobian(a0, a1, a2, a3);
	}
	void applyAdjointJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "applyAdjointJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticConstraint::applyAdjointJacobian(a0, a1, a2, a3);
	}
	void applyAdjointHessian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, double & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "applyAdjointHessian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StochasticConstraint::applyAdjointHessian(a0, a1, a2, a3, a4);
	}
	void applyAdjointJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, double & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "applyAdjointJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Constraint::applyAdjointJacobian(a0, a1, a2, a3, a4);
	}
	using _binder_ret_0 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_0 solveAugmentedSystem(class ROL::Vector<double> & a0, class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "solveAugmentedSystem");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_0>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_0> caster;
				return pybind11::detail::cast_ref<_binder_ret_0>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_0>(std::move(o));
		}
		return Constraint::solveAugmentedSystem(a0, a1, a2, a3, a4, a5);
	}
	void applyPreconditioner(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, double & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "applyPreconditioner");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Constraint::applyPreconditioner(a0, a1, a2, a3, a4);
	}
	double checkAdjointConsistencyJacobian(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool a3, std::ostream & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "checkAdjointConsistencyJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Constraint::checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4);
	}
	double checkAdjointConsistencyJacobian(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4, const bool a5, std::ostream & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "checkAdjointConsistencyJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Constraint::checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4, a5, a6);
	}
	void setParameter(const class std::vector<double, class std::allocator<double> > & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StochasticConstraint<double> *>(this), "setParameter");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Constraint::setParameter(a0);
	}
};

// ROL::StdBoundConstraint file:ROL_StdBoundConstraint.hpp line:58
struct PyCallBack_ROL_StdBoundConstraint_double_t : public ROL::StdBoundConstraint<double> {
	using ROL::StdBoundConstraint<double>::StdBoundConstraint;

	void project(class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "project");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdBoundConstraint::project(a0);
	}
	void projectInterior(class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "projectInterior");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdBoundConstraint::projectInterior(a0);
	}
	void pruneUpperActive(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "pruneUpperActive");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdBoundConstraint::pruneUpperActive(a0, a1, a2);
	}
	void pruneUpperActive(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double a3, double a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "pruneUpperActive");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdBoundConstraint::pruneUpperActive(a0, a1, a2, a3, a4);
	}
	void pruneLowerActive(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double a3, double a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "pruneLowerActive");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdBoundConstraint::pruneLowerActive(a0, a1, a2, a3, a4);
	}
	void pruneLowerActive(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "pruneLowerActive");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdBoundConstraint::pruneLowerActive(a0, a1, a2);
	}
	bool isFeasible(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "isFeasible");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<bool>::value) {
				static pybind11::detail::override_caster_t<bool> caster;
				return pybind11::detail::cast_ref<bool>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<bool>(std::move(o));
		}
		return StdBoundConstraint::isFeasible(a0);
	}
	void applyInverseScalingFunction(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "applyInverseScalingFunction");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdBoundConstraint::applyInverseScalingFunction(a0, a1, a2, a3);
	}
	void applyScalingFunctionJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "applyScalingFunctionJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return StdBoundConstraint::applyScalingFunctionJacobian(a0, a1, a2, a3);
	}
	const class Teuchos::RCP<const class ROL::Vector<double> > getLowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "getLowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<const class Teuchos::RCP<const class ROL::Vector<double> >>::value) {
				static pybind11::detail::override_caster_t<const class Teuchos::RCP<const class ROL::Vector<double> >> caster;
				return pybind11::detail::cast_ref<const class Teuchos::RCP<const class ROL::Vector<double> >>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<const class Teuchos::RCP<const class ROL::Vector<double> >>(std::move(o));
		}
		return BoundConstraint::getLowerBound();
	}
	const class Teuchos::RCP<const class ROL::Vector<double> > getUpperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::StdBoundConstraint<double> *>(this), "getUpperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<const class Teuchos::RCP<const class ROL::Vector<double> >>::value) {
				static pybind11::detail::override_caster_t<const class Teuchos::RCP<const class ROL::Vector<double> >> caster;
				return pybind11::detail::cast_ref<const class Teuchos::RCP<const class ROL::Vector<double> >>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<const class Teuchos::RCP<const class ROL::Vector<double> >>(std::move(o));
		}
		return BoundConstraint::getUpperBound();
	}
};

// ROL::RiskBoundConstraint file:ROL_RiskBoundConstraint.hpp line:54
struct PyCallBack_ROL_RiskBoundConstraint_double_t : public ROL::RiskBoundConstraint<double> {
	using ROL::RiskBoundConstraint<double>::RiskBoundConstraint;

	void project(class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "project");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskBoundConstraint::project(a0);
	}
	void projectInterior(class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "projectInterior");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskBoundConstraint::projectInterior(a0);
	}
	void pruneUpperActive(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "pruneUpperActive");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskBoundConstraint::pruneUpperActive(a0, a1, a2);
	}
	void pruneUpperActive(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double a3, double a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "pruneUpperActive");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskBoundConstraint::pruneUpperActive(a0, a1, a2, a3, a4);
	}
	void pruneLowerActive(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "pruneLowerActive");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskBoundConstraint::pruneLowerActive(a0, a1, a2);
	}
	void pruneLowerActive(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double a3, double a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "pruneLowerActive");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskBoundConstraint::pruneLowerActive(a0, a1, a2, a3, a4);
	}
	const class Teuchos::RCP<const class ROL::Vector<double> > getLowerBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "getLowerBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<const class Teuchos::RCP<const class ROL::Vector<double> >>::value) {
				static pybind11::detail::override_caster_t<const class Teuchos::RCP<const class ROL::Vector<double> >> caster;
				return pybind11::detail::cast_ref<const class Teuchos::RCP<const class ROL::Vector<double> >>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<const class Teuchos::RCP<const class ROL::Vector<double> >>(std::move(o));
		}
		return RiskBoundConstraint::getLowerBound();
	}
	const class Teuchos::RCP<const class ROL::Vector<double> > getUpperBound() const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "getUpperBound");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<const class Teuchos::RCP<const class ROL::Vector<double> >>::value) {
				static pybind11::detail::override_caster_t<const class Teuchos::RCP<const class ROL::Vector<double> >> caster;
				return pybind11::detail::cast_ref<const class Teuchos::RCP<const class ROL::Vector<double> >>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<const class Teuchos::RCP<const class ROL::Vector<double> >>(std::move(o));
		}
		return RiskBoundConstraint::getUpperBound();
	}
	bool isFeasible(const class ROL::Vector<double> & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "isFeasible");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<bool>::value) {
				static pybind11::detail::override_caster_t<bool> caster;
				return pybind11::detail::cast_ref<bool>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<bool>(std::move(o));
		}
		return RiskBoundConstraint::isFeasible(a0);
	}
	void applyInverseScalingFunction(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "applyInverseScalingFunction");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskBoundConstraint::applyInverseScalingFunction(a0, a1, a2, a3);
	}
	void applyScalingFunctionJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3) const override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskBoundConstraint<double> *>(this), "applyScalingFunctionJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskBoundConstraint::applyScalingFunctionJacobian(a0, a1, a2, a3);
	}
};

// ROL::RiskLessObjective file:ROL_RiskLessObjective.hpp line:53
struct PyCallBack_ROL_RiskLessObjective_double_t : public ROL::RiskLessObjective<double> {
	using ROL::RiskLessObjective<double>::RiskLessObjective;

	void update(const class ROL::Vector<double> & a0, enum ROL::UpdateType a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessObjective::update(a0, a1, a2);
	}
	void update(const class ROL::Vector<double> & a0, bool a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessObjective::update(a0, a1, a2);
	}
	double value(const class ROL::Vector<double> & a0, double & a1) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "value");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return RiskLessObjective::value(a0, a1);
	}
	void gradient(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "gradient");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessObjective::gradient(a0, a1, a2);
	}
	void hessVec(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "hessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessObjective::hessVec(a0, a1, a2, a3);
	}
	void precond(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "precond");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessObjective::precond(a0, a1, a2, a3);
	}
	void setParameter(const class std::vector<double, class std::allocator<double> > & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "setParameter");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessObjective::setParameter(a0);
	}
	double dirDeriv(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "dirDeriv");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Objective::dirDeriv(a0, a1, a2);
	}
	void invHessVec(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "invHessVec");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::invHessVec(a0, a1, a2, a3);
	}
	void prox(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "prox");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Objective::prox(a0, a1, a2, a3);
	}
	using _binder_ret_0 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_0 checkHessSym(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool a3, std::ostream & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "checkHessSym");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_0>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_0> caster;
				return pybind11::detail::cast_ref<_binder_ret_0>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_0>(std::move(o));
		}
		return Objective::checkHessSym(a0, a1, a2, a3, a4);
	}
	using _binder_ret_1 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_1 checkHessSym(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const bool a4, std::ostream & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessObjective<double> *>(this), "checkHessSym");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_1>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_1> caster;
				return pybind11::detail::cast_ref<_binder_ret_1>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_1>(std::move(o));
		}
		return Objective::checkHessSym(a0, a1, a2, a3, a4, a5);
	}
};

// ROL::RiskLessConstraint file:ROL_RiskLessConstraint.hpp line:53
struct PyCallBack_ROL_RiskLessConstraint_double_t : public ROL::RiskLessConstraint<double> {
	using ROL::RiskLessConstraint<double>::RiskLessConstraint;

	void update(const class ROL::Vector<double> & a0, enum ROL::UpdateType a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessConstraint::update(a0, a1, a2);
	}
	void update(const class ROL::Vector<double> & a0, bool a1, int a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "update");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessConstraint::update(a0, a1, a2);
	}
	void value(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, double & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "value");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessConstraint::value(a0, a1, a2);
	}
	void applyJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "applyJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessConstraint::applyJacobian(a0, a1, a2, a3);
	}
	void applyAdjointJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double & a3) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "applyAdjointJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessConstraint::applyAdjointJacobian(a0, a1, a2, a3);
	}
	void applyAdjointHessian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, double & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "applyAdjointHessian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessConstraint::applyAdjointHessian(a0, a1, a2, a3, a4);
	}
	void setParameter(const class std::vector<double, class std::allocator<double> > & a0) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "setParameter");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return RiskLessConstraint::setParameter(a0);
	}
	void applyAdjointJacobian(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, double & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "applyAdjointJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Constraint::applyAdjointJacobian(a0, a1, a2, a3, a4);
	}
	using _binder_ret_0 = class std::vector<double, class std::allocator<double> >;
	_binder_ret_0 solveAugmentedSystem(class ROL::Vector<double> & a0, class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4, double & a5) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "solveAugmentedSystem");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5);
			if (pybind11::detail::cast_is_temporary_value_reference<_binder_ret_0>::value) {
				static pybind11::detail::override_caster_t<_binder_ret_0> caster;
				return pybind11::detail::cast_ref<_binder_ret_0>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<_binder_ret_0>(std::move(o));
		}
		return Constraint::solveAugmentedSystem(a0, a1, a2, a3, a4, a5);
	}
	void applyPreconditioner(class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, double & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "applyPreconditioner");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Constraint::applyPreconditioner(a0, a1, a2, a3, a4);
	}
	double checkAdjointConsistencyJacobian(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool a3, std::ostream & a4) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "checkAdjointConsistencyJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Constraint::checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4);
	}
	double checkAdjointConsistencyJacobian(const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4, const bool a5, std::ostream & a6) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::RiskLessConstraint<double> *>(this), "checkAdjointConsistencyJacobian");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2, a3, a4, a5, a6);
			if (pybind11::detail::cast_is_temporary_value_reference<double>::value) {
				static pybind11::detail::override_caster_t<double> caster;
				return pybind11::detail::cast_ref<double>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<double>(std::move(o));
		}
		return Constraint::checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4, a5, a6);
	}
};

// ROL::Problem file: line:49
struct PyCallBack_ROL_Problem_double_t : public ROL::Problem<double> {
	using ROL::Problem<double>::Problem;

	void finalize(bool a0, bool a1, std::ostream & a2) override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Problem<double> *>(this), "finalize");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>(a0, a1, a2);
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Problem::finalize(a0, a1, a2);
	}
	void edit() override {
		pybind11::gil_scoped_acquire gil;
		pybind11::function overload = pybind11::get_overload(static_cast<const ROL::Problem<double> *>(this), "edit");
		if (overload) {
			auto o = overload.operator()<pybind11::return_value_policy::reference>();
			if (pybind11::detail::cast_is_temporary_value_reference<void>::value) {
				static pybind11::detail::override_caster_t<void> caster;
				return pybind11::detail::cast_ref<void>(std::move(o), caster);
			}
			else return pybind11::detail::cast_safe<void>(std::move(o));
		}
		return Problem::edit();
	}
};

void bind_pyrol_53(std::function< pybind11::module &(std::string const &namespace_) > &M)
{
	{ // ROL::SampleGenerator file:ROL_SampleGenerator.hpp line:54
		pybind11::class_<ROL::SampleGenerator<double>, Teuchos::RCP<ROL::SampleGenerator<double>>, PyCallBack_ROL_SampleGenerator_double_t> cl(M("ROL"), "SampleGenerator_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::BatchManager<double> > &>(), pybind11::arg("bman") );

		cl.def( pybind11::init( [](PyCallBack_ROL_SampleGenerator_double_t const &o){ return new PyCallBack_ROL_SampleGenerator_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::SampleGenerator<double> const &o){ return new ROL::SampleGenerator<double>(o); } ) );
		cl.def("update", (void (ROL::SampleGenerator<double>::*)(const class ROL::Vector<double> &)) &ROL::SampleGenerator<double>::update, "C++: ROL::SampleGenerator<double>::update(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("start", (int (ROL::SampleGenerator<double>::*)()) &ROL::SampleGenerator<double>::start, "C++: ROL::SampleGenerator<double>::start() --> int");
		cl.def("computeError", (double (ROL::SampleGenerator<double>::*)(class std::vector<double, class std::allocator<double> > &)) &ROL::SampleGenerator<double>::computeError, "C++: ROL::SampleGenerator<double>::computeError(class std::vector<double, class std::allocator<double> > &) --> double", pybind11::arg("vals"));
		cl.def("refine", (void (ROL::SampleGenerator<double>::*)()) &ROL::SampleGenerator<double>::refine, "C++: ROL::SampleGenerator<double>::refine() --> void");
		cl.def("setSamples", [](ROL::SampleGenerator<double> &o) -> void { return o.setSamples(); }, "");
		cl.def("setSamples", (void (ROL::SampleGenerator<double>::*)(bool)) &ROL::SampleGenerator<double>::setSamples, "C++: ROL::SampleGenerator<double>::setSamples(bool) --> void", pybind11::arg("inConstructor"));
		cl.def("numGlobalSamples", (int (ROL::SampleGenerator<double>::*)() const) &ROL::SampleGenerator<double>::numGlobalSamples, "C++: ROL::SampleGenerator<double>::numGlobalSamples() const --> int");
		cl.def("numMySamples", (int (ROL::SampleGenerator<double>::*)() const) &ROL::SampleGenerator<double>::numMySamples, "C++: ROL::SampleGenerator<double>::numMySamples() const --> int");
		cl.def("getMyPoint", (class std::vector<double, class std::allocator<double> > (ROL::SampleGenerator<double>::*)(const int) const) &ROL::SampleGenerator<double>::getMyPoint, "C++: ROL::SampleGenerator<double>::getMyPoint(const int) const --> class std::vector<double, class std::allocator<double> >", pybind11::arg("i"));
		cl.def("getMyWeight", (double (ROL::SampleGenerator<double>::*)(const int) const) &ROL::SampleGenerator<double>::getMyWeight, "C++: ROL::SampleGenerator<double>::getMyWeight(const int) const --> double", pybind11::arg("i"));
		cl.def("batchID", (int (ROL::SampleGenerator<double>::*)() const) &ROL::SampleGenerator<double>::batchID, "C++: ROL::SampleGenerator<double>::batchID() const --> int");
		cl.def("numBatches", (int (ROL::SampleGenerator<double>::*)() const) &ROL::SampleGenerator<double>::numBatches, "C++: ROL::SampleGenerator<double>::numBatches() const --> int");
		cl.def("sumAll", (void (ROL::SampleGenerator<double>::*)(double *, double *, int) const) &ROL::SampleGenerator<double>::sumAll, "C++: ROL::SampleGenerator<double>::sumAll(double *, double *, int) const --> void", pybind11::arg("input"), pybind11::arg("output"), pybind11::arg("dim"));
		cl.def("sumAll", (void (ROL::SampleGenerator<double>::*)(class ROL::Vector<double> &, class ROL::Vector<double> &) const) &ROL::SampleGenerator<double>::sumAll, "C++: ROL::SampleGenerator<double>::sumAll(class ROL::Vector<double> &, class ROL::Vector<double> &) const --> void", pybind11::arg("input"), pybind11::arg("output"));
		cl.def("broadcast", (void (ROL::SampleGenerator<double>::*)(double *, int, int) const) &ROL::SampleGenerator<double>::broadcast, "C++: ROL::SampleGenerator<double>::broadcast(double *, int, int) const --> void", pybind11::arg("input"), pybind11::arg("cnt"), pybind11::arg("root"));
		cl.def("barrier", (void (ROL::SampleGenerator<double>::*)() const) &ROL::SampleGenerator<double>::barrier, "C++: ROL::SampleGenerator<double>::barrier() const --> void");
		cl.def("getBatchManager", (const class Teuchos::RCP<class ROL::BatchManager<double> > (ROL::SampleGenerator<double>::*)() const) &ROL::SampleGenerator<double>::getBatchManager, "C++: ROL::SampleGenerator<double>::getBatchManager() const --> const class Teuchos::RCP<class ROL::BatchManager<double> >");
		cl.def("print", [](ROL::SampleGenerator<double> const &o) -> void { return o.print(); }, "");
		cl.def("print", [](ROL::SampleGenerator<double> const &o, const std::string & a0) -> void { return o.print(a0); }, "", pybind11::arg("filename"));
		cl.def("print", (void (ROL::SampleGenerator<double>::*)(const std::string &, const int) const) &ROL::SampleGenerator<double>::print, "C++: ROL::SampleGenerator<double>::print(const std::string &, const int) const --> void", pybind11::arg("filename"), pybind11::arg("prec"));
		cl.def("assign", (class ROL::SampleGenerator<double> & (ROL::SampleGenerator<double>::*)(const class ROL::SampleGenerator<double> &)) &ROL::SampleGenerator<double>::operator=, "C++: ROL::SampleGenerator<double>::operator=(const class ROL::SampleGenerator<double> &) --> class ROL::SampleGenerator<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::MeanValueObjective file:ROL_MeanValueObjective.hpp line:53
		pybind11::class_<ROL::MeanValueObjective<double>, Teuchos::RCP<ROL::MeanValueObjective<double>>, PyCallBack_ROL_MeanValueObjective_double_t, ROL::Objective<double>> cl(M("ROL"), "MeanValueObjective_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &>(), pybind11::arg("obj"), pybind11::arg("sampler") );

		cl.def( pybind11::init( [](PyCallBack_ROL_MeanValueObjective_double_t const &o){ return new PyCallBack_ROL_MeanValueObjective_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::MeanValueObjective<double> const &o){ return new ROL::MeanValueObjective<double>(o); } ) );
		cl.def("update", [](ROL::MeanValueObjective<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::MeanValueObjective<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::MeanValueObjective<double>::update, "C++: ROL::MeanValueObjective<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::MeanValueObjective<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::MeanValueObjective<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::MeanValueObjective<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::MeanValueObjective<double>::update, "C++: ROL::MeanValueObjective<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (double (ROL::MeanValueObjective<double>::*)(const class ROL::Vector<double> &, double &)) &ROL::MeanValueObjective<double>::value, "C++: ROL::MeanValueObjective<double>::value(const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("gradient", (void (ROL::MeanValueObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::MeanValueObjective<double>::gradient, "C++: ROL::MeanValueObjective<double>::gradient(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("hessVec", (void (ROL::MeanValueObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::MeanValueObjective<double>::hessVec, "C++: ROL::MeanValueObjective<double>::hessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("precond", (void (ROL::MeanValueObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::MeanValueObjective<double>::precond, "C++: ROL::MeanValueObjective<double>::precond(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::Objective<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::Objective<double>::update, "Update objective function. \n\n      This function updates the objective function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is the type of update requested.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Objective<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::Objective<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::Objective<double>::update, "Update objective function. \n\n      This function updates the objective function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is true if the iterate has changed.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Objective<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (double (ROL::Objective<double>::*)(const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::value, "C++: ROL::Objective<double>::value(const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("gradient", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::gradient, "Compute gradient.\n\n      This function returns the objective function gradient.\n      \n\n   is the gradient.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\n      The default implementation is a finite-difference approximation based on the function value.\n      This requires the definition of a basis \n\n for the optimization vectors x and\n      the definition of a basis \n\n for the dual optimization vectors (gradient vectors g).\n      The bases must be related through the Riesz map, i.e., \n\n,\n      and this must be reflected in the implementation of the ROL::Vector::dual() method.\n\nC++: ROL::Objective<double>::gradient(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("dirDeriv", (double (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::dirDeriv, "Compute directional derivative.\n\n      This function returns the directional derivative of the objective function in the \n direction.\n      \n\n   is the current iterate.\n      \n\n   is the direction.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::dirDeriv(const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("d"), pybind11::arg("tol"));
		cl.def("hessVec", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::hessVec, "Apply Hessian approximation to vector.\n\n      This function applies the Hessian of the objective function to the vector \n.\n      \n\n  is the the action of the Hessian on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::hessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("invHessVec", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::invHessVec, "Apply inverse Hessian approximation to vector.\n\n      This function applies the inverse Hessian of the objective function to the vector \n.\n      \n\n  is the action of the inverse Hessian on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::invHessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("precond", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::precond, "Apply preconditioner to vector.\n\n      This function applies a preconditioner for the Hessian of the objective function to the vector \n.\n      \n\n  is the action of the Hessian preconditioner on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::precond(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("prox", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double &)) &ROL::Objective<double>::prox, "C++: ROL::Objective<double>::prox(class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("t"), pybind11::arg("tol"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2); }, "", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool & a3) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3); }, "", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"));
		cl.def("checkHessSym", (class std::vector<double, class std::allocator<double> > (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Objective<double>::checkHessSym, "Hessian symmetry check.\n\n      This function checks the symmetry of the Hessian by comparing \n      \n\n\n\n\n\n      \n             is an optimization variable.\n      \n\n             is a direction vector.\n      \n\n             is a direction vector.\n      \n\n is a flag that turns on/off output.\n      \n\n     is the output stream.\n\nC++: ROL::Objective<double>::checkHessSym(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3); }, "", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const bool & a4) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3, a4); }, "", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"));
		cl.def("checkHessSym", (class std::vector<double, class std::allocator<double> > (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Objective<double>::checkHessSym, "Hessian symmetry check.\n\n      This function checks the symmetry of the Hessian by comparing \n      \n\n\n\n\n\n      \n             is an optimization variable.\n      \n\n            is used to create temporary Hessian-times-vector vectors.\n      \n\n             is a direction vector.\n      \n\n             is a direction vector.\n      \n\n is a flag that turns on/off output.\n      \n\n     is the output stream.\n\nC++: ROL::Objective<double>::checkHessSym(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("setParameter", (void (ROL::Objective<double>::*)(const class std::vector<double, class std::allocator<double> > &)) &ROL::Objective<double>::setParameter, "C++: ROL::Objective<double>::setParameter(const class std::vector<double, class std::allocator<double> > &) --> void", pybind11::arg("param"));
		cl.def("assign", (class ROL::Objective<double> & (ROL::Objective<double>::*)(const class ROL::Objective<double> &)) &ROL::Objective<double>::operator=, "C++: ROL::Objective<double>::operator=(const class ROL::Objective<double> &) --> class ROL::Objective<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::RiskNeutralObjective file:ROL_RiskNeutralObjective.hpp line:56
		pybind11::class_<ROL::RiskNeutralObjective<double>, Teuchos::RCP<ROL::RiskNeutralObjective<double>>, PyCallBack_ROL_RiskNeutralObjective_double_t, ROL::Objective<double>> cl(M("ROL"), "RiskNeutralObjective_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3){ return new ROL::RiskNeutralObjective<double>(a0, a1, a2, a3); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3){ return new PyCallBack_ROL_RiskNeutralObjective_double_t(a0, a1, a2, a3); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const bool>(), pybind11::arg("pObj"), pybind11::arg("vsampler"), pybind11::arg("gsampler"), pybind11::arg("hsampler"), pybind11::arg("storage") );

		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2){ return new ROL::RiskNeutralObjective<double>(a0, a1, a2); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2){ return new PyCallBack_ROL_RiskNeutralObjective_double_t(a0, a1, a2); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const bool>(), pybind11::arg("pObj"), pybind11::arg("vsampler"), pybind11::arg("gsampler"), pybind11::arg("storage") );

		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a1){ return new ROL::RiskNeutralObjective<double>(a0, a1); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a1){ return new PyCallBack_ROL_RiskNeutralObjective_double_t(a0, a1); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const bool>(), pybind11::arg("pObj"), pybind11::arg("sampler"), pybind11::arg("storage") );

		cl.def( pybind11::init( [](PyCallBack_ROL_RiskNeutralObjective_double_t const &o){ return new PyCallBack_ROL_RiskNeutralObjective_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::RiskNeutralObjective<double> const &o){ return new ROL::RiskNeutralObjective<double>(o); } ) );
		cl.def("update", [](ROL::RiskNeutralObjective<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::RiskNeutralObjective<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::RiskNeutralObjective<double>::update, "C++: ROL::RiskNeutralObjective<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::RiskNeutralObjective<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::RiskNeutralObjective<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::RiskNeutralObjective<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::RiskNeutralObjective<double>::update, "C++: ROL::RiskNeutralObjective<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (double (ROL::RiskNeutralObjective<double>::*)(const class ROL::Vector<double> &, double &)) &ROL::RiskNeutralObjective<double>::value, "C++: ROL::RiskNeutralObjective<double>::value(const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("gradient", (void (ROL::RiskNeutralObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::RiskNeutralObjective<double>::gradient, "C++: ROL::RiskNeutralObjective<double>::gradient(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("hessVec", (void (ROL::RiskNeutralObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::RiskNeutralObjective<double>::hessVec, "C++: ROL::RiskNeutralObjective<double>::hessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("precond", (void (ROL::RiskNeutralObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::RiskNeutralObjective<double>::precond, "C++: ROL::RiskNeutralObjective<double>::precond(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("assign", (class ROL::RiskNeutralObjective<double> & (ROL::RiskNeutralObjective<double>::*)(const class ROL::RiskNeutralObjective<double> &)) &ROL::RiskNeutralObjective<double>::operator=, "C++: ROL::RiskNeutralObjective<double>::operator=(const class ROL::RiskNeutralObjective<double> &) --> class ROL::RiskNeutralObjective<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::Objective<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::Objective<double>::update, "Update objective function. \n\n      This function updates the objective function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is the type of update requested.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Objective<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::Objective<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::Objective<double>::update, "Update objective function. \n\n      This function updates the objective function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is true if the iterate has changed.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Objective<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (double (ROL::Objective<double>::*)(const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::value, "C++: ROL::Objective<double>::value(const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("gradient", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::gradient, "Compute gradient.\n\n      This function returns the objective function gradient.\n      \n\n   is the gradient.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\n      The default implementation is a finite-difference approximation based on the function value.\n      This requires the definition of a basis \n\n for the optimization vectors x and\n      the definition of a basis \n\n for the dual optimization vectors (gradient vectors g).\n      The bases must be related through the Riesz map, i.e., \n\n,\n      and this must be reflected in the implementation of the ROL::Vector::dual() method.\n\nC++: ROL::Objective<double>::gradient(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("dirDeriv", (double (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::dirDeriv, "Compute directional derivative.\n\n      This function returns the directional derivative of the objective function in the \n direction.\n      \n\n   is the current iterate.\n      \n\n   is the direction.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::dirDeriv(const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("d"), pybind11::arg("tol"));
		cl.def("hessVec", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::hessVec, "Apply Hessian approximation to vector.\n\n      This function applies the Hessian of the objective function to the vector \n.\n      \n\n  is the the action of the Hessian on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::hessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("invHessVec", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::invHessVec, "Apply inverse Hessian approximation to vector.\n\n      This function applies the inverse Hessian of the objective function to the vector \n.\n      \n\n  is the action of the inverse Hessian on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::invHessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("precond", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::precond, "Apply preconditioner to vector.\n\n      This function applies a preconditioner for the Hessian of the objective function to the vector \n.\n      \n\n  is the action of the Hessian preconditioner on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::precond(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("prox", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double &)) &ROL::Objective<double>::prox, "C++: ROL::Objective<double>::prox(class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("t"), pybind11::arg("tol"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2); }, "", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool & a3) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3); }, "", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"));
		cl.def("checkHessSym", (class std::vector<double, class std::allocator<double> > (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Objective<double>::checkHessSym, "Hessian symmetry check.\n\n      This function checks the symmetry of the Hessian by comparing \n      \n\n\n\n\n\n      \n             is an optimization variable.\n      \n\n             is a direction vector.\n      \n\n             is a direction vector.\n      \n\n is a flag that turns on/off output.\n      \n\n     is the output stream.\n\nC++: ROL::Objective<double>::checkHessSym(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3); }, "", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const bool & a4) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3, a4); }, "", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"));
		cl.def("checkHessSym", (class std::vector<double, class std::allocator<double> > (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Objective<double>::checkHessSym, "Hessian symmetry check.\n\n      This function checks the symmetry of the Hessian by comparing \n      \n\n\n\n\n\n      \n             is an optimization variable.\n      \n\n            is used to create temporary Hessian-times-vector vectors.\n      \n\n             is a direction vector.\n      \n\n             is a direction vector.\n      \n\n is a flag that turns on/off output.\n      \n\n     is the output stream.\n\nC++: ROL::Objective<double>::checkHessSym(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("setParameter", (void (ROL::Objective<double>::*)(const class std::vector<double, class std::allocator<double> > &)) &ROL::Objective<double>::setParameter, "C++: ROL::Objective<double>::setParameter(const class std::vector<double, class std::allocator<double> > &) --> void", pybind11::arg("param"));
		cl.def("assign", (class ROL::Objective<double> & (ROL::Objective<double>::*)(const class ROL::Objective<double> &)) &ROL::Objective<double>::operator=, "C++: ROL::Objective<double>::operator=(const class ROL::Objective<double> &) --> class ROL::Objective<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::RandVarFunctional file:ROL_RandVarFunctional.hpp line:81
		pybind11::class_<ROL::RandVarFunctional<double>, Teuchos::RCP<ROL::RandVarFunctional<double>>, PyCallBack_ROL_RandVarFunctional_double_t> cl(M("ROL"), "RandVarFunctional_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::RandVarFunctional<double>(); }, [](){ return new PyCallBack_ROL_RandVarFunctional_double_t(); } ) );
		cl.def( pybind11::init( [](PyCallBack_ROL_RandVarFunctional_double_t const &o){ return new PyCallBack_ROL_RandVarFunctional_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::RandVarFunctional<double> const &o){ return new ROL::RandVarFunctional<double>(o); } ) );
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::StdVector file:ROL_StdVector.hpp line:61
		pybind11::class_<ROL::StdVector<double,double>, Teuchos::RCP<ROL::StdVector<double,double>>, PyCallBack_ROL_StdVector_double_double_t, ROL::Vector<double>> cl(M("ROL"), "StdVector_double_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const class Teuchos::RCP<class std::vector<double, class std::allocator<double> > > &>(), pybind11::arg("std_vec") );

		cl.def( pybind11::init( [](const int & a0){ return new ROL::StdVector<double,double>(a0); }, [](const int & a0){ return new PyCallBack_ROL_StdVector_double_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const int, const double>(), pybind11::arg("dim"), pybind11::arg("val") );

		cl.def( pybind11::init( [](PyCallBack_ROL_StdVector_double_double_t const &o){ return new PyCallBack_ROL_StdVector_double_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::StdVector<double,double> const &o){ return new ROL::StdVector<double,double>(o); } ) );
		cl.def("__getitem__", (double & (ROL::StdVector<double,double>::*)(int)) &ROL::StdVector<double, double>::operator[], "C++: ROL::StdVector<double, double>::operator[](int) --> double &", pybind11::return_value_policy::automatic, pybind11::arg("i"));
		cl.def("set", (void (ROL::StdVector<double,double>::*)(const class ROL::Vector<double> &)) &ROL::StdVector<double, double>::set, "C++: ROL::StdVector<double, double>::set(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("plus", (void (ROL::StdVector<double,double>::*)(const class ROL::Vector<double> &)) &ROL::StdVector<double, double>::plus, "C++: ROL::StdVector<double, double>::plus(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("axpy", (void (ROL::StdVector<double,double>::*)(const double, const class ROL::Vector<double> &)) &ROL::StdVector<double, double>::axpy, "C++: ROL::StdVector<double, double>::axpy(const double, const class ROL::Vector<double> &) --> void", pybind11::arg("alpha"), pybind11::arg("x"));
		cl.def("scale", (void (ROL::StdVector<double,double>::*)(const double)) &ROL::StdVector<double, double>::scale, "C++: ROL::StdVector<double, double>::scale(const double) --> void", pybind11::arg("alpha"));
		cl.def("dot", (double (ROL::StdVector<double,double>::*)(const class ROL::Vector<double> &) const) &ROL::StdVector<double, double>::dot, "C++: ROL::StdVector<double, double>::dot(const class ROL::Vector<double> &) const --> double", pybind11::arg("x"));
		cl.def("norm", (double (ROL::StdVector<double,double>::*)() const) &ROL::StdVector<double, double>::norm, "C++: ROL::StdVector<double, double>::norm() const --> double");
		cl.def("clone", (class Teuchos::RCP<class ROL::Vector<double> > (ROL::StdVector<double,double>::*)() const) &ROL::StdVector<double, double>::clone, "C++: ROL::StdVector<double, double>::clone() const --> class Teuchos::RCP<class ROL::Vector<double> >");
		cl.def("getVector", (class Teuchos::RCP<class std::vector<double, class std::allocator<double> > > (ROL::StdVector<double,double>::*)()) &ROL::StdVector<double, double>::getVector, "C++: ROL::StdVector<double, double>::getVector() --> class Teuchos::RCP<class std::vector<double, class std::allocator<double> > >");
		cl.def("basis", (class Teuchos::RCP<class ROL::Vector<double> > (ROL::StdVector<double,double>::*)(const int) const) &ROL::StdVector<double, double>::basis, "C++: ROL::StdVector<double, double>::basis(const int) const --> class Teuchos::RCP<class ROL::Vector<double> >", pybind11::arg("i"));
		cl.def("dimension", (int (ROL::StdVector<double,double>::*)() const) &ROL::StdVector<double, double>::dimension, "C++: ROL::StdVector<double, double>::dimension() const --> int");
		cl.def("applyUnary", (void (ROL::StdVector<double,double>::*)(const class ROL::Elementwise::UnaryFunction<double> &)) &ROL::StdVector<double, double>::applyUnary, "C++: ROL::StdVector<double, double>::applyUnary(const class ROL::Elementwise::UnaryFunction<double> &) --> void", pybind11::arg("f"));
		cl.def("applyBinary", (void (ROL::StdVector<double,double>::*)(const class ROL::Elementwise::BinaryFunction<double> &, const class ROL::Vector<double> &)) &ROL::StdVector<double, double>::applyBinary, "C++: ROL::StdVector<double, double>::applyBinary(const class ROL::Elementwise::BinaryFunction<double> &, const class ROL::Vector<double> &) --> void", pybind11::arg("f"), pybind11::arg("x"));
		cl.def("reduce", (double (ROL::StdVector<double,double>::*)(const class ROL::Elementwise::ReductionOp<double> &) const) &ROL::StdVector<double, double>::reduce, "C++: ROL::StdVector<double, double>::reduce(const class ROL::Elementwise::ReductionOp<double> &) const --> double", pybind11::arg("r"));
		cl.def("setScalar", (void (ROL::StdVector<double,double>::*)(const double)) &ROL::StdVector<double, double>::setScalar, "C++: ROL::StdVector<double, double>::setScalar(const double) --> void", pybind11::arg("C"));
		cl.def("randomize", [](ROL::StdVector<double,double> &o) -> void { return o.randomize(); }, "");
		cl.def("randomize", [](ROL::StdVector<double,double> &o, const double & a0) -> void { return o.randomize(a0); }, "", pybind11::arg("l"));
		cl.def("randomize", (void (ROL::StdVector<double,double>::*)(const double, const double)) &ROL::StdVector<double, double>::randomize, "C++: ROL::StdVector<double, double>::randomize(const double, const double) --> void", pybind11::arg("l"), pybind11::arg("u"));
		cl.def("print", (void (ROL::StdVector<double,double>::*)(std::ostream &) const) &ROL::StdVector<double, double>::print, "C++: ROL::StdVector<double, double>::print(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::StdVector<double, double> & (ROL::StdVector<double,double>::*)(const class ROL::StdVector<double, double> &)) &ROL::StdVector<double, double>::operator=, "C++: ROL::StdVector<double, double>::operator=(const class ROL::StdVector<double, double> &) --> class ROL::StdVector<double, double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("plus", (void (ROL::Vector<double>::*)(const class ROL::Vector<double> &)) &ROL::Vector<double>::plus, "C++: ROL::Vector<double>::plus(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("scale", (void (ROL::Vector<double>::*)(const double)) &ROL::Vector<double>::scale, "C++: ROL::Vector<double>::scale(const double) --> void", pybind11::arg("alpha"));
		cl.def("dot", (double (ROL::Vector<double>::*)(const class ROL::Vector<double> &) const) &ROL::Vector<double>::dot, "C++: ROL::Vector<double>::dot(const class ROL::Vector<double> &) const --> double", pybind11::arg("x"));
		cl.def("norm", (double (ROL::Vector<double>::*)() const) &ROL::Vector<double>::norm, "C++: ROL::Vector<double>::norm() const --> double");
		cl.def("clone", (class Teuchos::RCP<class ROL::Vector<double> > (ROL::Vector<double>::*)() const) &ROL::Vector<double>::clone, "C++: ROL::Vector<double>::clone() const --> class Teuchos::RCP<class ROL::Vector<double> >");
		cl.def("axpy", (void (ROL::Vector<double>::*)(const double, const class ROL::Vector<double> &)) &ROL::Vector<double>::axpy, "Compute \n where \n.\n\n             \n is the scaling of \n             \n\n     is a vector.\n\n             On return \n.\n             Uses #clone, #set, #scale and #plus for the computation.\n             Please overload if a more efficient implementation is needed.\n\n             ---\n\nC++: ROL::Vector<double>::axpy(const double, const class ROL::Vector<double> &) --> void", pybind11::arg("alpha"), pybind11::arg("x"));
		cl.def("zero", (void (ROL::Vector<double>::*)()) &ROL::Vector<double>::zero, "Set to zero vector.\n\n             Uses #scale by zero for the computation.\n             Please overload if a more efficient implementation is needed.\n\n             ---\n\nC++: ROL::Vector<double>::zero() --> void");
		cl.def("basis", (class Teuchos::RCP<class ROL::Vector<double> > (ROL::Vector<double>::*)(const int) const) &ROL::Vector<double>::basis, "Return i-th basis vector.\n\n             \n is the index of the basis function.\n             \n\n A reference-counted pointer to the basis vector with index \n\n             Overloading the basis is only required if the default gradient implementation\n             is used, which computes a finite-difference approximation.\n\n             ---\n\nC++: ROL::Vector<double>::basis(const int) const --> class Teuchos::RCP<class ROL::Vector<double> >", pybind11::arg("i"));
		cl.def("dimension", (int (ROL::Vector<double>::*)() const) &ROL::Vector<double>::dimension, "Return dimension of the vector space.\n\n             \n The dimension of the vector space, i.e., the total number of basis vectors.\n\n             Overload if the basis is overloaded.\n\n             ---\n\nC++: ROL::Vector<double>::dimension() const --> int");
		cl.def("set", (void (ROL::Vector<double>::*)(const class ROL::Vector<double> &)) &ROL::Vector<double>::set, "Set \n where \n.\n\n             \n     is a vector.\n\n             On return \n.\n             Uses #zero and #plus methods for the computation.\n             Please overload if a more efficient implementation is needed.\n\n             ---\n\nC++: ROL::Vector<double>::set(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("dual", (const class ROL::Vector<double> & (ROL::Vector<double>::*)() const) &ROL::Vector<double>::dual, "Return dual representation of \n, for example,\n             the result of applying a Riesz map, or change of basis, or\n             change of memory layout.\n\n             \n         A const reference to dual representation.\n\n             By default, returns the current object.\n             Please overload if you need a dual representation.\n\n             ---\n\nC++: ROL::Vector<double>::dual() const --> const class ROL::Vector<double> &", pybind11::return_value_policy::automatic);
		cl.def("apply", (double (ROL::Vector<double>::*)(const class ROL::Vector<double> &) const) &ROL::Vector<double>::apply, "Apply \n to a dual vector.  This is equivalent\n             to the call \n\n.\n\n             \n      is a vector\n             \n\n         The number equal to \n.\n\n             ---\n\nC++: ROL::Vector<double>::apply(const class ROL::Vector<double> &) const --> double", pybind11::arg("x"));
		cl.def("applyUnary", (void (ROL::Vector<double>::*)(const class ROL::Elementwise::UnaryFunction<double> &)) &ROL::Vector<double>::applyUnary, "C++: ROL::Vector<double>::applyUnary(const class ROL::Elementwise::UnaryFunction<double> &) --> void", pybind11::arg("f"));
		cl.def("applyBinary", (void (ROL::Vector<double>::*)(const class ROL::Elementwise::BinaryFunction<double> &, const class ROL::Vector<double> &)) &ROL::Vector<double>::applyBinary, "C++: ROL::Vector<double>::applyBinary(const class ROL::Elementwise::BinaryFunction<double> &, const class ROL::Vector<double> &) --> void", pybind11::arg("f"), pybind11::arg("x"));
		cl.def("reduce", (double (ROL::Vector<double>::*)(const class ROL::Elementwise::ReductionOp<double> &) const) &ROL::Vector<double>::reduce, "C++: ROL::Vector<double>::reduce(const class ROL::Elementwise::ReductionOp<double> &) const --> double", pybind11::arg("r"));
		cl.def("print", (void (ROL::Vector<double>::*)(std::ostream &) const) &ROL::Vector<double>::print, "C++: ROL::Vector<double>::print(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("setScalar", (void (ROL::Vector<double>::*)(const double)) &ROL::Vector<double>::setScalar, "Set \n where \n.\n\n             \n     is a scalar.\n\n             On return \n.\n             Uses #applyUnary methods for the computation.\n             Please overload if a more efficient implementation is needed.\n\n             ---\n\nC++: ROL::Vector<double>::setScalar(const double) --> void", pybind11::arg("C"));
		cl.def("randomize", [](ROL::Vector<double> &o) -> void { return o.randomize(); }, "");
		cl.def("randomize", [](ROL::Vector<double> &o, const double & a0) -> void { return o.randomize(a0); }, "", pybind11::arg("l"));
		cl.def("randomize", (void (ROL::Vector<double>::*)(const double, const double)) &ROL::Vector<double>::randomize, "Set vector to be uniform random between [l,u].\n\n             \n     is a the lower bound.\n             \n\n     is a the upper bound.\n\n             On return the components of \n are uniform\n             random numbers on the interval \n\n.\n       	     The default implementation uses #applyUnary methods for the\n       	     computation. Please overload if a more efficient implementation is\n             needed.\n\n             ---\n\nC++: ROL::Vector<double>::randomize(const double, const double) --> void", pybind11::arg("l"), pybind11::arg("u"));
		cl.def("checkVector", [](ROL::Vector<double> const &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> std::vector<double, class std::allocator<double> > { return o.checkVector(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("y"));
		cl.def("checkVector", [](ROL::Vector<double> const &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const bool & a2) -> std::vector<double, class std::allocator<double> > { return o.checkVector(a0, a1, a2); }, "", pybind11::arg("x"), pybind11::arg("y"), pybind11::arg("printToStream"));
		cl.def("checkVector", (class std::vector<double, class std::allocator<double> > (ROL::Vector<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) const) &ROL::Vector<double>::checkVector, "Verify vector-space methods.\n\n             \n     is a vector.\n             \n\n     is a vector.\n\n             Returns a vector of Reals, all of which should be close to zero.\n             They represent consistency errors in the vector space properties,\n             as follows:\n\n             - Commutativity of addition: \n.\n             - Associativity of addition: \n\n.\n             - Identity element of addition: \n\n.\n             - Inverse elements of addition: \n\n.\n             - Identity element of scalar multiplication: \n\n.\n             - Consistency of scalar multiplication with field multiplication: \n\n.\n             - Distributivity of scalar multiplication with respect to field addition: \n\n.\n             - Distributivity of scalar multiplication with respect to vector addition: \n\n.\n             - Commutativity of dot (inner) product over the field of reals: \n\n.\n             - Additivity of dot (inner) product: \n\n.\n             - Consistency of scalar multiplication and norm: \n\n.\n             - Reflexivity: \n\n .\n             - Consistency of apply and dual: \n\n.\n\n             The consistency errors are defined as the norms or absolute values of the differences between the left-hand\n             side and the right-hand side terms in the above equalities.\n\n             ---\n\nC++: ROL::Vector<double>::checkVector(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) const --> class std::vector<double, class std::allocator<double> >", pybind11::arg("x"), pybind11::arg("y"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Vector<double> & (ROL::Vector<double>::*)(const class ROL::Vector<double> &)) &ROL::Vector<double>::operator=, "C++: ROL::Vector<double>::operator=(const class ROL::Vector<double> &) --> class ROL::Vector<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	// ROL::RiskMeasureInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) file:ROL_RiskMeasureInfo.hpp line:53
	M("ROL").def("RiskMeasureInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5) -> void { return ROL::RiskMeasureInfo(a0, a1, a2, a3, a4, a5); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"));
	M("ROL").def("RiskMeasureInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5, const bool & a6) -> void { return ROL::RiskMeasureInfo(a0, a1, a2, a3, a4, a5, a6); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"));
	M("ROL").def("RiskMeasureInfo", (void (*)(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &)) &ROL::RiskMeasureInfo<double>, "C++: ROL::RiskMeasureInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) --> void", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"), pybind11::arg("outStream"));

	// ROL::DeviationMeasureInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) file:ROL_DeviationMeasureInfo.hpp line:53
	M("ROL").def("DeviationMeasureInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5) -> void { return ROL::DeviationMeasureInfo(a0, a1, a2, a3, a4, a5); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"));
	M("ROL").def("DeviationMeasureInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5, const bool & a6) -> void { return ROL::DeviationMeasureInfo(a0, a1, a2, a3, a4, a5, a6); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"));
	M("ROL").def("DeviationMeasureInfo", (void (*)(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &)) &ROL::DeviationMeasureInfo<double>, "C++: ROL::DeviationMeasureInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) --> void", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"), pybind11::arg("outStream"));

	// ROL::ErrorMeasureInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) file:ROL_ErrorMeasureInfo.hpp line:53
	M("ROL").def("ErrorMeasureInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5) -> void { return ROL::ErrorMeasureInfo(a0, a1, a2, a3, a4, a5); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"));
	M("ROL").def("ErrorMeasureInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5, const bool & a6) -> void { return ROL::ErrorMeasureInfo(a0, a1, a2, a3, a4, a5, a6); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"));
	M("ROL").def("ErrorMeasureInfo", (void (*)(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &)) &ROL::ErrorMeasureInfo<double>, "C++: ROL::ErrorMeasureInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) --> void", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"), pybind11::arg("outStream"));

	// ROL::RegretMeasureInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) file:ROL_RegretMeasureInfo.hpp line:53
	M("ROL").def("RegretMeasureInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5) -> void { return ROL::RegretMeasureInfo(a0, a1, a2, a3, a4, a5); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"));
	M("ROL").def("RegretMeasureInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5, const bool & a6) -> void { return ROL::RegretMeasureInfo(a0, a1, a2, a3, a4, a5, a6); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"));
	M("ROL").def("RegretMeasureInfo", (void (*)(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &)) &ROL::RegretMeasureInfo<double>, "C++: ROL::RegretMeasureInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) --> void", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"), pybind11::arg("outStream"));

	// ROL::ProbabilityInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) file:ROL_ProbabilityInfo.hpp line:53
	M("ROL").def("ProbabilityInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5) -> void { return ROL::ProbabilityInfo(a0, a1, a2, a3, a4, a5); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"));
	M("ROL").def("ProbabilityInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5, const bool & a6) -> void { return ROL::ProbabilityInfo(a0, a1, a2, a3, a4, a5, a6); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"));
	M("ROL").def("ProbabilityInfo", (void (*)(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &)) &ROL::ProbabilityInfo<double>, "C++: ROL::ProbabilityInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) --> void", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"), pybind11::arg("outStream"));

	// ROL::RandVarFunctionalInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) file:ROL_RandVarFunctionalInfo.hpp line:56
	M("ROL").def("RandVarFunctionalInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5) -> void { return ROL::RandVarFunctionalInfo(a0, a1, a2, a3, a4, a5); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"));
	M("ROL").def("RandVarFunctionalInfo", [](class Teuchos::ParameterList & a0, std::string & a1, int & a2, class std::vector<double, class std::allocator<double> > & a3, class std::vector<double, class std::allocator<double> > & a4, bool & a5, const bool & a6) -> void { return ROL::RandVarFunctionalInfo(a0, a1, a2, a3, a4, a5, a6); }, "", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"));
	M("ROL").def("RandVarFunctionalInfo", (void (*)(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &)) &ROL::RandVarFunctionalInfo<double>, "C++: ROL::RandVarFunctionalInfo(class Teuchos::ParameterList &, std::string &, int &, class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, bool &, const bool, std::ostream &) --> void", pybind11::arg("parlist"), pybind11::arg("name"), pybind11::arg("nStatistic"), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("isBoundActivated"), pybind11::arg("printToStream"), pybind11::arg("outStream"));

	{ // ROL::RiskVector file:ROL_RiskVector.hpp line:54
		pybind11::class_<ROL::RiskVector<double>, Teuchos::RCP<ROL::RiskVector<double>>, PyCallBack_ROL_RiskVector_double_t, ROL::Vector<double>> cl(M("ROL"), "RiskVector_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](class Teuchos::RCP<class Teuchos::ParameterList> & a0, const class Teuchos::RCP<class ROL::Vector<double> > & a1){ return new ROL::RiskVector<double>(a0, a1); }, [](class Teuchos::RCP<class Teuchos::ParameterList> & a0, const class Teuchos::RCP<class ROL::Vector<double> > & a1){ return new PyCallBack_ROL_RiskVector_double_t(a0, a1); } ), "doc");
		cl.def( pybind11::init<class Teuchos::RCP<class Teuchos::ParameterList> &, const class Teuchos::RCP<class ROL::Vector<double> > &, const double>(), pybind11::arg("parlist"), pybind11::arg("vec"), pybind11::arg("stat") );

		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Vector<double> > &>(), pybind11::arg("vec") );

		cl.def( pybind11::init( [](PyCallBack_ROL_RiskVector_double_t const &o){ return new PyCallBack_ROL_RiskVector_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::RiskVector<double> const &o){ return new ROL::RiskVector<double>(o); } ) );
		cl.def("set", (void (ROL::RiskVector<double>::*)(const class ROL::Vector<double> &)) &ROL::RiskVector<double>::set, "C++: ROL::RiskVector<double>::set(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("plus", (void (ROL::RiskVector<double>::*)(const class ROL::Vector<double> &)) &ROL::RiskVector<double>::plus, "C++: ROL::RiskVector<double>::plus(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("scale", (void (ROL::RiskVector<double>::*)(const double)) &ROL::RiskVector<double>::scale, "C++: ROL::RiskVector<double>::scale(const double) --> void", pybind11::arg("alpha"));
		cl.def("axpy", (void (ROL::RiskVector<double>::*)(const double, const class ROL::Vector<double> &)) &ROL::RiskVector<double>::axpy, "C++: ROL::RiskVector<double>::axpy(const double, const class ROL::Vector<double> &) --> void", pybind11::arg("alpha"), pybind11::arg("x"));
		cl.def("dot", (double (ROL::RiskVector<double>::*)(const class ROL::Vector<double> &) const) &ROL::RiskVector<double>::dot, "C++: ROL::RiskVector<double>::dot(const class ROL::Vector<double> &) const --> double", pybind11::arg("x"));
		cl.def("norm", (double (ROL::RiskVector<double>::*)() const) &ROL::RiskVector<double>::norm, "C++: ROL::RiskVector<double>::norm() const --> double");
		cl.def("clone", (class Teuchos::RCP<class ROL::Vector<double> > (ROL::RiskVector<double>::*)() const) &ROL::RiskVector<double>::clone, "C++: ROL::RiskVector<double>::clone() const --> class Teuchos::RCP<class ROL::Vector<double> >");
		cl.def("dual", (const class ROL::Vector<double> & (ROL::RiskVector<double>::*)() const) &ROL::RiskVector<double>::dual, "C++: ROL::RiskVector<double>::dual() const --> const class ROL::Vector<double> &", pybind11::return_value_policy::automatic);
		cl.def("basis", (class Teuchos::RCP<class ROL::Vector<double> > (ROL::RiskVector<double>::*)(const int) const) &ROL::RiskVector<double>::basis, "C++: ROL::RiskVector<double>::basis(const int) const --> class Teuchos::RCP<class ROL::Vector<double> >", pybind11::arg("i"));
		cl.def("applyUnary", (void (ROL::RiskVector<double>::*)(const class ROL::Elementwise::UnaryFunction<double> &)) &ROL::RiskVector<double>::applyUnary, "C++: ROL::RiskVector<double>::applyUnary(const class ROL::Elementwise::UnaryFunction<double> &) --> void", pybind11::arg("f"));
		cl.def("applyBinary", (void (ROL::RiskVector<double>::*)(const class ROL::Elementwise::BinaryFunction<double> &, const class ROL::Vector<double> &)) &ROL::RiskVector<double>::applyBinary, "C++: ROL::RiskVector<double>::applyBinary(const class ROL::Elementwise::BinaryFunction<double> &, const class ROL::Vector<double> &) --> void", pybind11::arg("f"), pybind11::arg("x"));
		cl.def("reduce", (double (ROL::RiskVector<double>::*)(const class ROL::Elementwise::ReductionOp<double> &) const) &ROL::RiskVector<double>::reduce, "C++: ROL::RiskVector<double>::reduce(const class ROL::Elementwise::ReductionOp<double> &) const --> double", pybind11::arg("r"));
		cl.def("setScalar", (void (ROL::RiskVector<double>::*)(const double)) &ROL::RiskVector<double>::setScalar, "C++: ROL::RiskVector<double>::setScalar(const double) --> void", pybind11::arg("C"));
		cl.def("randomize", [](ROL::RiskVector<double> &o) -> void { return o.randomize(); }, "");
		cl.def("randomize", [](ROL::RiskVector<double> &o, const double & a0) -> void { return o.randomize(a0); }, "", pybind11::arg("l"));
		cl.def("randomize", (void (ROL::RiskVector<double>::*)(const double, const double)) &ROL::RiskVector<double>::randomize, "C++: ROL::RiskVector<double>::randomize(const double, const double) --> void", pybind11::arg("l"), pybind11::arg("u"));
		cl.def("dimension", (int (ROL::RiskVector<double>::*)() const) &ROL::RiskVector<double>::dimension, "C++: ROL::RiskVector<double>::dimension() const --> int");
		cl.def("getStatisticVector", [](ROL::RiskVector<double> &o, const int & a0) -> Teuchos::RCP<class ROL::StdVector<double, double> > { return o.getStatisticVector(a0); }, "", pybind11::arg("comp"));
		cl.def("getStatisticVector", (class Teuchos::RCP<class ROL::StdVector<double, double> > (ROL::RiskVector<double>::*)(const int, const int)) &ROL::RiskVector<double>::getStatisticVector, "C++: ROL::RiskVector<double>::getStatisticVector(const int, const int) --> class Teuchos::RCP<class ROL::StdVector<double, double> >", pybind11::arg("comp"), pybind11::arg("index"));
		cl.def("getVector", (class Teuchos::RCP<class ROL::Vector<double> > (ROL::RiskVector<double>::*)()) &ROL::RiskVector<double>::getVector, "C++: ROL::RiskVector<double>::getVector() --> class Teuchos::RCP<class ROL::Vector<double> >");
		cl.def("getStatistic", [](ROL::RiskVector<double> &o) -> Teuchos::RCP<class std::vector<double, class std::allocator<double> > > { return o.getStatistic(); }, "");
		cl.def("getStatistic", [](ROL::RiskVector<double> &o, const int & a0) -> Teuchos::RCP<class std::vector<double, class std::allocator<double> > > { return o.getStatistic(a0); }, "", pybind11::arg("comp"));
		cl.def("getStatistic", (class Teuchos::RCP<class std::vector<double, class std::allocator<double> > > (ROL::RiskVector<double>::*)(const int, const int)) &ROL::RiskVector<double>::getStatistic, "C++: ROL::RiskVector<double>::getStatistic(const int, const int) --> class Teuchos::RCP<class std::vector<double, class std::allocator<double> > >", pybind11::arg("comp"), pybind11::arg("index"));
		cl.def("setStatistic", [](ROL::RiskVector<double> &o, const double & a0) -> void { return o.setStatistic(a0); }, "", pybind11::arg("stat"));
		cl.def("setStatistic", [](ROL::RiskVector<double> &o, const double & a0, const int & a1) -> void { return o.setStatistic(a0, a1); }, "", pybind11::arg("stat"), pybind11::arg("comp"));
		cl.def("setStatistic", (void (ROL::RiskVector<double>::*)(const double, const int, const int)) &ROL::RiskVector<double>::setStatistic, "C++: ROL::RiskVector<double>::setStatistic(const double, const int, const int) --> void", pybind11::arg("stat"), pybind11::arg("comp"), pybind11::arg("index"));
		cl.def("setStatistic", [](ROL::RiskVector<double> &o, const class std::vector<double, class std::allocator<double> > & a0) -> void { return o.setStatistic(a0); }, "", pybind11::arg("stat"));
		cl.def("setStatistic", [](ROL::RiskVector<double> &o, const class std::vector<double, class std::allocator<double> > & a0, const int & a1) -> void { return o.setStatistic(a0, a1); }, "", pybind11::arg("stat"), pybind11::arg("comp"));
		cl.def("setStatistic", (void (ROL::RiskVector<double>::*)(const class std::vector<double, class std::allocator<double> > &, const int, const int)) &ROL::RiskVector<double>::setStatistic, "C++: ROL::RiskVector<double>::setStatistic(const class std::vector<double, class std::allocator<double> > &, const int, const int) --> void", pybind11::arg("stat"), pybind11::arg("comp"), pybind11::arg("index"));
		cl.def("setVector", (void (ROL::RiskVector<double>::*)(const class ROL::Vector<double> &)) &ROL::RiskVector<double>::setVector, "C++: ROL::RiskVector<double>::setVector(const class ROL::Vector<double> &) --> void", pybind11::arg("vec"));
		cl.def("assign", (class ROL::RiskVector<double> & (ROL::RiskVector<double>::*)(const class ROL::RiskVector<double> &)) &ROL::RiskVector<double>::operator=, "C++: ROL::RiskVector<double>::operator=(const class ROL::RiskVector<double> &) --> class ROL::RiskVector<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("plus", (void (ROL::Vector<double>::*)(const class ROL::Vector<double> &)) &ROL::Vector<double>::plus, "C++: ROL::Vector<double>::plus(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("scale", (void (ROL::Vector<double>::*)(const double)) &ROL::Vector<double>::scale, "C++: ROL::Vector<double>::scale(const double) --> void", pybind11::arg("alpha"));
		cl.def("dot", (double (ROL::Vector<double>::*)(const class ROL::Vector<double> &) const) &ROL::Vector<double>::dot, "C++: ROL::Vector<double>::dot(const class ROL::Vector<double> &) const --> double", pybind11::arg("x"));
		cl.def("norm", (double (ROL::Vector<double>::*)() const) &ROL::Vector<double>::norm, "C++: ROL::Vector<double>::norm() const --> double");
		cl.def("clone", (class Teuchos::RCP<class ROL::Vector<double> > (ROL::Vector<double>::*)() const) &ROL::Vector<double>::clone, "C++: ROL::Vector<double>::clone() const --> class Teuchos::RCP<class ROL::Vector<double> >");
		cl.def("axpy", (void (ROL::Vector<double>::*)(const double, const class ROL::Vector<double> &)) &ROL::Vector<double>::axpy, "Compute \n where \n.\n\n             \n is the scaling of \n             \n\n     is a vector.\n\n             On return \n.\n             Uses #clone, #set, #scale and #plus for the computation.\n             Please overload if a more efficient implementation is needed.\n\n             ---\n\nC++: ROL::Vector<double>::axpy(const double, const class ROL::Vector<double> &) --> void", pybind11::arg("alpha"), pybind11::arg("x"));
		cl.def("zero", (void (ROL::Vector<double>::*)()) &ROL::Vector<double>::zero, "Set to zero vector.\n\n             Uses #scale by zero for the computation.\n             Please overload if a more efficient implementation is needed.\n\n             ---\n\nC++: ROL::Vector<double>::zero() --> void");
		cl.def("basis", (class Teuchos::RCP<class ROL::Vector<double> > (ROL::Vector<double>::*)(const int) const) &ROL::Vector<double>::basis, "Return i-th basis vector.\n\n             \n is the index of the basis function.\n             \n\n A reference-counted pointer to the basis vector with index \n\n             Overloading the basis is only required if the default gradient implementation\n             is used, which computes a finite-difference approximation.\n\n             ---\n\nC++: ROL::Vector<double>::basis(const int) const --> class Teuchos::RCP<class ROL::Vector<double> >", pybind11::arg("i"));
		cl.def("dimension", (int (ROL::Vector<double>::*)() const) &ROL::Vector<double>::dimension, "Return dimension of the vector space.\n\n             \n The dimension of the vector space, i.e., the total number of basis vectors.\n\n             Overload if the basis is overloaded.\n\n             ---\n\nC++: ROL::Vector<double>::dimension() const --> int");
		cl.def("set", (void (ROL::Vector<double>::*)(const class ROL::Vector<double> &)) &ROL::Vector<double>::set, "Set \n where \n.\n\n             \n     is a vector.\n\n             On return \n.\n             Uses #zero and #plus methods for the computation.\n             Please overload if a more efficient implementation is needed.\n\n             ---\n\nC++: ROL::Vector<double>::set(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("dual", (const class ROL::Vector<double> & (ROL::Vector<double>::*)() const) &ROL::Vector<double>::dual, "Return dual representation of \n, for example,\n             the result of applying a Riesz map, or change of basis, or\n             change of memory layout.\n\n             \n         A const reference to dual representation.\n\n             By default, returns the current object.\n             Please overload if you need a dual representation.\n\n             ---\n\nC++: ROL::Vector<double>::dual() const --> const class ROL::Vector<double> &", pybind11::return_value_policy::automatic);
		cl.def("apply", (double (ROL::Vector<double>::*)(const class ROL::Vector<double> &) const) &ROL::Vector<double>::apply, "Apply \n to a dual vector.  This is equivalent\n             to the call \n\n.\n\n             \n      is a vector\n             \n\n         The number equal to \n.\n\n             ---\n\nC++: ROL::Vector<double>::apply(const class ROL::Vector<double> &) const --> double", pybind11::arg("x"));
		cl.def("applyUnary", (void (ROL::Vector<double>::*)(const class ROL::Elementwise::UnaryFunction<double> &)) &ROL::Vector<double>::applyUnary, "C++: ROL::Vector<double>::applyUnary(const class ROL::Elementwise::UnaryFunction<double> &) --> void", pybind11::arg("f"));
		cl.def("applyBinary", (void (ROL::Vector<double>::*)(const class ROL::Elementwise::BinaryFunction<double> &, const class ROL::Vector<double> &)) &ROL::Vector<double>::applyBinary, "C++: ROL::Vector<double>::applyBinary(const class ROL::Elementwise::BinaryFunction<double> &, const class ROL::Vector<double> &) --> void", pybind11::arg("f"), pybind11::arg("x"));
		cl.def("reduce", (double (ROL::Vector<double>::*)(const class ROL::Elementwise::ReductionOp<double> &) const) &ROL::Vector<double>::reduce, "C++: ROL::Vector<double>::reduce(const class ROL::Elementwise::ReductionOp<double> &) const --> double", pybind11::arg("r"));
		cl.def("print", (void (ROL::Vector<double>::*)(std::ostream &) const) &ROL::Vector<double>::print, "C++: ROL::Vector<double>::print(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("setScalar", (void (ROL::Vector<double>::*)(const double)) &ROL::Vector<double>::setScalar, "Set \n where \n.\n\n             \n     is a scalar.\n\n             On return \n.\n             Uses #applyUnary methods for the computation.\n             Please overload if a more efficient implementation is needed.\n\n             ---\n\nC++: ROL::Vector<double>::setScalar(const double) --> void", pybind11::arg("C"));
		cl.def("randomize", [](ROL::Vector<double> &o) -> void { return o.randomize(); }, "");
		cl.def("randomize", [](ROL::Vector<double> &o, const double & a0) -> void { return o.randomize(a0); }, "", pybind11::arg("l"));
		cl.def("randomize", (void (ROL::Vector<double>::*)(const double, const double)) &ROL::Vector<double>::randomize, "Set vector to be uniform random between [l,u].\n\n             \n     is a the lower bound.\n             \n\n     is a the upper bound.\n\n             On return the components of \n are uniform\n             random numbers on the interval \n\n.\n       	     The default implementation uses #applyUnary methods for the\n       	     computation. Please overload if a more efficient implementation is\n             needed.\n\n             ---\n\nC++: ROL::Vector<double>::randomize(const double, const double) --> void", pybind11::arg("l"), pybind11::arg("u"));
		cl.def("checkVector", [](ROL::Vector<double> const &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> std::vector<double, class std::allocator<double> > { return o.checkVector(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("y"));
		cl.def("checkVector", [](ROL::Vector<double> const &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const bool & a2) -> std::vector<double, class std::allocator<double> > { return o.checkVector(a0, a1, a2); }, "", pybind11::arg("x"), pybind11::arg("y"), pybind11::arg("printToStream"));
		cl.def("checkVector", (class std::vector<double, class std::allocator<double> > (ROL::Vector<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) const) &ROL::Vector<double>::checkVector, "Verify vector-space methods.\n\n             \n     is a vector.\n             \n\n     is a vector.\n\n             Returns a vector of Reals, all of which should be close to zero.\n             They represent consistency errors in the vector space properties,\n             as follows:\n\n             - Commutativity of addition: \n.\n             - Associativity of addition: \n\n.\n             - Identity element of addition: \n\n.\n             - Inverse elements of addition: \n\n.\n             - Identity element of scalar multiplication: \n\n.\n             - Consistency of scalar multiplication with field multiplication: \n\n.\n             - Distributivity of scalar multiplication with respect to field addition: \n\n.\n             - Distributivity of scalar multiplication with respect to vector addition: \n\n.\n             - Commutativity of dot (inner) product over the field of reals: \n\n.\n             - Additivity of dot (inner) product: \n\n.\n             - Consistency of scalar multiplication and norm: \n\n.\n             - Reflexivity: \n\n .\n             - Consistency of apply and dual: \n\n.\n\n             The consistency errors are defined as the norms or absolute values of the differences between the left-hand\n             side and the right-hand side terms in the above equalities.\n\n             ---\n\nC++: ROL::Vector<double>::checkVector(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) const --> class std::vector<double, class std::allocator<double> >", pybind11::arg("x"), pybind11::arg("y"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Vector<double> & (ROL::Vector<double>::*)(const class ROL::Vector<double> &)) &ROL::Vector<double>::operator=, "C++: ROL::Vector<double>::operator=(const class ROL::Vector<double> &) --> class ROL::Vector<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::PositiveFunction file:ROL_PositiveFunction.hpp line:50
		pybind11::class_<ROL::PositiveFunction<double>, Teuchos::RCP<ROL::PositiveFunction<double>>, PyCallBack_ROL_PositiveFunction_double_t> cl(M("ROL"), "PositiveFunction_double_t", "", pybind11::module_local());
		cl.def(pybind11::init<PyCallBack_ROL_PositiveFunction_double_t const &>());
		cl.def( pybind11::init( [](){ return new PyCallBack_ROL_PositiveFunction_double_t(); } ) );
		cl.def("evaluate", (double (ROL::PositiveFunction<double>::*)(double, int)) &ROL::PositiveFunction<double>::evaluate, "C++: ROL::PositiveFunction<double>::evaluate(double, int) --> double", pybind11::arg("input"), pybind11::arg("deriv"));
		cl.def("assign", (class ROL::PositiveFunction<double> & (ROL::PositiveFunction<double>::*)(const class ROL::PositiveFunction<double> &)) &ROL::PositiveFunction<double>::operator=, "C++: ROL::PositiveFunction<double>::operator=(const class ROL::PositiveFunction<double> &) --> class ROL::PositiveFunction<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Distribution file:ROL_Distribution.hpp line:55
		pybind11::class_<ROL::Distribution<double>, Teuchos::RCP<ROL::Distribution<double>>, PyCallBack_ROL_Distribution_double_t> cl(M("ROL"), "Distribution_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](PyCallBack_ROL_Distribution_double_t const &o){ return new PyCallBack_ROL_Distribution_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Distribution<double> const &o){ return new ROL::Distribution<double>(o); } ) );
		cl.def( pybind11::init( [](){ return new ROL::Distribution<double>(); }, [](){ return new PyCallBack_ROL_Distribution_double_t(); } ) );
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Dirac file:ROL_Dirac.hpp line:53
		pybind11::class_<ROL::Dirac<double>, Teuchos::RCP<ROL::Dirac<double>>, PyCallBack_ROL_Dirac_double_t, ROL::Distribution<double>> cl(M("ROL"), "Dirac_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Dirac<double>(); }, [](){ return new PyCallBack_ROL_Dirac_double_t(); } ), "doc");
		cl.def( pybind11::init<const double>(), pybind11::arg("data") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Dirac_double_t const &o){ return new PyCallBack_ROL_Dirac_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Dirac<double> const &o){ return new ROL::Dirac<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Dirac<double>::*)(const double) const) &ROL::Dirac<double>::evaluatePDF, "C++: ROL::Dirac<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Dirac<double>::*)(const double) const) &ROL::Dirac<double>::evaluateCDF, "C++: ROL::Dirac<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Dirac<double>::*)(const double) const) &ROL::Dirac<double>::integrateCDF, "C++: ROL::Dirac<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Dirac<double>::*)(const double) const) &ROL::Dirac<double>::invertCDF, "C++: ROL::Dirac<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Dirac<double>::*)(const unsigned long) const) &ROL::Dirac<double>::moment, "C++: ROL::Dirac<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Dirac<double>::*)() const) &ROL::Dirac<double>::lowerBound, "C++: ROL::Dirac<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Dirac<double>::*)() const) &ROL::Dirac<double>::upperBound, "C++: ROL::Dirac<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Dirac<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Dirac<double>::*)(std::ostream &) const) &ROL::Dirac<double>::test, "C++: ROL::Dirac<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Dirac<double> & (ROL::Dirac<double>::*)(const class ROL::Dirac<double> &)) &ROL::Dirac<double>::operator=, "C++: ROL::Dirac<double>::operator=(const class ROL::Dirac<double> &) --> class ROL::Dirac<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Gaussian file:ROL_Gaussian.hpp line:53
		pybind11::class_<ROL::Gaussian<double>, Teuchos::RCP<ROL::Gaussian<double>>, PyCallBack_ROL_Gaussian_double_t, ROL::Distribution<double>> cl(M("ROL"), "Gaussian_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Gaussian<double>(); }, [](){ return new PyCallBack_ROL_Gaussian_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Gaussian<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Gaussian_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("mean"), pybind11::arg("variance") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Gaussian_double_t const &o){ return new PyCallBack_ROL_Gaussian_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Gaussian<double> const &o){ return new ROL::Gaussian<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Gaussian<double>::*)(const double) const) &ROL::Gaussian<double>::evaluatePDF, "C++: ROL::Gaussian<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Gaussian<double>::*)(const double) const) &ROL::Gaussian<double>::evaluateCDF, "C++: ROL::Gaussian<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Gaussian<double>::*)(const double) const) &ROL::Gaussian<double>::integrateCDF, "C++: ROL::Gaussian<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Gaussian<double>::*)(const double) const) &ROL::Gaussian<double>::invertCDF, "C++: ROL::Gaussian<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Gaussian<double>::*)(const unsigned long) const) &ROL::Gaussian<double>::moment, "C++: ROL::Gaussian<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Gaussian<double>::*)() const) &ROL::Gaussian<double>::lowerBound, "C++: ROL::Gaussian<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Gaussian<double>::*)() const) &ROL::Gaussian<double>::upperBound, "C++: ROL::Gaussian<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Gaussian<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Gaussian<double>::*)(std::ostream &) const) &ROL::Gaussian<double>::test, "C++: ROL::Gaussian<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Gaussian<double> & (ROL::Gaussian<double>::*)(const class ROL::Gaussian<double> &)) &ROL::Gaussian<double>::operator=, "C++: ROL::Gaussian<double>::operator=(const class ROL::Gaussian<double> &) --> class ROL::Gaussian<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::TruncatedGaussian file:ROL_TruncatedGaussian.hpp line:55
		pybind11::class_<ROL::TruncatedGaussian<double>, Teuchos::RCP<ROL::TruncatedGaussian<double>>, PyCallBack_ROL_TruncatedGaussian_double_t, ROL::Distribution<double>> cl(M("ROL"), "TruncatedGaussian_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::TruncatedGaussian<double>(); }, [](){ return new PyCallBack_ROL_TruncatedGaussian_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::TruncatedGaussian<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_TruncatedGaussian_double_t(a0); } ), "doc");
		cl.def( pybind11::init( [](const double & a0, const double & a1){ return new ROL::TruncatedGaussian<double>(a0, a1); }, [](const double & a0, const double & a1){ return new PyCallBack_ROL_TruncatedGaussian_double_t(a0, a1); } ), "doc");
		cl.def( pybind11::init( [](const double & a0, const double & a1, const double & a2){ return new ROL::TruncatedGaussian<double>(a0, a1, a2); }, [](const double & a0, const double & a1, const double & a2){ return new PyCallBack_ROL_TruncatedGaussian_double_t(a0, a1, a2); } ), "doc");
		cl.def( pybind11::init<const double, const double, const double, const double>(), pybind11::arg("lo"), pybind11::arg("up"), pybind11::arg("mean"), pybind11::arg("sdev") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_TruncatedGaussian_double_t const &o){ return new PyCallBack_ROL_TruncatedGaussian_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::TruncatedGaussian<double> const &o){ return new ROL::TruncatedGaussian<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::TruncatedGaussian<double>::*)(const double) const) &ROL::TruncatedGaussian<double>::evaluatePDF, "C++: ROL::TruncatedGaussian<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::TruncatedGaussian<double>::*)(const double) const) &ROL::TruncatedGaussian<double>::evaluateCDF, "C++: ROL::TruncatedGaussian<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::TruncatedGaussian<double>::*)(const double) const) &ROL::TruncatedGaussian<double>::integrateCDF, "C++: ROL::TruncatedGaussian<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::TruncatedGaussian<double>::*)(const double) const) &ROL::TruncatedGaussian<double>::invertCDF, "C++: ROL::TruncatedGaussian<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::TruncatedGaussian<double>::*)(const unsigned long) const) &ROL::TruncatedGaussian<double>::moment, "C++: ROL::TruncatedGaussian<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::TruncatedGaussian<double>::*)() const) &ROL::TruncatedGaussian<double>::lowerBound, "C++: ROL::TruncatedGaussian<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::TruncatedGaussian<double>::*)() const) &ROL::TruncatedGaussian<double>::upperBound, "C++: ROL::TruncatedGaussian<double>::upperBound() const --> double");
		cl.def("test", [](ROL::TruncatedGaussian<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::TruncatedGaussian<double>::*)(std::ostream &) const) &ROL::TruncatedGaussian<double>::test, "C++: ROL::TruncatedGaussian<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::TruncatedGaussian<double> & (ROL::TruncatedGaussian<double>::*)(const class ROL::TruncatedGaussian<double> &)) &ROL::TruncatedGaussian<double>::operator=, "C++: ROL::TruncatedGaussian<double>::operator=(const class ROL::TruncatedGaussian<double> &) --> class ROL::TruncatedGaussian<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Uniform file:ROL_Uniform.hpp line:53
		pybind11::class_<ROL::Uniform<double>, Teuchos::RCP<ROL::Uniform<double>>, PyCallBack_ROL_Uniform_double_t, ROL::Distribution<double>> cl(M("ROL"), "Uniform_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Uniform<double>(); }, [](){ return new PyCallBack_ROL_Uniform_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Uniform<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Uniform_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("lo"), pybind11::arg("up") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Uniform_double_t const &o){ return new PyCallBack_ROL_Uniform_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Uniform<double> const &o){ return new ROL::Uniform<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Uniform<double>::*)(const double) const) &ROL::Uniform<double>::evaluatePDF, "C++: ROL::Uniform<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Uniform<double>::*)(const double) const) &ROL::Uniform<double>::evaluateCDF, "C++: ROL::Uniform<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Uniform<double>::*)(const double) const) &ROL::Uniform<double>::integrateCDF, "C++: ROL::Uniform<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Uniform<double>::*)(const double) const) &ROL::Uniform<double>::invertCDF, "C++: ROL::Uniform<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Uniform<double>::*)(const unsigned long) const) &ROL::Uniform<double>::moment, "C++: ROL::Uniform<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Uniform<double>::*)() const) &ROL::Uniform<double>::lowerBound, "C++: ROL::Uniform<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Uniform<double>::*)() const) &ROL::Uniform<double>::upperBound, "C++: ROL::Uniform<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Uniform<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Uniform<double>::*)(std::ostream &) const) &ROL::Uniform<double>::test, "C++: ROL::Uniform<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Uniform<double> & (ROL::Uniform<double>::*)(const class ROL::Uniform<double> &)) &ROL::Uniform<double>::operator=, "C++: ROL::Uniform<double>::operator=(const class ROL::Uniform<double> &) --> class ROL::Uniform<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Logistic file:ROL_Logistic.hpp line:53
		pybind11::class_<ROL::Logistic<double>, Teuchos::RCP<ROL::Logistic<double>>, PyCallBack_ROL_Logistic_double_t, ROL::Distribution<double>> cl(M("ROL"), "Logistic_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Logistic<double>(); }, [](){ return new PyCallBack_ROL_Logistic_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Logistic<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Logistic_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("mean"), pybind11::arg("var") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Logistic_double_t const &o){ return new PyCallBack_ROL_Logistic_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Logistic<double> const &o){ return new ROL::Logistic<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Logistic<double>::*)(const double) const) &ROL::Logistic<double>::evaluatePDF, "C++: ROL::Logistic<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Logistic<double>::*)(const double) const) &ROL::Logistic<double>::evaluateCDF, "C++: ROL::Logistic<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Logistic<double>::*)(const double) const) &ROL::Logistic<double>::integrateCDF, "C++: ROL::Logistic<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Logistic<double>::*)(const double) const) &ROL::Logistic<double>::invertCDF, "C++: ROL::Logistic<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Logistic<double>::*)(const unsigned long) const) &ROL::Logistic<double>::moment, "C++: ROL::Logistic<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Logistic<double>::*)() const) &ROL::Logistic<double>::lowerBound, "C++: ROL::Logistic<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Logistic<double>::*)() const) &ROL::Logistic<double>::upperBound, "C++: ROL::Logistic<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Logistic<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Logistic<double>::*)(std::ostream &) const) &ROL::Logistic<double>::test, "C++: ROL::Logistic<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Logistic<double> & (ROL::Logistic<double>::*)(const class ROL::Logistic<double> &)) &ROL::Logistic<double>::operator=, "C++: ROL::Logistic<double>::operator=(const class ROL::Logistic<double> &) --> class ROL::Logistic<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Triangle file:ROL_Triangle.hpp line:53
		pybind11::class_<ROL::Triangle<double>, Teuchos::RCP<ROL::Triangle<double>>, PyCallBack_ROL_Triangle_double_t, ROL::Distribution<double>> cl(M("ROL"), "Triangle_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Triangle<double>(); }, [](){ return new PyCallBack_ROL_Triangle_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Triangle<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Triangle_double_t(a0); } ), "doc");
		cl.def( pybind11::init( [](const double & a0, const double & a1){ return new ROL::Triangle<double>(a0, a1); }, [](const double & a0, const double & a1){ return new PyCallBack_ROL_Triangle_double_t(a0, a1); } ), "doc");
		cl.def( pybind11::init<const double, const double, const double>(), pybind11::arg("a"), pybind11::arg("b"), pybind11::arg("c") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Triangle_double_t const &o){ return new PyCallBack_ROL_Triangle_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Triangle<double> const &o){ return new ROL::Triangle<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Triangle<double>::*)(const double) const) &ROL::Triangle<double>::evaluatePDF, "C++: ROL::Triangle<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Triangle<double>::*)(const double) const) &ROL::Triangle<double>::evaluateCDF, "C++: ROL::Triangle<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Triangle<double>::*)(const double) const) &ROL::Triangle<double>::integrateCDF, "C++: ROL::Triangle<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Triangle<double>::*)(const double) const) &ROL::Triangle<double>::invertCDF, "C++: ROL::Triangle<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Triangle<double>::*)(const unsigned long) const) &ROL::Triangle<double>::moment, "C++: ROL::Triangle<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Triangle<double>::*)() const) &ROL::Triangle<double>::lowerBound, "C++: ROL::Triangle<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Triangle<double>::*)() const) &ROL::Triangle<double>::upperBound, "C++: ROL::Triangle<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Triangle<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Triangle<double>::*)(std::ostream &) const) &ROL::Triangle<double>::test, "C++: ROL::Triangle<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Triangle<double> & (ROL::Triangle<double>::*)(const class ROL::Triangle<double> &)) &ROL::Triangle<double>::operator=, "C++: ROL::Triangle<double>::operator=(const class ROL::Triangle<double> &) --> class ROL::Triangle<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Parabolic file:ROL_Parabolic.hpp line:53
		pybind11::class_<ROL::Parabolic<double>, Teuchos::RCP<ROL::Parabolic<double>>, PyCallBack_ROL_Parabolic_double_t, ROL::Distribution<double>> cl(M("ROL"), "Parabolic_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Parabolic<double>(); }, [](){ return new PyCallBack_ROL_Parabolic_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Parabolic<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Parabolic_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("a"), pybind11::arg("b") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Parabolic_double_t const &o){ return new PyCallBack_ROL_Parabolic_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Parabolic<double> const &o){ return new ROL::Parabolic<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Parabolic<double>::*)(const double) const) &ROL::Parabolic<double>::evaluatePDF, "C++: ROL::Parabolic<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Parabolic<double>::*)(const double) const) &ROL::Parabolic<double>::evaluateCDF, "C++: ROL::Parabolic<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Parabolic<double>::*)(const double) const) &ROL::Parabolic<double>::integrateCDF, "C++: ROL::Parabolic<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Parabolic<double>::*)(const double) const) &ROL::Parabolic<double>::invertCDF, "C++: ROL::Parabolic<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Parabolic<double>::*)(const unsigned long) const) &ROL::Parabolic<double>::moment, "C++: ROL::Parabolic<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Parabolic<double>::*)() const) &ROL::Parabolic<double>::lowerBound, "C++: ROL::Parabolic<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Parabolic<double>::*)() const) &ROL::Parabolic<double>::upperBound, "C++: ROL::Parabolic<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Parabolic<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Parabolic<double>::*)(std::ostream &) const) &ROL::Parabolic<double>::test, "C++: ROL::Parabolic<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Parabolic<double> & (ROL::Parabolic<double>::*)(const class ROL::Parabolic<double> &)) &ROL::Parabolic<double>::operator=, "C++: ROL::Parabolic<double>::operator=(const class ROL::Parabolic<double> &) --> class ROL::Parabolic<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::RaisedCosine file:ROL_RaisedCosine.hpp line:53
		pybind11::class_<ROL::RaisedCosine<double>, Teuchos::RCP<ROL::RaisedCosine<double>>, PyCallBack_ROL_RaisedCosine_double_t, ROL::Distribution<double>> cl(M("ROL"), "RaisedCosine_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::RaisedCosine<double>(); }, [](){ return new PyCallBack_ROL_RaisedCosine_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::RaisedCosine<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_RaisedCosine_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("mean"), pybind11::arg("var") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_RaisedCosine_double_t const &o){ return new PyCallBack_ROL_RaisedCosine_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::RaisedCosine<double> const &o){ return new ROL::RaisedCosine<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::RaisedCosine<double>::*)(const double) const) &ROL::RaisedCosine<double>::evaluatePDF, "C++: ROL::RaisedCosine<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::RaisedCosine<double>::*)(const double) const) &ROL::RaisedCosine<double>::evaluateCDF, "C++: ROL::RaisedCosine<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::RaisedCosine<double>::*)(const double) const) &ROL::RaisedCosine<double>::integrateCDF, "C++: ROL::RaisedCosine<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::RaisedCosine<double>::*)(const double) const) &ROL::RaisedCosine<double>::invertCDF, "C++: ROL::RaisedCosine<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::RaisedCosine<double>::*)(const unsigned long) const) &ROL::RaisedCosine<double>::moment, "C++: ROL::RaisedCosine<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::RaisedCosine<double>::*)() const) &ROL::RaisedCosine<double>::lowerBound, "C++: ROL::RaisedCosine<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::RaisedCosine<double>::*)() const) &ROL::RaisedCosine<double>::upperBound, "C++: ROL::RaisedCosine<double>::upperBound() const --> double");
		cl.def("test", [](ROL::RaisedCosine<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::RaisedCosine<double>::*)(std::ostream &) const) &ROL::RaisedCosine<double>::test, "C++: ROL::RaisedCosine<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::RaisedCosine<double> & (ROL::RaisedCosine<double>::*)(const class ROL::RaisedCosine<double> &)) &ROL::RaisedCosine<double>::operator=, "C++: ROL::RaisedCosine<double>::operator=(const class ROL::RaisedCosine<double> &) --> class ROL::RaisedCosine<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Laplace file:ROL_Laplace.hpp line:53
		pybind11::class_<ROL::Laplace<double>, Teuchos::RCP<ROL::Laplace<double>>, PyCallBack_ROL_Laplace_double_t, ROL::Distribution<double>> cl(M("ROL"), "Laplace_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Laplace<double>(); }, [](){ return new PyCallBack_ROL_Laplace_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Laplace<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Laplace_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("mean"), pybind11::arg("scale") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Laplace_double_t const &o){ return new PyCallBack_ROL_Laplace_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Laplace<double> const &o){ return new ROL::Laplace<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Laplace<double>::*)(const double) const) &ROL::Laplace<double>::evaluatePDF, "C++: ROL::Laplace<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Laplace<double>::*)(const double) const) &ROL::Laplace<double>::evaluateCDF, "C++: ROL::Laplace<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Laplace<double>::*)(const double) const) &ROL::Laplace<double>::integrateCDF, "C++: ROL::Laplace<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Laplace<double>::*)(const double) const) &ROL::Laplace<double>::invertCDF, "C++: ROL::Laplace<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Laplace<double>::*)(const unsigned long) const) &ROL::Laplace<double>::moment, "C++: ROL::Laplace<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Laplace<double>::*)() const) &ROL::Laplace<double>::lowerBound, "C++: ROL::Laplace<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Laplace<double>::*)() const) &ROL::Laplace<double>::upperBound, "C++: ROL::Laplace<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Laplace<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Laplace<double>::*)(std::ostream &) const) &ROL::Laplace<double>::test, "C++: ROL::Laplace<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Laplace<double> & (ROL::Laplace<double>::*)(const class ROL::Laplace<double> &)) &ROL::Laplace<double>::operator=, "C++: ROL::Laplace<double>::operator=(const class ROL::Laplace<double> &) --> class ROL::Laplace<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Cauchy file:ROL_Cauchy.hpp line:53
		pybind11::class_<ROL::Cauchy<double>, Teuchos::RCP<ROL::Cauchy<double>>, PyCallBack_ROL_Cauchy_double_t, ROL::Distribution<double>> cl(M("ROL"), "Cauchy_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Cauchy<double>(); }, [](){ return new PyCallBack_ROL_Cauchy_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Cauchy<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Cauchy_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("loc"), pybind11::arg("scale") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Cauchy_double_t const &o){ return new PyCallBack_ROL_Cauchy_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Cauchy<double> const &o){ return new ROL::Cauchy<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Cauchy<double>::*)(const double) const) &ROL::Cauchy<double>::evaluatePDF, "C++: ROL::Cauchy<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Cauchy<double>::*)(const double) const) &ROL::Cauchy<double>::evaluateCDF, "C++: ROL::Cauchy<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Cauchy<double>::*)(const double) const) &ROL::Cauchy<double>::integrateCDF, "C++: ROL::Cauchy<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Cauchy<double>::*)(const double) const) &ROL::Cauchy<double>::invertCDF, "C++: ROL::Cauchy<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Cauchy<double>::*)(const unsigned long) const) &ROL::Cauchy<double>::moment, "C++: ROL::Cauchy<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Cauchy<double>::*)() const) &ROL::Cauchy<double>::lowerBound, "C++: ROL::Cauchy<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Cauchy<double>::*)() const) &ROL::Cauchy<double>::upperBound, "C++: ROL::Cauchy<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Cauchy<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Cauchy<double>::*)(std::ostream &) const) &ROL::Cauchy<double>::test, "C++: ROL::Cauchy<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Cauchy<double> & (ROL::Cauchy<double>::*)(const class ROL::Cauchy<double> &)) &ROL::Cauchy<double>::operator=, "C++: ROL::Cauchy<double>::operator=(const class ROL::Cauchy<double> &) --> class ROL::Cauchy<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Smale file:ROL_Smale.hpp line:53
		pybind11::class_<ROL::Smale<double>, Teuchos::RCP<ROL::Smale<double>>, PyCallBack_ROL_Smale_double_t, ROL::Distribution<double>> cl(M("ROL"), "Smale_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Smale<double>(); }, [](){ return new PyCallBack_ROL_Smale_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Smale<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Smale_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("a"), pybind11::arg("b") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Smale_double_t const &o){ return new PyCallBack_ROL_Smale_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Smale<double> const &o){ return new ROL::Smale<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Smale<double>::*)(const double) const) &ROL::Smale<double>::evaluatePDF, "C++: ROL::Smale<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Smale<double>::*)(const double) const) &ROL::Smale<double>::evaluateCDF, "C++: ROL::Smale<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Smale<double>::*)(const double) const) &ROL::Smale<double>::integrateCDF, "C++: ROL::Smale<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Smale<double>::*)(const double) const) &ROL::Smale<double>::invertCDF, "C++: ROL::Smale<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Smale<double>::*)(const unsigned long) const) &ROL::Smale<double>::moment, "C++: ROL::Smale<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Smale<double>::*)() const) &ROL::Smale<double>::lowerBound, "C++: ROL::Smale<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Smale<double>::*)() const) &ROL::Smale<double>::upperBound, "C++: ROL::Smale<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Smale<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Smale<double>::*)(std::ostream &) const) &ROL::Smale<double>::test, "C++: ROL::Smale<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Smale<double> & (ROL::Smale<double>::*)(const class ROL::Smale<double> &)) &ROL::Smale<double>::operator=, "C++: ROL::Smale<double>::operator=(const class ROL::Smale<double> &) --> class ROL::Smale<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Arcsine file:ROL_Arcsine.hpp line:53
		pybind11::class_<ROL::Arcsine<double>, Teuchos::RCP<ROL::Arcsine<double>>, PyCallBack_ROL_Arcsine_double_t, ROL::Distribution<double>> cl(M("ROL"), "Arcsine_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Arcsine<double>(); }, [](){ return new PyCallBack_ROL_Arcsine_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Arcsine<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Arcsine_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("a"), pybind11::arg("b") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Arcsine_double_t const &o){ return new PyCallBack_ROL_Arcsine_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Arcsine<double> const &o){ return new ROL::Arcsine<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Arcsine<double>::*)(const double) const) &ROL::Arcsine<double>::evaluatePDF, "C++: ROL::Arcsine<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Arcsine<double>::*)(const double) const) &ROL::Arcsine<double>::evaluateCDF, "C++: ROL::Arcsine<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Arcsine<double>::*)(const double) const) &ROL::Arcsine<double>::integrateCDF, "C++: ROL::Arcsine<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Arcsine<double>::*)(const double) const) &ROL::Arcsine<double>::invertCDF, "C++: ROL::Arcsine<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Arcsine<double>::*)(const unsigned long) const) &ROL::Arcsine<double>::moment, "C++: ROL::Arcsine<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Arcsine<double>::*)() const) &ROL::Arcsine<double>::lowerBound, "C++: ROL::Arcsine<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Arcsine<double>::*)() const) &ROL::Arcsine<double>::upperBound, "C++: ROL::Arcsine<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Arcsine<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Arcsine<double>::*)(std::ostream &) const) &ROL::Arcsine<double>::test, "C++: ROL::Arcsine<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Arcsine<double> & (ROL::Arcsine<double>::*)(const class ROL::Arcsine<double> &)) &ROL::Arcsine<double>::operator=, "C++: ROL::Arcsine<double>::operator=(const class ROL::Arcsine<double> &) --> class ROL::Arcsine<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Kumaraswamy file:ROL_Kumaraswamy.hpp line:55
		pybind11::class_<ROL::Kumaraswamy<double>, Teuchos::RCP<ROL::Kumaraswamy<double>>, PyCallBack_ROL_Kumaraswamy_double_t, ROL::Distribution<double>> cl(M("ROL"), "Kumaraswamy_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Kumaraswamy<double>(); }, [](){ return new PyCallBack_ROL_Kumaraswamy_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Kumaraswamy<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Kumaraswamy_double_t(a0); } ), "doc");
		cl.def( pybind11::init( [](const double & a0, const double & a1){ return new ROL::Kumaraswamy<double>(a0, a1); }, [](const double & a0, const double & a1){ return new PyCallBack_ROL_Kumaraswamy_double_t(a0, a1); } ), "doc");
		cl.def( pybind11::init( [](const double & a0, const double & a1, const double & a2){ return new ROL::Kumaraswamy<double>(a0, a1, a2); }, [](const double & a0, const double & a1, const double & a2){ return new PyCallBack_ROL_Kumaraswamy_double_t(a0, a1, a2); } ), "doc");
		cl.def( pybind11::init<const double, const double, const double, const double>(), pybind11::arg("a"), pybind11::arg("b"), pybind11::arg("exp1"), pybind11::arg("exp2") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Kumaraswamy_double_t const &o){ return new PyCallBack_ROL_Kumaraswamy_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Kumaraswamy<double> const &o){ return new ROL::Kumaraswamy<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Kumaraswamy<double>::*)(const double) const) &ROL::Kumaraswamy<double>::evaluatePDF, "C++: ROL::Kumaraswamy<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Kumaraswamy<double>::*)(const double) const) &ROL::Kumaraswamy<double>::evaluateCDF, "C++: ROL::Kumaraswamy<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Kumaraswamy<double>::*)(const double) const) &ROL::Kumaraswamy<double>::integrateCDF, "C++: ROL::Kumaraswamy<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Kumaraswamy<double>::*)(const double) const) &ROL::Kumaraswamy<double>::invertCDF, "C++: ROL::Kumaraswamy<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Kumaraswamy<double>::*)(const unsigned long) const) &ROL::Kumaraswamy<double>::moment, "C++: ROL::Kumaraswamy<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Kumaraswamy<double>::*)() const) &ROL::Kumaraswamy<double>::lowerBound, "C++: ROL::Kumaraswamy<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Kumaraswamy<double>::*)() const) &ROL::Kumaraswamy<double>::upperBound, "C++: ROL::Kumaraswamy<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Kumaraswamy<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Kumaraswamy<double>::*)(std::ostream &) const) &ROL::Kumaraswamy<double>::test, "C++: ROL::Kumaraswamy<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Kumaraswamy<double> & (ROL::Kumaraswamy<double>::*)(const class ROL::Kumaraswamy<double> &)) &ROL::Kumaraswamy<double>::operator=, "C++: ROL::Kumaraswamy<double>::operator=(const class ROL::Kumaraswamy<double> &) --> class ROL::Kumaraswamy<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Exponential file:ROL_Exponential.hpp line:53
		pybind11::class_<ROL::Exponential<double>, Teuchos::RCP<ROL::Exponential<double>>, PyCallBack_ROL_Exponential_double_t, ROL::Distribution<double>> cl(M("ROL"), "Exponential_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Exponential<double>(); }, [](){ return new PyCallBack_ROL_Exponential_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Exponential<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Exponential_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("loc"), pybind11::arg("scale") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Exponential_double_t const &o){ return new PyCallBack_ROL_Exponential_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Exponential<double> const &o){ return new ROL::Exponential<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Exponential<double>::*)(const double) const) &ROL::Exponential<double>::evaluatePDF, "C++: ROL::Exponential<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Exponential<double>::*)(const double) const) &ROL::Exponential<double>::evaluateCDF, "C++: ROL::Exponential<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Exponential<double>::*)(const double) const) &ROL::Exponential<double>::integrateCDF, "C++: ROL::Exponential<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Exponential<double>::*)(const double) const) &ROL::Exponential<double>::invertCDF, "C++: ROL::Exponential<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Exponential<double>::*)(const unsigned long) const) &ROL::Exponential<double>::moment, "C++: ROL::Exponential<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Exponential<double>::*)() const) &ROL::Exponential<double>::lowerBound, "C++: ROL::Exponential<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Exponential<double>::*)() const) &ROL::Exponential<double>::upperBound, "C++: ROL::Exponential<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Exponential<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Exponential<double>::*)(std::ostream &) const) &ROL::Exponential<double>::test, "C++: ROL::Exponential<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Exponential<double> & (ROL::Exponential<double>::*)(const class ROL::Exponential<double> &)) &ROL::Exponential<double>::operator=, "C++: ROL::Exponential<double>::operator=(const class ROL::Exponential<double> &) --> class ROL::Exponential<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::TruncatedExponential file:ROL_TruncatedExponential.hpp line:53
		pybind11::class_<ROL::TruncatedExponential<double>, Teuchos::RCP<ROL::TruncatedExponential<double>>, PyCallBack_ROL_TruncatedExponential_double_t, ROL::Distribution<double>> cl(M("ROL"), "TruncatedExponential_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::TruncatedExponential<double>(); }, [](){ return new PyCallBack_ROL_TruncatedExponential_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::TruncatedExponential<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_TruncatedExponential_double_t(a0); } ), "doc");
		cl.def( pybind11::init( [](const double & a0, const double & a1){ return new ROL::TruncatedExponential<double>(a0, a1); }, [](const double & a0, const double & a1){ return new PyCallBack_ROL_TruncatedExponential_double_t(a0, a1); } ), "doc");
		cl.def( pybind11::init<const double, const double, const double>(), pybind11::arg("a"), pybind11::arg("b"), pybind11::arg("scale") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_TruncatedExponential_double_t const &o){ return new PyCallBack_ROL_TruncatedExponential_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::TruncatedExponential<double> const &o){ return new ROL::TruncatedExponential<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::TruncatedExponential<double>::*)(const double) const) &ROL::TruncatedExponential<double>::evaluatePDF, "C++: ROL::TruncatedExponential<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::TruncatedExponential<double>::*)(const double) const) &ROL::TruncatedExponential<double>::evaluateCDF, "C++: ROL::TruncatedExponential<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::TruncatedExponential<double>::*)(const double) const) &ROL::TruncatedExponential<double>::integrateCDF, "C++: ROL::TruncatedExponential<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::TruncatedExponential<double>::*)(const double) const) &ROL::TruncatedExponential<double>::invertCDF, "C++: ROL::TruncatedExponential<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::TruncatedExponential<double>::*)(const unsigned long) const) &ROL::TruncatedExponential<double>::moment, "C++: ROL::TruncatedExponential<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::TruncatedExponential<double>::*)() const) &ROL::TruncatedExponential<double>::lowerBound, "C++: ROL::TruncatedExponential<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::TruncatedExponential<double>::*)() const) &ROL::TruncatedExponential<double>::upperBound, "C++: ROL::TruncatedExponential<double>::upperBound() const --> double");
		cl.def("test", [](ROL::TruncatedExponential<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::TruncatedExponential<double>::*)(std::ostream &) const) &ROL::TruncatedExponential<double>::test, "C++: ROL::TruncatedExponential<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::TruncatedExponential<double> & (ROL::TruncatedExponential<double>::*)(const class ROL::TruncatedExponential<double> &)) &ROL::TruncatedExponential<double>::operator=, "C++: ROL::TruncatedExponential<double>::operator=(const class ROL::TruncatedExponential<double> &) --> class ROL::TruncatedExponential<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Gamma file:ROL_Gamma.hpp line:55
		pybind11::class_<ROL::Gamma<double>, Teuchos::RCP<ROL::Gamma<double>>, PyCallBack_ROL_Gamma_double_t, ROL::Distribution<double>> cl(M("ROL"), "Gamma_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Gamma<double>(); }, [](){ return new PyCallBack_ROL_Gamma_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Gamma<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Gamma_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("shape"), pybind11::arg("scale") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Gamma_double_t const &o){ return new PyCallBack_ROL_Gamma_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Gamma<double> const &o){ return new ROL::Gamma<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Gamma<double>::*)(const double) const) &ROL::Gamma<double>::evaluatePDF, "C++: ROL::Gamma<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Gamma<double>::*)(const double) const) &ROL::Gamma<double>::evaluateCDF, "C++: ROL::Gamma<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Gamma<double>::*)(const double) const) &ROL::Gamma<double>::integrateCDF, "C++: ROL::Gamma<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Gamma<double>::*)(const double) const) &ROL::Gamma<double>::invertCDF, "C++: ROL::Gamma<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Gamma<double>::*)(const unsigned long) const) &ROL::Gamma<double>::moment, "C++: ROL::Gamma<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Gamma<double>::*)() const) &ROL::Gamma<double>::lowerBound, "C++: ROL::Gamma<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Gamma<double>::*)() const) &ROL::Gamma<double>::upperBound, "C++: ROL::Gamma<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Gamma<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Gamma<double>::*)(std::ostream &) const) &ROL::Gamma<double>::test, "C++: ROL::Gamma<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Gamma<double> & (ROL::Gamma<double>::*)(const class ROL::Gamma<double> &)) &ROL::Gamma<double>::operator=, "C++: ROL::Gamma<double>::operator=(const class ROL::Gamma<double> &) --> class ROL::Gamma<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Beta file:ROL_Beta.hpp line:55
		pybind11::class_<ROL::Beta<double>, Teuchos::RCP<ROL::Beta<double>>, PyCallBack_ROL_Beta_double_t, ROL::Distribution<double>> cl(M("ROL"), "Beta_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::Beta<double>(); }, [](){ return new PyCallBack_ROL_Beta_double_t(); } ), "doc");
		cl.def( pybind11::init( [](const double & a0){ return new ROL::Beta<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_Beta_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("shape1"), pybind11::arg("shape2") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Beta_double_t const &o){ return new PyCallBack_ROL_Beta_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Beta<double> const &o){ return new ROL::Beta<double>(o); } ) );
		cl.def("evaluatePDF", (double (ROL::Beta<double>::*)(const double) const) &ROL::Beta<double>::evaluatePDF, "C++: ROL::Beta<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Beta<double>::*)(const double) const) &ROL::Beta<double>::evaluateCDF, "C++: ROL::Beta<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Beta<double>::*)(const double) const) &ROL::Beta<double>::integrateCDF, "C++: ROL::Beta<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Beta<double>::*)(const double) const) &ROL::Beta<double>::invertCDF, "C++: ROL::Beta<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Beta<double>::*)(const unsigned long) const) &ROL::Beta<double>::moment, "C++: ROL::Beta<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Beta<double>::*)() const) &ROL::Beta<double>::lowerBound, "C++: ROL::Beta<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Beta<double>::*)() const) &ROL::Beta<double>::upperBound, "C++: ROL::Beta<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Beta<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Beta<double>::*)(std::ostream &) const) &ROL::Beta<double>::test, "C++: ROL::Beta<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Beta<double> & (ROL::Beta<double>::*)(const class ROL::Beta<double> &)) &ROL::Beta<double>::operator=, "C++: ROL::Beta<double>::operator=(const class ROL::Beta<double> &) --> class ROL::Beta<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluatePDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluatePDF, "C++: ROL::Distribution<double>::evaluatePDF(const double) const --> double", pybind11::arg("input"));
		cl.def("evaluateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::evaluateCDF, "C++: ROL::Distribution<double>::evaluateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("integrateCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::integrateCDF, "C++: ROL::Distribution<double>::integrateCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("invertCDF", (double (ROL::Distribution<double>::*)(const double) const) &ROL::Distribution<double>::invertCDF, "C++: ROL::Distribution<double>::invertCDF(const double) const --> double", pybind11::arg("input"));
		cl.def("moment", (double (ROL::Distribution<double>::*)(const unsigned long) const) &ROL::Distribution<double>::moment, "C++: ROL::Distribution<double>::moment(const unsigned long) const --> double", pybind11::arg("m"));
		cl.def("lowerBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::lowerBound, "C++: ROL::Distribution<double>::lowerBound() const --> double");
		cl.def("upperBound", (double (ROL::Distribution<double>::*)() const) &ROL::Distribution<double>::upperBound, "C++: ROL::Distribution<double>::upperBound() const --> double");
		cl.def("test", [](ROL::Distribution<double> const &o) -> void { return o.test(); }, "");
		cl.def("test", (void (ROL::Distribution<double>::*)(std::ostream &) const) &ROL::Distribution<double>::test, "C++: ROL::Distribution<double>::test(std::ostream &) const --> void", pybind11::arg("outStream"));
		cl.def("assign", (class ROL::Distribution<double> & (ROL::Distribution<double>::*)(const class ROL::Distribution<double> &)) &ROL::Distribution<double>::operator=, "C++: ROL::Distribution<double>::operator=(const class ROL::Distribution<double> &) --> class ROL::Distribution<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	// ROL::EDistribution file:ROL_DistributionFactory.hpp line:69
	pybind11::enum_<ROL::EDistribution>(M("ROL"), "EDistribution", pybind11::arithmetic(), "", pybind11::module_local())
		.value("DISTRIBUTION_ARCSINE", ROL::DISTRIBUTION_ARCSINE)
		.value("DISTRIBUTION_BETA", ROL::DISTRIBUTION_BETA)
		.value("DISTRIBUTION_CAUCHY", ROL::DISTRIBUTION_CAUCHY)
		.value("DISTRIBUTION_DIRAC", ROL::DISTRIBUTION_DIRAC)
		.value("DISTRIBUTION_EXPONENTIAL", ROL::DISTRIBUTION_EXPONENTIAL)
		.value("DISTRIBUTION_GAMMA", ROL::DISTRIBUTION_GAMMA)
		.value("DISTRIBUTION_GAUSSIAN", ROL::DISTRIBUTION_GAUSSIAN)
		.value("DISTRIBUTION_KUMARASWAMY", ROL::DISTRIBUTION_KUMARASWAMY)
		.value("DISTRIBUTION_LAPLACE", ROL::DISTRIBUTION_LAPLACE)
		.value("DISTRIBUTION_LOGISTIC", ROL::DISTRIBUTION_LOGISTIC)
		.value("DISTRIBUTION_PARABOLIC", ROL::DISTRIBUTION_PARABOLIC)
		.value("DISTRIBUTION_RAISEDCOSINE", ROL::DISTRIBUTION_RAISEDCOSINE)
		.value("DISTRIBUTION_SMALE", ROL::DISTRIBUTION_SMALE)
		.value("DISTRIBUTION_TRIANGLE", ROL::DISTRIBUTION_TRIANGLE)
		.value("DISTRIBUTION_TRUNCATEDEXPONENTIAL", ROL::DISTRIBUTION_TRUNCATEDEXPONENTIAL)
		.value("DISTRIBUTION_TRUNCATEDGAUSSIAN", ROL::DISTRIBUTION_TRUNCATEDGAUSSIAN)
		.value("DISTRIBUTION_UNIFORM", ROL::DISTRIBUTION_UNIFORM)
		.value("DISTRIBUTION_LAST", ROL::DISTRIBUTION_LAST)
		.export_values();

;

	// ROL::EDistributionToString(enum ROL::EDistribution) file:ROL_DistributionFactory.hpp line:90
	M("ROL").def("EDistributionToString", (std::string (*)(enum ROL::EDistribution)) &ROL::EDistributionToString, "C++: ROL::EDistributionToString(enum ROL::EDistribution) --> std::string", pybind11::arg("ed"));

	// ROL::isValidDistribution(enum ROL::EDistribution) file:ROL_DistributionFactory.hpp line:116
	M("ROL").def("isValidDistribution", (int (*)(enum ROL::EDistribution)) &ROL::isValidDistribution, "C++: ROL::isValidDistribution(enum ROL::EDistribution) --> int", pybind11::arg("ed"));

	// ROL::StringToEDistribution(std::string) file:ROL_DistributionFactory.hpp line:156
	M("ROL").def("StringToEDistribution", (enum ROL::EDistribution (*)(std::string)) &ROL::StringToEDistribution, "C++: ROL::StringToEDistribution(std::string) --> enum ROL::EDistribution", pybind11::arg("s"));

	// ROL::DistributionFactory(class Teuchos::ParameterList &) file:ROL_DistributionFactory.hpp line:167
	M("ROL").def("DistributionFactory", (class Teuchos::RCP<class ROL::Distribution<double> > (*)(class Teuchos::ParameterList &)) &ROL::DistributionFactory<double>, "C++: ROL::DistributionFactory(class Teuchos::ParameterList &) --> class Teuchos::RCP<class ROL::Distribution<double> >", pybind11::arg("parlist"));

	{ // ROL::PlusFunction file:ROL_PlusFunction.hpp line:55
		pybind11::class_<ROL::PlusFunction<double>, Teuchos::RCP<ROL::PlusFunction<double>>, PyCallBack_ROL_PlusFunction_double_t, ROL::PositiveFunction<double>> cl(M("ROL"), "PlusFunction_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](class Teuchos::RCP<class ROL::Distribution<double> > & a0){ return new ROL::PlusFunction<double>(a0); }, [](class Teuchos::RCP<class ROL::Distribution<double> > & a0){ return new PyCallBack_ROL_PlusFunction_double_t(a0); } ), "doc");
		cl.def( pybind11::init<class Teuchos::RCP<class ROL::Distribution<double> > &, double>(), pybind11::arg("dist"), pybind11::arg("param") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_PlusFunction_double_t const &o){ return new PyCallBack_ROL_PlusFunction_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::PlusFunction<double> const &o){ return new ROL::PlusFunction<double>(o); } ) );
		cl.def("evaluate", (double (ROL::PlusFunction<double>::*)(double, int)) &ROL::PlusFunction<double>::evaluate, "C++: ROL::PlusFunction<double>::evaluate(double, int) --> double", pybind11::arg("input"), pybind11::arg("deriv"));
		cl.def("test", (void (ROL::PlusFunction<double>::*)(double)) &ROL::PlusFunction<double>::test, "C++: ROL::PlusFunction<double>::test(double) --> void", pybind11::arg("x"));
		cl.def("assign", (class ROL::PlusFunction<double> & (ROL::PlusFunction<double>::*)(const class ROL::PlusFunction<double> &)) &ROL::PlusFunction<double>::operator=, "C++: ROL::PlusFunction<double>::operator=(const class ROL::PlusFunction<double> &) --> class ROL::PlusFunction<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluate", (double (ROL::PositiveFunction<double>::*)(double, int)) &ROL::PositiveFunction<double>::evaluate, "C++: ROL::PositiveFunction<double>::evaluate(double, int) --> double", pybind11::arg("input"), pybind11::arg("deriv"));
		cl.def("assign", (class ROL::PositiveFunction<double> & (ROL::PositiveFunction<double>::*)(const class ROL::PositiveFunction<double> &)) &ROL::PositiveFunction<double>::operator=, "C++: ROL::PositiveFunction<double>::operator=(const class ROL::PositiveFunction<double> &) --> class ROL::PositiveFunction<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::CVaR file:ROL_CVaR.hpp line:78
		pybind11::class_<ROL::CVaR<double>, Teuchos::RCP<ROL::CVaR<double>>, PyCallBack_ROL_CVaR_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "CVaR_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double, const double, const class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("prob"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_CVaR_double_t const &o){ return new PyCallBack_ROL_CVaR_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::CVaR<double> const &o){ return new ROL::CVaR<double>(o); } ) );
		cl.def("updateValue", (void (ROL::CVaR<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::CVaR<double>::updateValue, "C++: ROL::CVaR<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::CVaR<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::CVaR<double>::updateGradient, "C++: ROL::CVaR<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::CVaR<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::CVaR<double>::updateHessVec, "C++: ROL::CVaR<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::CVaR<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::CVaR<double>::getValue, "C++: ROL::CVaR<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::CVaR<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::CVaR<double>::getGradient, "C++: ROL::CVaR<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::CVaR<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::CVaR<double>::getHessVec, "C++: ROL::CVaR<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::CVaR<double> & (ROL::CVaR<double>::*)(const class ROL::CVaR<double> &)) &ROL::CVaR<double>::operator=, "C++: ROL::CVaR<double>::operator=(const class ROL::CVaR<double> &) --> class ROL::CVaR<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::MixedCVaR file:ROL_MixedCVaR.hpp line:88
		pybind11::class_<ROL::MixedCVaR<double>, Teuchos::RCP<ROL::MixedCVaR<double>>, PyCallBack_ROL_MixedCVaR_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "MixedCVaR_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init<const class std::vector<double, class std::allocator<double> > &, const class std::vector<double, class std::allocator<double> > &, const class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("prob"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init( [](PyCallBack_ROL_MixedCVaR_double_t const &o){ return new PyCallBack_ROL_MixedCVaR_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::MixedCVaR<double> const &o){ return new ROL::MixedCVaR<double>(o); } ) );
		cl.def("initialize", (void (ROL::MixedCVaR<double>::*)(const class ROL::Vector<double> &)) &ROL::MixedCVaR<double>::initialize, "C++: ROL::MixedCVaR<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("computeStatistic", (double (ROL::MixedCVaR<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::MixedCVaR<double>::computeStatistic, "C++: ROL::MixedCVaR<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::MixedCVaR<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MixedCVaR<double>::updateValue, "C++: ROL::MixedCVaR<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::MixedCVaR<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MixedCVaR<double>::getValue, "C++: ROL::MixedCVaR<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::MixedCVaR<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MixedCVaR<double>::updateGradient, "C++: ROL::MixedCVaR<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::MixedCVaR<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MixedCVaR<double>::getGradient, "C++: ROL::MixedCVaR<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::MixedCVaR<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MixedCVaR<double>::updateHessVec, "C++: ROL::MixedCVaR<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::MixedCVaR<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MixedCVaR<double>::getHessVec, "C++: ROL::MixedCVaR<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::MixedCVaR<double> & (ROL::MixedCVaR<double>::*)(const class ROL::MixedCVaR<double> &)) &ROL::MixedCVaR<double>::operator=, "C++: ROL::MixedCVaR<double>::operator=(const class ROL::MixedCVaR<double> &) --> class ROL::MixedCVaR<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::SpectralRisk file:ROL_SpectralRisk.hpp line:89
		pybind11::class_<ROL::SpectralRisk<double>, Teuchos::RCP<ROL::SpectralRisk<double>>, PyCallBack_ROL_SpectralRisk_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "SpectralRisk_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::SpectralRisk<double>(); }, [](){ return new PyCallBack_ROL_SpectralRisk_double_t(); } ) );
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Distribution<double> > &, const int, const class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("dist"), pybind11::arg("nQuad"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init<const class std::vector<double, class std::allocator<double> > &, const class std::vector<double, class std::allocator<double> > &, const class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("pts"), pybind11::arg("wts"), pybind11::arg("pf") );

		cl.def( pybind11::init( [](PyCallBack_ROL_SpectralRisk_double_t const &o){ return new PyCallBack_ROL_SpectralRisk_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::SpectralRisk<double> const &o){ return new ROL::SpectralRisk<double>(o); } ) );
		cl.def("setStorage", (void (ROL::SpectralRisk<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::SpectralRisk<double>::setStorage, "C++: ROL::SpectralRisk<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::SpectralRisk<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::SpectralRisk<double>::setHessVecStorage, "C++: ROL::SpectralRisk<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("setSample", (void (ROL::SpectralRisk<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::SpectralRisk<double>::setSample, "C++: ROL::SpectralRisk<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("resetStorage", [](ROL::SpectralRisk<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::SpectralRisk<double>::*)(bool)) &ROL::SpectralRisk<double>::resetStorage, "C++: ROL::SpectralRisk<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::SpectralRisk<double>::*)(enum ROL::UpdateType)) &ROL::SpectralRisk<double>::resetStorage, "C++: ROL::SpectralRisk<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::SpectralRisk<double>::*)(const class ROL::Vector<double> &)) &ROL::SpectralRisk<double>::initialize, "C++: ROL::SpectralRisk<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("computeStatistic", (double (ROL::SpectralRisk<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::SpectralRisk<double>::computeStatistic, "C++: ROL::SpectralRisk<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::SpectralRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SpectralRisk<double>::updateValue, "C++: ROL::SpectralRisk<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::SpectralRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SpectralRisk<double>::updateGradient, "C++: ROL::SpectralRisk<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::SpectralRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SpectralRisk<double>::updateHessVec, "C++: ROL::SpectralRisk<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::SpectralRisk<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SpectralRisk<double>::getValue, "C++: ROL::SpectralRisk<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::SpectralRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SpectralRisk<double>::getGradient, "C++: ROL::SpectralRisk<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::SpectralRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SpectralRisk<double>::getHessVec, "C++: ROL::SpectralRisk<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::SpectralRisk<double> & (ROL::SpectralRisk<double>::*)(const class ROL::SpectralRisk<double> &)) &ROL::SpectralRisk<double>::operator=, "C++: ROL::SpectralRisk<double>::operator=(const class ROL::SpectralRisk<double> &) --> class ROL::SpectralRisk<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Quadrature1D file:ROL_Quadrature1D.hpp line:52
		pybind11::class_<ROL::Quadrature1D<double>, Teuchos::RCP<ROL::Quadrature1D<double>>> cl(M("ROL"), "Quadrature1D_double_t", "", pybind11::module_local());
		cl.def("get", (void (ROL::Quadrature1D<double>::*)(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const) &ROL::Quadrature1D<double>::get, "C++: ROL::Quadrature1D<double>::get(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const --> void", pybind11::arg("pts"), pybind11::arg("wts"));
		cl.def("assign", (class ROL::Quadrature1D<double> & (ROL::Quadrature1D<double>::*)(const class ROL::Quadrature1D<double> &)) &ROL::Quadrature1D<double>::operator=, "C++: ROL::Quadrature1D<double>::operator=(const class ROL::Quadrature1D<double> &) --> class ROL::Quadrature1D<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::GaussLegendreQuadrature file:ROL_GaussLegendreQuadrature.hpp line:52
		pybind11::class_<ROL::GaussLegendreQuadrature<double>, Teuchos::RCP<ROL::GaussLegendreQuadrature<double>>, PyCallBack_ROL_GaussLegendreQuadrature_double_t, ROL::Quadrature1D<double>> cl(M("ROL"), "GaussLegendreQuadrature_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const int>(), pybind11::arg("nQuad") );

		cl.def( pybind11::init( [](PyCallBack_ROL_GaussLegendreQuadrature_double_t const &o){ return new PyCallBack_ROL_GaussLegendreQuadrature_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::GaussLegendreQuadrature<double> const &o){ return new ROL::GaussLegendreQuadrature<double>(o); } ) );
		cl.def("get", (void (ROL::Quadrature1D<double>::*)(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const) &ROL::Quadrature1D<double>::get, "C++: ROL::Quadrature1D<double>::get(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const --> void", pybind11::arg("pts"), pybind11::arg("wts"));
		cl.def("assign", (class ROL::Quadrature1D<double> & (ROL::Quadrature1D<double>::*)(const class ROL::Quadrature1D<double> &)) &ROL::Quadrature1D<double>::operator=, "C++: ROL::Quadrature1D<double>::operator=(const class ROL::Quadrature1D<double> &) --> class ROL::Quadrature1D<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Fejer2Quadrature file:ROL_Fejer2Quadrature.hpp line:52
		pybind11::class_<ROL::Fejer2Quadrature<double>, Teuchos::RCP<ROL::Fejer2Quadrature<double>>, PyCallBack_ROL_Fejer2Quadrature_double_t, ROL::Quadrature1D<double>> cl(M("ROL"), "Fejer2Quadrature_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const int>(), pybind11::arg("nQuad") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Fejer2Quadrature_double_t const &o){ return new PyCallBack_ROL_Fejer2Quadrature_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Fejer2Quadrature<double> const &o){ return new ROL::Fejer2Quadrature<double>(o); } ) );
		cl.def("get", (void (ROL::Quadrature1D<double>::*)(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const) &ROL::Quadrature1D<double>::get, "C++: ROL::Quadrature1D<double>::get(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const --> void", pybind11::arg("pts"), pybind11::arg("wts"));
		cl.def("assign", (class ROL::Quadrature1D<double> & (ROL::Quadrature1D<double>::*)(const class ROL::Quadrature1D<double> &)) &ROL::Quadrature1D<double>::operator=, "C++: ROL::Quadrature1D<double>::operator=(const class ROL::Quadrature1D<double> &) --> class ROL::Quadrature1D<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::SecondOrderCVaR file:ROL_SecondOrderCVaR.hpp line:85
		pybind11::class_<ROL::SecondOrderCVaR<double>, Teuchos::RCP<ROL::SecondOrderCVaR<double>>, PyCallBack_ROL_SecondOrderCVaR_double_t, ROL::SpectralRisk<double>> cl(M("ROL"), "SecondOrderCVaR_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](const double & a0, const int & a1, const class Teuchos::RCP<class ROL::PlusFunction<double> > & a2){ return new ROL::SecondOrderCVaR<double>(a0, a1, a2); }, [](const double & a0, const int & a1, const class Teuchos::RCP<class ROL::PlusFunction<double> > & a2){ return new PyCallBack_ROL_SecondOrderCVaR_double_t(a0, a1, a2); } ), "doc");
		cl.def( pybind11::init<const double, const int, const class Teuchos::RCP<class ROL::PlusFunction<double> > &, const bool>(), pybind11::arg("alpha"), pybind11::arg("nQuad"), pybind11::arg("pf"), pybind11::arg("useGauss") );

		cl.def( pybind11::init( [](PyCallBack_ROL_SecondOrderCVaR_double_t const &o){ return new PyCallBack_ROL_SecondOrderCVaR_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::SecondOrderCVaR<double> const &o){ return new ROL::SecondOrderCVaR<double>(o); } ) );
		cl.def("assign", (class ROL::SecondOrderCVaR<double> & (ROL::SecondOrderCVaR<double>::*)(const class ROL::SecondOrderCVaR<double> &)) &ROL::SecondOrderCVaR<double>::operator=, "C++: ROL::SecondOrderCVaR<double>::operator=(const class ROL::SecondOrderCVaR<double> &) --> class ROL::SecondOrderCVaR<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("setStorage", (void (ROL::SpectralRisk<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::SpectralRisk<double>::setStorage, "C++: ROL::SpectralRisk<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::SpectralRisk<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::SpectralRisk<double>::setHessVecStorage, "C++: ROL::SpectralRisk<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("setSample", (void (ROL::SpectralRisk<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::SpectralRisk<double>::setSample, "C++: ROL::SpectralRisk<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("resetStorage", [](ROL::SpectralRisk<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::SpectralRisk<double>::*)(bool)) &ROL::SpectralRisk<double>::resetStorage, "C++: ROL::SpectralRisk<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::SpectralRisk<double>::*)(enum ROL::UpdateType)) &ROL::SpectralRisk<double>::resetStorage, "C++: ROL::SpectralRisk<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::SpectralRisk<double>::*)(const class ROL::Vector<double> &)) &ROL::SpectralRisk<double>::initialize, "C++: ROL::SpectralRisk<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("computeStatistic", (double (ROL::SpectralRisk<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::SpectralRisk<double>::computeStatistic, "C++: ROL::SpectralRisk<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::SpectralRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SpectralRisk<double>::updateValue, "C++: ROL::SpectralRisk<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::SpectralRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SpectralRisk<double>::updateGradient, "C++: ROL::SpectralRisk<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::SpectralRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SpectralRisk<double>::updateHessVec, "C++: ROL::SpectralRisk<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::SpectralRisk<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SpectralRisk<double>::getValue, "C++: ROL::SpectralRisk<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::SpectralRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SpectralRisk<double>::getGradient, "C++: ROL::SpectralRisk<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::SpectralRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SpectralRisk<double>::getHessVec, "C++: ROL::SpectralRisk<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::SpectralRisk<double> & (ROL::SpectralRisk<double>::*)(const class ROL::SpectralRisk<double> &)) &ROL::SpectralRisk<double>::operator=, "C++: ROL::SpectralRisk<double>::operator=(const class ROL::SpectralRisk<double> &) --> class ROL::SpectralRisk<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::GaussChebyshev1Quadrature file:ROL_GaussChebyshev1Quadrature.hpp line:52
		pybind11::class_<ROL::GaussChebyshev1Quadrature<double>, Teuchos::RCP<ROL::GaussChebyshev1Quadrature<double>>, PyCallBack_ROL_GaussChebyshev1Quadrature_double_t, ROL::Quadrature1D<double>> cl(M("ROL"), "GaussChebyshev1Quadrature_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const int>(), pybind11::arg("nQuad") );

		cl.def( pybind11::init( [](PyCallBack_ROL_GaussChebyshev1Quadrature_double_t const &o){ return new PyCallBack_ROL_GaussChebyshev1Quadrature_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::GaussChebyshev1Quadrature<double> const &o){ return new ROL::GaussChebyshev1Quadrature<double>(o); } ) );
		cl.def("get", (void (ROL::Quadrature1D<double>::*)(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const) &ROL::Quadrature1D<double>::get, "C++: ROL::Quadrature1D<double>::get(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const --> void", pybind11::arg("pts"), pybind11::arg("wts"));
		cl.def("assign", (class ROL::Quadrature1D<double> & (ROL::Quadrature1D<double>::*)(const class ROL::Quadrature1D<double> &)) &ROL::Quadrature1D<double>::operator=, "C++: ROL::Quadrature1D<double>::operator=(const class ROL::Quadrature1D<double> &) --> class ROL::Quadrature1D<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::GaussChebyshev2Quadrature file:ROL_GaussChebyshev2Quadrature.hpp line:53
		pybind11::class_<ROL::GaussChebyshev2Quadrature<double>, Teuchos::RCP<ROL::GaussChebyshev2Quadrature<double>>, PyCallBack_ROL_GaussChebyshev2Quadrature_double_t, ROL::Quadrature1D<double>> cl(M("ROL"), "GaussChebyshev2Quadrature_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const int>(), pybind11::arg("nQuad") );

		cl.def( pybind11::init( [](PyCallBack_ROL_GaussChebyshev2Quadrature_double_t const &o){ return new PyCallBack_ROL_GaussChebyshev2Quadrature_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::GaussChebyshev2Quadrature<double> const &o){ return new ROL::GaussChebyshev2Quadrature<double>(o); } ) );
		cl.def("get", (void (ROL::Quadrature1D<double>::*)(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const) &ROL::Quadrature1D<double>::get, "C++: ROL::Quadrature1D<double>::get(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const --> void", pybind11::arg("pts"), pybind11::arg("wts"));
		cl.def("assign", (class ROL::Quadrature1D<double> & (ROL::Quadrature1D<double>::*)(const class ROL::Quadrature1D<double> &)) &ROL::Quadrature1D<double>::operator=, "C++: ROL::Quadrature1D<double>::operator=(const class ROL::Quadrature1D<double> &) --> class ROL::Quadrature1D<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::GaussChebyshev3Quadrature file:ROL_GaussChebyshev3Quadrature.hpp line:53
		pybind11::class_<ROL::GaussChebyshev3Quadrature<double>, Teuchos::RCP<ROL::GaussChebyshev3Quadrature<double>>, PyCallBack_ROL_GaussChebyshev3Quadrature_double_t, ROL::Quadrature1D<double>> cl(M("ROL"), "GaussChebyshev3Quadrature_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const int>(), pybind11::arg("nQuad") );

		cl.def( pybind11::init( [](PyCallBack_ROL_GaussChebyshev3Quadrature_double_t const &o){ return new PyCallBack_ROL_GaussChebyshev3Quadrature_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::GaussChebyshev3Quadrature<double> const &o){ return new ROL::GaussChebyshev3Quadrature<double>(o); } ) );
		cl.def("get", (void (ROL::Quadrature1D<double>::*)(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const) &ROL::Quadrature1D<double>::get, "C++: ROL::Quadrature1D<double>::get(class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &) const --> void", pybind11::arg("pts"), pybind11::arg("wts"));
		cl.def("assign", (class ROL::Quadrature1D<double> & (ROL::Quadrature1D<double>::*)(const class ROL::Quadrature1D<double> &)) &ROL::Quadrature1D<double>::operator=, "C++: ROL::Quadrature1D<double>::operator=(const class ROL::Quadrature1D<double> &) --> class ROL::Quadrature1D<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::ChebyshevSpectral file:ROL_ChebyshevSpectral.hpp line:97
		pybind11::class_<ROL::ChebyshevSpectral<double>, Teuchos::RCP<ROL::ChebyshevSpectral<double>>, PyCallBack_ROL_ChebyshevSpectral_double_t, ROL::SpectralRisk<double>> cl(M("ROL"), "ChebyshevSpectral_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init<const double, const double, const int, const int, const class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("lower"), pybind11::arg("upper"), pybind11::arg("nQuad"), pybind11::arg("wType"), pybind11::arg("pf") );

		cl.def( pybind11::init( [](PyCallBack_ROL_ChebyshevSpectral_double_t const &o){ return new PyCallBack_ROL_ChebyshevSpectral_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::ChebyshevSpectral<double> const &o){ return new ROL::ChebyshevSpectral<double>(o); } ) );
		cl.def("assign", (class ROL::ChebyshevSpectral<double> & (ROL::ChebyshevSpectral<double>::*)(const class ROL::ChebyshevSpectral<double> &)) &ROL::ChebyshevSpectral<double>::operator=, "C++: ROL::ChebyshevSpectral<double>::operator=(const class ROL::ChebyshevSpectral<double> &) --> class ROL::ChebyshevSpectral<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("setStorage", (void (ROL::SpectralRisk<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::SpectralRisk<double>::setStorage, "C++: ROL::SpectralRisk<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::SpectralRisk<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::SpectralRisk<double>::setHessVecStorage, "C++: ROL::SpectralRisk<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("setSample", (void (ROL::SpectralRisk<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::SpectralRisk<double>::setSample, "C++: ROL::SpectralRisk<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("resetStorage", [](ROL::SpectralRisk<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::SpectralRisk<double>::*)(bool)) &ROL::SpectralRisk<double>::resetStorage, "C++: ROL::SpectralRisk<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::SpectralRisk<double>::*)(enum ROL::UpdateType)) &ROL::SpectralRisk<double>::resetStorage, "C++: ROL::SpectralRisk<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::SpectralRisk<double>::*)(const class ROL::Vector<double> &)) &ROL::SpectralRisk<double>::initialize, "C++: ROL::SpectralRisk<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("computeStatistic", (double (ROL::SpectralRisk<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::SpectralRisk<double>::computeStatistic, "C++: ROL::SpectralRisk<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::SpectralRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SpectralRisk<double>::updateValue, "C++: ROL::SpectralRisk<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::SpectralRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SpectralRisk<double>::updateGradient, "C++: ROL::SpectralRisk<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::SpectralRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SpectralRisk<double>::updateHessVec, "C++: ROL::SpectralRisk<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::SpectralRisk<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SpectralRisk<double>::getValue, "C++: ROL::SpectralRisk<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::SpectralRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SpectralRisk<double>::getGradient, "C++: ROL::SpectralRisk<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::SpectralRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SpectralRisk<double>::getHessVec, "C++: ROL::SpectralRisk<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::SpectralRisk<double> & (ROL::SpectralRisk<double>::*)(const class ROL::SpectralRisk<double> &)) &ROL::SpectralRisk<double>::operator=, "C++: ROL::SpectralRisk<double>::operator=(const class ROL::SpectralRisk<double> &) --> class ROL::SpectralRisk<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::QuantileRadius file:ROL_QuantileRadius.hpp line:55
		pybind11::class_<ROL::QuantileRadius<double>, Teuchos::RCP<ROL::QuantileRadius<double>>, PyCallBack_ROL_QuantileRadius_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "QuantileRadius_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init<const double, const double, const class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("prob"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init( [](PyCallBack_ROL_QuantileRadius_double_t const &o){ return new PyCallBack_ROL_QuantileRadius_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::QuantileRadius<double> const &o){ return new ROL::QuantileRadius<double>(o); } ) );
		cl.def("initialize", (void (ROL::QuantileRadius<double>::*)(const class ROL::Vector<double> &)) &ROL::QuantileRadius<double>::initialize, "C++: ROL::QuantileRadius<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("computeStatistic", (double (ROL::QuantileRadius<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::QuantileRadius<double>::computeStatistic, "C++: ROL::QuantileRadius<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::QuantileRadius<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::QuantileRadius<double>::updateValue, "C++: ROL::QuantileRadius<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::QuantileRadius<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::QuantileRadius<double>::getValue, "C++: ROL::QuantileRadius<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::QuantileRadius<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::QuantileRadius<double>::updateGradient, "C++: ROL::QuantileRadius<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::QuantileRadius<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::QuantileRadius<double>::getGradient, "C++: ROL::QuantileRadius<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::QuantileRadius<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::QuantileRadius<double>::updateHessVec, "C++: ROL::QuantileRadius<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::QuantileRadius<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::QuantileRadius<double>::getHessVec, "C++: ROL::QuantileRadius<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::QuantileRadius<double> & (ROL::QuantileRadius<double>::*)(const class ROL::QuantileRadius<double> &)) &ROL::QuantileRadius<double>::operator=, "C++: ROL::QuantileRadius<double>::operator=(const class ROL::QuantileRadius<double> &) --> class ROL::QuantileRadius<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::HMCR file:ROL_HMCR.hpp line:75
		pybind11::class_<ROL::HMCR<double>, Teuchos::RCP<ROL::HMCR<double>>, PyCallBack_ROL_HMCR_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "HMCR_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double, const double, const unsigned int, const class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("prob"), pybind11::arg("lambda"), pybind11::arg("order"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_HMCR_double_t const &o){ return new PyCallBack_ROL_HMCR_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::HMCR<double> const &o){ return new ROL::HMCR<double>(o); } ) );
		cl.def("initialize", (void (ROL::HMCR<double>::*)(const class ROL::Vector<double> &)) &ROL::HMCR<double>::initialize, "C++: ROL::HMCR<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("updateValue", (void (ROL::HMCR<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::HMCR<double>::updateValue, "C++: ROL::HMCR<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::HMCR<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::HMCR<double>::getValue, "C++: ROL::HMCR<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::HMCR<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::HMCR<double>::updateGradient, "C++: ROL::HMCR<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::HMCR<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::HMCR<double>::getGradient, "C++: ROL::HMCR<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::HMCR<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::HMCR<double>::updateHessVec, "C++: ROL::HMCR<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::HMCR<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::HMCR<double>::getHessVec, "C++: ROL::HMCR<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::HMCR<double> & (ROL::HMCR<double>::*)(const class ROL::HMCR<double> &)) &ROL::HMCR<double>::operator=, "C++: ROL::HMCR<double>::operator=(const class ROL::HMCR<double> &) --> class ROL::HMCR<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::EntropicRisk file:ROL_EntropicRisk.hpp line:66
		pybind11::class_<ROL::EntropicRisk<double>, Teuchos::RCP<ROL::EntropicRisk<double>>, PyCallBack_ROL_EntropicRisk_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "EntropicRisk_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::EntropicRisk<double>(); }, [](){ return new PyCallBack_ROL_EntropicRisk_double_t(); } ), "doc");
		cl.def( pybind11::init<const double>(), pybind11::arg("coeff") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_EntropicRisk_double_t const &o){ return new PyCallBack_ROL_EntropicRisk_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::EntropicRisk<double> const &o){ return new ROL::EntropicRisk<double>(o); } ) );
		cl.def("updateValue", (void (ROL::EntropicRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::EntropicRisk<double>::updateValue, "C++: ROL::EntropicRisk<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::EntropicRisk<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::EntropicRisk<double>::getValue, "C++: ROL::EntropicRisk<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::EntropicRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::EntropicRisk<double>::updateGradient, "C++: ROL::EntropicRisk<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::EntropicRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::EntropicRisk<double>::getGradient, "C++: ROL::EntropicRisk<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::EntropicRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::EntropicRisk<double>::updateHessVec, "C++: ROL::EntropicRisk<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::EntropicRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::EntropicRisk<double>::getHessVec, "C++: ROL::EntropicRisk<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::EntropicRisk<double> & (ROL::EntropicRisk<double>::*)(const class ROL::EntropicRisk<double> &)) &ROL::EntropicRisk<double>::operator=, "C++: ROL::EntropicRisk<double>::operator=(const class ROL::EntropicRisk<double> &) --> class ROL::EntropicRisk<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::CoherentEntropicRisk file:ROL_CoherentEntropicRisk.hpp line:67
		pybind11::class_<ROL::CoherentEntropicRisk<double>, Teuchos::RCP<ROL::CoherentEntropicRisk<double>>, PyCallBack_ROL_CoherentEntropicRisk_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "CoherentEntropicRisk_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::CoherentEntropicRisk<double>(); }, [](){ return new PyCallBack_ROL_CoherentEntropicRisk_double_t(); } ) );
		cl.def( pybind11::init( [](PyCallBack_ROL_CoherentEntropicRisk_double_t const &o){ return new PyCallBack_ROL_CoherentEntropicRisk_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::CoherentEntropicRisk<double> const &o){ return new ROL::CoherentEntropicRisk<double>(o); } ) );
		cl.def("initialize", (void (ROL::CoherentEntropicRisk<double>::*)(const class ROL::Vector<double> &)) &ROL::CoherentEntropicRisk<double>::initialize, "C++: ROL::CoherentEntropicRisk<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("updateValue", (void (ROL::CoherentEntropicRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::CoherentEntropicRisk<double>::updateValue, "C++: ROL::CoherentEntropicRisk<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::CoherentEntropicRisk<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::CoherentEntropicRisk<double>::getValue, "C++: ROL::CoherentEntropicRisk<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::CoherentEntropicRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::CoherentEntropicRisk<double>::updateGradient, "C++: ROL::CoherentEntropicRisk<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::CoherentEntropicRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::CoherentEntropicRisk<double>::getGradient, "C++: ROL::CoherentEntropicRisk<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::CoherentEntropicRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::CoherentEntropicRisk<double>::updateHessVec, "C++: ROL::CoherentEntropicRisk<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::CoherentEntropicRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::CoherentEntropicRisk<double>::getHessVec, "C++: ROL::CoherentEntropicRisk<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::CoherentEntropicRisk<double> & (ROL::CoherentEntropicRisk<double>::*)(const class ROL::CoherentEntropicRisk<double> &)) &ROL::CoherentEntropicRisk<double>::operator=, "C++: ROL::CoherentEntropicRisk<double>::operator=(const class ROL::CoherentEntropicRisk<double> &) --> class ROL::CoherentEntropicRisk<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	// ROL::EAbsoluteValue file:ROL_AbsoluteValue.hpp line:53
	pybind11::enum_<ROL::EAbsoluteValue>(M("ROL"), "EAbsoluteValue", pybind11::arithmetic(), "", pybind11::module_local())
		.value("ABSOLUTEVALUE_TRUE", ROL::ABSOLUTEVALUE_TRUE)
		.value("ABSOLUTEVALUE_SQUAREROOT", ROL::ABSOLUTEVALUE_SQUAREROOT)
		.value("ABSOLUTEVALUE_SQRTDENOM", ROL::ABSOLUTEVALUE_SQRTDENOM)
		.value("ABSOLUTEVALUE_C2", ROL::ABSOLUTEVALUE_C2)
		.value("ABSOLUTEVALUE_LAST", ROL::ABSOLUTEVALUE_LAST)
		.export_values();

;

	{ // ROL::AbsoluteValue file:ROL_AbsoluteValue.hpp line:62
		pybind11::class_<ROL::AbsoluteValue<double>, Teuchos::RCP<ROL::AbsoluteValue<double>>, PyCallBack_ROL_AbsoluteValue_double_t, ROL::PositiveFunction<double>> cl(M("ROL"), "AbsoluteValue_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::AbsoluteValue<double>(); }, [](){ return new PyCallBack_ROL_AbsoluteValue_double_t(); } ), "doc");
		cl.def( pybind11::init( [](double const & a0){ return new ROL::AbsoluteValue<double>(a0); }, [](double const & a0){ return new PyCallBack_ROL_AbsoluteValue_double_t(a0); } ), "doc");
		cl.def( pybind11::init<double, enum ROL::EAbsoluteValue>(), pybind11::arg("param"), pybind11::arg("eav") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_AbsoluteValue_double_t const &o){ return new PyCallBack_ROL_AbsoluteValue_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::AbsoluteValue<double> const &o){ return new ROL::AbsoluteValue<double>(o); } ) );
		cl.def("evaluate", (double (ROL::AbsoluteValue<double>::*)(double, int)) &ROL::AbsoluteValue<double>::evaluate, "C++: ROL::AbsoluteValue<double>::evaluate(double, int) --> double", pybind11::arg("input"), pybind11::arg("deriv"));
		cl.def("assign", (class ROL::AbsoluteValue<double> & (ROL::AbsoluteValue<double>::*)(const class ROL::AbsoluteValue<double> &)) &ROL::AbsoluteValue<double>::operator=, "C++: ROL::AbsoluteValue<double>::operator=(const class ROL::AbsoluteValue<double> &) --> class ROL::AbsoluteValue<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("evaluate", (double (ROL::PositiveFunction<double>::*)(double, int)) &ROL::PositiveFunction<double>::evaluate, "C++: ROL::PositiveFunction<double>::evaluate(double, int) --> double", pybind11::arg("input"), pybind11::arg("deriv"));
		cl.def("assign", (class ROL::PositiveFunction<double> & (ROL::PositiveFunction<double>::*)(const class ROL::PositiveFunction<double> &)) &ROL::PositiveFunction<double>::operator=, "C++: ROL::PositiveFunction<double>::operator=(const class ROL::PositiveFunction<double> &) --> class ROL::PositiveFunction<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::MeanDeviationFromTarget file:ROL_MeanDeviationFromTarget.hpp line:76
		pybind11::class_<ROL::MeanDeviationFromTarget<double>, Teuchos::RCP<ROL::MeanDeviationFromTarget<double>>, PyCallBack_ROL_MeanDeviationFromTarget_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "MeanDeviationFromTarget_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double, const double, const double, const class Teuchos::RCP<class ROL::PositiveFunction<double> > &>(), pybind11::arg("target"), pybind11::arg("order"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init<const class std::vector<double, class std::allocator<double> > &, const class std::vector<double, class std::allocator<double> > &, const class std::vector<double, class std::allocator<double> > &, const class Teuchos::RCP<class ROL::PositiveFunction<double> > &>(), pybind11::arg("target"), pybind11::arg("order"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_MeanDeviationFromTarget_double_t const &o){ return new PyCallBack_ROL_MeanDeviationFromTarget_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::MeanDeviationFromTarget<double> const &o){ return new ROL::MeanDeviationFromTarget<double>(o); } ) );
		cl.def("initialize", (void (ROL::MeanDeviationFromTarget<double>::*)(const class ROL::Vector<double> &)) &ROL::MeanDeviationFromTarget<double>::initialize, "C++: ROL::MeanDeviationFromTarget<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("updateValue", (void (ROL::MeanDeviationFromTarget<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanDeviationFromTarget<double>::updateValue, "C++: ROL::MeanDeviationFromTarget<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::MeanDeviationFromTarget<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanDeviationFromTarget<double>::getValue, "C++: ROL::MeanDeviationFromTarget<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::MeanDeviationFromTarget<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanDeviationFromTarget<double>::updateGradient, "C++: ROL::MeanDeviationFromTarget<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::MeanDeviationFromTarget<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanDeviationFromTarget<double>::getGradient, "C++: ROL::MeanDeviationFromTarget<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::MeanDeviationFromTarget<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanDeviationFromTarget<double>::updateHessVec, "C++: ROL::MeanDeviationFromTarget<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::MeanDeviationFromTarget<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanDeviationFromTarget<double>::getHessVec, "C++: ROL::MeanDeviationFromTarget<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::MeanDeviationFromTarget<double> & (ROL::MeanDeviationFromTarget<double>::*)(const class ROL::MeanDeviationFromTarget<double> &)) &ROL::MeanDeviationFromTarget<double>::operator=, "C++: ROL::MeanDeviationFromTarget<double>::operator=(const class ROL::MeanDeviationFromTarget<double> &) --> class ROL::MeanDeviationFromTarget<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::MeanDeviation file:ROL_MeanDeviation.hpp line:80
		pybind11::class_<ROL::MeanDeviation<double>, Teuchos::RCP<ROL::MeanDeviation<double>>, PyCallBack_ROL_MeanDeviation_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "MeanDeviation_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double, const double, const class Teuchos::RCP<class ROL::PositiveFunction<double> > &>(), pybind11::arg("order"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init<const class std::vector<double, class std::allocator<double> > &, const class std::vector<double, class std::allocator<double> > &, const class Teuchos::RCP<class ROL::PositiveFunction<double> > &>(), pybind11::arg("order"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_MeanDeviation_double_t const &o){ return new PyCallBack_ROL_MeanDeviation_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::MeanDeviation<double> const &o){ return new ROL::MeanDeviation<double>(o); } ) );
		cl.def("setStorage", (void (ROL::MeanDeviation<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::MeanDeviation<double>::setStorage, "C++: ROL::MeanDeviation<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::MeanDeviation<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::MeanDeviation<double>::setHessVecStorage, "C++: ROL::MeanDeviation<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("initialize", (void (ROL::MeanDeviation<double>::*)(const class ROL::Vector<double> &)) &ROL::MeanDeviation<double>::initialize, "C++: ROL::MeanDeviation<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("updateValue", (void (ROL::MeanDeviation<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanDeviation<double>::updateValue, "C++: ROL::MeanDeviation<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::MeanDeviation<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanDeviation<double>::updateGradient, "C++: ROL::MeanDeviation<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::MeanDeviation<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanDeviation<double>::updateHessVec, "C++: ROL::MeanDeviation<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::MeanDeviation<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanDeviation<double>::getValue, "C++: ROL::MeanDeviation<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::MeanDeviation<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanDeviation<double>::getGradient, "C++: ROL::MeanDeviation<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::MeanDeviation<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanDeviation<double>::getHessVec, "C++: ROL::MeanDeviation<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::MeanDeviation<double> & (ROL::MeanDeviation<double>::*)(const class ROL::MeanDeviation<double> &)) &ROL::MeanDeviation<double>::operator=, "C++: ROL::MeanDeviation<double>::operator=(const class ROL::MeanDeviation<double> &) --> class ROL::MeanDeviation<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::MeanVarianceFromTarget file:ROL_MeanVarianceFromTarget.hpp line:77
		pybind11::class_<ROL::MeanVarianceFromTarget<double>, Teuchos::RCP<ROL::MeanVarianceFromTarget<double>>, PyCallBack_ROL_MeanVarianceFromTarget_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "MeanVarianceFromTarget_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double, const double, const double, const class Teuchos::RCP<class ROL::PositiveFunction<double> > &>(), pybind11::arg("target"), pybind11::arg("order"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init<const class std::vector<double, class std::allocator<double> > &, const class std::vector<double, class std::allocator<double> > &, const class std::vector<double, class std::allocator<double> > &, const class Teuchos::RCP<class ROL::PositiveFunction<double> > &>(), pybind11::arg("target"), pybind11::arg("order"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_MeanVarianceFromTarget_double_t const &o){ return new PyCallBack_ROL_MeanVarianceFromTarget_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::MeanVarianceFromTarget<double> const &o){ return new ROL::MeanVarianceFromTarget<double>(o); } ) );
		cl.def("updateValue", (void (ROL::MeanVarianceFromTarget<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanVarianceFromTarget<double>::updateValue, "C++: ROL::MeanVarianceFromTarget<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::MeanVarianceFromTarget<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanVarianceFromTarget<double>::updateGradient, "C++: ROL::MeanVarianceFromTarget<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::MeanVarianceFromTarget<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanVarianceFromTarget<double>::updateHessVec, "C++: ROL::MeanVarianceFromTarget<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("assign", (class ROL::MeanVarianceFromTarget<double> & (ROL::MeanVarianceFromTarget<double>::*)(const class ROL::MeanVarianceFromTarget<double> &)) &ROL::MeanVarianceFromTarget<double>::operator=, "C++: ROL::MeanVarianceFromTarget<double>::operator=(const class ROL::MeanVarianceFromTarget<double> &) --> class ROL::MeanVarianceFromTarget<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::MeanVariance file:ROL_MeanVariance.hpp line:78
		pybind11::class_<ROL::MeanVariance<double>, Teuchos::RCP<ROL::MeanVariance<double>>, PyCallBack_ROL_MeanVariance_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "MeanVariance_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double, const double, const class Teuchos::RCP<class ROL::PositiveFunction<double> > &>(), pybind11::arg("order"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init<const class std::vector<double, class std::allocator<double> > &, const class std::vector<double, class std::allocator<double> > &, const class Teuchos::RCP<class ROL::PositiveFunction<double> > &>(), pybind11::arg("order"), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_MeanVariance_double_t const &o){ return new PyCallBack_ROL_MeanVariance_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::MeanVariance<double> const &o){ return new ROL::MeanVariance<double>(o); } ) );
		cl.def("setStorage", (void (ROL::MeanVariance<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::MeanVariance<double>::setStorage, "C++: ROL::MeanVariance<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::MeanVariance<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::MeanVariance<double>::setHessVecStorage, "C++: ROL::MeanVariance<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("updateValue", (void (ROL::MeanVariance<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanVariance<double>::updateValue, "C++: ROL::MeanVariance<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::MeanVariance<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanVariance<double>::getValue, "C++: ROL::MeanVariance<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::MeanVariance<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanVariance<double>::updateGradient, "C++: ROL::MeanVariance<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::MeanVariance<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanVariance<double>::getGradient, "C++: ROL::MeanVariance<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::MeanVariance<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanVariance<double>::updateHessVec, "C++: ROL::MeanVariance<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::MeanVariance<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanVariance<double>::getHessVec, "C++: ROL::MeanVariance<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::MeanVariance<double> & (ROL::MeanVariance<double>::*)(const class ROL::MeanVariance<double> &)) &ROL::MeanVariance<double>::operator=, "C++: ROL::MeanVariance<double>::operator=(const class ROL::MeanVariance<double> &) --> class ROL::MeanVariance<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::MeanSemiDeviation file:ROL_MeanSemiDeviation.hpp line:72
		pybind11::class_<ROL::MeanSemiDeviation<double>, Teuchos::RCP<ROL::MeanSemiDeviation<double>>, PyCallBack_ROL_MeanSemiDeviation_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "MeanSemiDeviation_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double, const class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("coeff"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_MeanSemiDeviation_double_t const &o){ return new PyCallBack_ROL_MeanSemiDeviation_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::MeanSemiDeviation<double> const &o){ return new ROL::MeanSemiDeviation<double>(o); } ) );
		cl.def("setStorage", (void (ROL::MeanSemiDeviation<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::MeanSemiDeviation<double>::setStorage, "C++: ROL::MeanSemiDeviation<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::MeanSemiDeviation<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::MeanSemiDeviation<double>::setHessVecStorage, "C++: ROL::MeanSemiDeviation<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("initialize", (void (ROL::MeanSemiDeviation<double>::*)(const class ROL::Vector<double> &)) &ROL::MeanSemiDeviation<double>::initialize, "C++: ROL::MeanSemiDeviation<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("updateValue", (void (ROL::MeanSemiDeviation<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanSemiDeviation<double>::updateValue, "C++: ROL::MeanSemiDeviation<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::MeanSemiDeviation<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanSemiDeviation<double>::getValue, "C++: ROL::MeanSemiDeviation<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::MeanSemiDeviation<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanSemiDeviation<double>::updateGradient, "C++: ROL::MeanSemiDeviation<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::MeanSemiDeviation<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanSemiDeviation<double>::getGradient, "C++: ROL::MeanSemiDeviation<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::MeanSemiDeviation<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanSemiDeviation<double>::updateHessVec, "C++: ROL::MeanSemiDeviation<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::MeanSemiDeviation<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanSemiDeviation<double>::getHessVec, "C++: ROL::MeanSemiDeviation<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::MeanSemiDeviation<double> & (ROL::MeanSemiDeviation<double>::*)(const class ROL::MeanSemiDeviation<double> &)) &ROL::MeanSemiDeviation<double>::operator=, "C++: ROL::MeanSemiDeviation<double>::operator=(const class ROL::MeanSemiDeviation<double> &) --> class ROL::MeanSemiDeviation<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::MeanSemiDeviationFromTarget file:ROL_MeanSemiDeviationFromTarget.hpp line:72
		pybind11::class_<ROL::MeanSemiDeviationFromTarget<double>, Teuchos::RCP<ROL::MeanSemiDeviationFromTarget<double>>, PyCallBack_ROL_MeanSemiDeviationFromTarget_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "MeanSemiDeviationFromTarget_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double, const double, const class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("coeff"), pybind11::arg("target"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_MeanSemiDeviationFromTarget_double_t const &o){ return new PyCallBack_ROL_MeanSemiDeviationFromTarget_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::MeanSemiDeviationFromTarget<double> const &o){ return new ROL::MeanSemiDeviationFromTarget<double>(o); } ) );
		cl.def("updateValue", (void (ROL::MeanSemiDeviationFromTarget<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanSemiDeviationFromTarget<double>::updateValue, "C++: ROL::MeanSemiDeviationFromTarget<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::MeanSemiDeviationFromTarget<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanSemiDeviationFromTarget<double>::getValue, "C++: ROL::MeanSemiDeviationFromTarget<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::MeanSemiDeviationFromTarget<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanSemiDeviationFromTarget<double>::updateGradient, "C++: ROL::MeanSemiDeviationFromTarget<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::MeanSemiDeviationFromTarget<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanSemiDeviationFromTarget<double>::getGradient, "C++: ROL::MeanSemiDeviationFromTarget<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::MeanSemiDeviationFromTarget<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::MeanSemiDeviationFromTarget<double>::updateHessVec, "C++: ROL::MeanSemiDeviationFromTarget<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::MeanSemiDeviationFromTarget<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::MeanSemiDeviationFromTarget<double>::getHessVec, "C++: ROL::MeanSemiDeviationFromTarget<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::MeanSemiDeviationFromTarget<double> & (ROL::MeanSemiDeviationFromTarget<double>::*)(const class ROL::MeanSemiDeviationFromTarget<double> &)) &ROL::MeanSemiDeviationFromTarget<double>::operator=, "C++: ROL::MeanSemiDeviationFromTarget<double>::operator=(const class ROL::MeanSemiDeviationFromTarget<double> &) --> class ROL::MeanSemiDeviationFromTarget<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::ExpectationQuad file:ROL_ExpectationQuad.hpp line:87
		pybind11::class_<ROL::ExpectationQuad<double>, Teuchos::RCP<ROL::ExpectationQuad<double>>, PyCallBack_ROL_ExpectationQuad_double_t> cl(M("ROL"), "ExpectationQuad_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new PyCallBack_ROL_ExpectationQuad_double_t(); } ) );
		cl.def(pybind11::init<PyCallBack_ROL_ExpectationQuad_double_t const &>());
		cl.def("regret", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::regret, "C++: ROL::ExpectationQuad<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("error", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::error, "C++: ROL::ExpectationQuad<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::ExpectationQuad<double>::*)()) &ROL::ExpectationQuad<double>::check, "C++: ROL::ExpectationQuad<double>::check() --> void");
		cl.def("assign", (class ROL::ExpectationQuad<double> & (ROL::ExpectationQuad<double>::*)(const class ROL::ExpectationQuad<double> &)) &ROL::ExpectationQuad<double>::operator=, "C++: ROL::ExpectationQuad<double>::operator=(const class ROL::ExpectationQuad<double> &) --> class ROL::ExpectationQuad<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::ExpectationQuadRisk file:ROL_ExpectationQuadRisk.hpp line:89
		pybind11::class_<ROL::ExpectationQuadRisk<double>, Teuchos::RCP<ROL::ExpectationQuadRisk<double>>, PyCallBack_ROL_ExpectationQuadRisk_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "ExpectationQuadRisk_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::ExpectationQuad<double> > &>(), pybind11::arg("eq") );

		cl.def( pybind11::init( [](PyCallBack_ROL_ExpectationQuadRisk_double_t const &o){ return new PyCallBack_ROL_ExpectationQuadRisk_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::ExpectationQuadRisk<double> const &o){ return new ROL::ExpectationQuadRisk<double>(o); } ) );
		cl.def("checkRegret", (void (ROL::ExpectationQuadRisk<double>::*)()) &ROL::ExpectationQuadRisk<double>::checkRegret, "C++: ROL::ExpectationQuadRisk<double>::checkRegret() --> void");
		cl.def("updateValue", (void (ROL::ExpectationQuadRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadRisk<double>::updateValue, "C++: ROL::ExpectationQuadRisk<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::ExpectationQuadRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadRisk<double>::updateGradient, "C++: ROL::ExpectationQuadRisk<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::ExpectationQuadRisk<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadRisk<double>::updateHessVec, "C++: ROL::ExpectationQuadRisk<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::ExpectationQuadRisk<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadRisk<double>::getValue, "C++: ROL::ExpectationQuadRisk<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::ExpectationQuadRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadRisk<double>::getGradient, "C++: ROL::ExpectationQuadRisk<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::ExpectationQuadRisk<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadRisk<double>::getHessVec, "C++: ROL::ExpectationQuadRisk<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::ExpectationQuadRisk<double> & (ROL::ExpectationQuadRisk<double>::*)(const class ROL::ExpectationQuadRisk<double> &)) &ROL::ExpectationQuadRisk<double>::operator=, "C++: ROL::ExpectationQuadRisk<double>::operator=(const class ROL::ExpectationQuadRisk<double> &) --> class ROL::ExpectationQuadRisk<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::LogQuantileQuadrangle file:ROL_LogQuantileQuadrangle.hpp line:76
		pybind11::class_<ROL::LogQuantileQuadrangle<double>, Teuchos::RCP<ROL::LogQuantileQuadrangle<double>>, PyCallBack_ROL_LogQuantileQuadrangle_double_t, ROL::ExpectationQuad<double>> cl(M("ROL"), "LogQuantileQuadrangle_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<double, double, double, class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("alpha"), pybind11::arg("rate"), pybind11::arg("eps"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_LogQuantileQuadrangle_double_t const &o){ return new PyCallBack_ROL_LogQuantileQuadrangle_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::LogQuantileQuadrangle<double> const &o){ return new ROL::LogQuantileQuadrangle<double>(o); } ) );
		cl.def("error", [](ROL::LogQuantileQuadrangle<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::LogQuantileQuadrangle<double>::*)(double, int)) &ROL::LogQuantileQuadrangle<double>::error, "C++: ROL::LogQuantileQuadrangle<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("regret", [](ROL::LogQuantileQuadrangle<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::LogQuantileQuadrangle<double>::*)(double, int)) &ROL::LogQuantileQuadrangle<double>::regret, "C++: ROL::LogQuantileQuadrangle<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::LogQuantileQuadrangle<double>::*)()) &ROL::LogQuantileQuadrangle<double>::check, "C++: ROL::LogQuantileQuadrangle<double>::check() --> void");
		cl.def("assign", (class ROL::LogQuantileQuadrangle<double> & (ROL::LogQuantileQuadrangle<double>::*)(const class ROL::LogQuantileQuadrangle<double> &)) &ROL::LogQuantileQuadrangle<double>::operator=, "C++: ROL::LogQuantileQuadrangle<double>::operator=(const class ROL::LogQuantileQuadrangle<double> &) --> class ROL::LogQuantileQuadrangle<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("regret", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::regret, "C++: ROL::ExpectationQuad<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("error", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::error, "C++: ROL::ExpectationQuad<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::ExpectationQuad<double>::*)()) &ROL::ExpectationQuad<double>::check, "C++: ROL::ExpectationQuad<double>::check() --> void");
		cl.def("assign", (class ROL::ExpectationQuad<double> & (ROL::ExpectationQuad<double>::*)(const class ROL::ExpectationQuad<double> &)) &ROL::ExpectationQuad<double>::operator=, "C++: ROL::ExpectationQuad<double>::operator=(const class ROL::ExpectationQuad<double> &) --> class ROL::ExpectationQuad<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::SmoothedWorstCaseQuadrangle file:ROL_SmoothedWorstCaseQuadrangle.hpp line:89
		pybind11::class_<ROL::SmoothedWorstCaseQuadrangle<double>, Teuchos::RCP<ROL::SmoothedWorstCaseQuadrangle<double>>, PyCallBack_ROL_SmoothedWorstCaseQuadrangle_double_t, ROL::ExpectationQuad<double>> cl(M("ROL"), "SmoothedWorstCaseQuadrangle_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double>(), pybind11::arg("eps") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_SmoothedWorstCaseQuadrangle_double_t const &o){ return new PyCallBack_ROL_SmoothedWorstCaseQuadrangle_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::SmoothedWorstCaseQuadrangle<double> const &o){ return new ROL::SmoothedWorstCaseQuadrangle<double>(o); } ) );
		cl.def("error", [](ROL::SmoothedWorstCaseQuadrangle<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::SmoothedWorstCaseQuadrangle<double>::*)(double, int)) &ROL::SmoothedWorstCaseQuadrangle<double>::error, "C++: ROL::SmoothedWorstCaseQuadrangle<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("regret", [](ROL::SmoothedWorstCaseQuadrangle<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::SmoothedWorstCaseQuadrangle<double>::*)(double, int)) &ROL::SmoothedWorstCaseQuadrangle<double>::regret, "C++: ROL::SmoothedWorstCaseQuadrangle<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::SmoothedWorstCaseQuadrangle<double>::*)()) &ROL::SmoothedWorstCaseQuadrangle<double>::check, "C++: ROL::SmoothedWorstCaseQuadrangle<double>::check() --> void");
		cl.def("assign", (class ROL::SmoothedWorstCaseQuadrangle<double> & (ROL::SmoothedWorstCaseQuadrangle<double>::*)(const class ROL::SmoothedWorstCaseQuadrangle<double> &)) &ROL::SmoothedWorstCaseQuadrangle<double>::operator=, "C++: ROL::SmoothedWorstCaseQuadrangle<double>::operator=(const class ROL::SmoothedWorstCaseQuadrangle<double> &) --> class ROL::SmoothedWorstCaseQuadrangle<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("regret", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::regret, "C++: ROL::ExpectationQuad<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("error", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::error, "C++: ROL::ExpectationQuad<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::ExpectationQuad<double>::*)()) &ROL::ExpectationQuad<double>::check, "C++: ROL::ExpectationQuad<double>::check() --> void");
		cl.def("assign", (class ROL::ExpectationQuad<double> & (ROL::ExpectationQuad<double>::*)(const class ROL::ExpectationQuad<double> &)) &ROL::ExpectationQuad<double>::operator=, "C++: ROL::ExpectationQuad<double>::operator=(const class ROL::ExpectationQuad<double> &) --> class ROL::ExpectationQuad<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::TruncatedMeanQuadrangle file:ROL_TruncatedMeanQuadrangle.hpp line:52
		pybind11::class_<ROL::TruncatedMeanQuadrangle<double>, Teuchos::RCP<ROL::TruncatedMeanQuadrangle<double>>, PyCallBack_ROL_TruncatedMeanQuadrangle_double_t, ROL::ExpectationQuad<double>> cl(M("ROL"), "TruncatedMeanQuadrangle_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<double>(), pybind11::arg("beta") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_TruncatedMeanQuadrangle_double_t const &o){ return new PyCallBack_ROL_TruncatedMeanQuadrangle_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::TruncatedMeanQuadrangle<double> const &o){ return new ROL::TruncatedMeanQuadrangle<double>(o); } ) );
		cl.def("error", [](ROL::TruncatedMeanQuadrangle<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::TruncatedMeanQuadrangle<double>::*)(double, int)) &ROL::TruncatedMeanQuadrangle<double>::error, "C++: ROL::TruncatedMeanQuadrangle<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("regret", [](ROL::TruncatedMeanQuadrangle<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::TruncatedMeanQuadrangle<double>::*)(double, int)) &ROL::TruncatedMeanQuadrangle<double>::regret, "C++: ROL::TruncatedMeanQuadrangle<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::TruncatedMeanQuadrangle<double>::*)()) &ROL::TruncatedMeanQuadrangle<double>::check, "C++: ROL::TruncatedMeanQuadrangle<double>::check() --> void");
		cl.def("assign", (class ROL::TruncatedMeanQuadrangle<double> & (ROL::TruncatedMeanQuadrangle<double>::*)(const class ROL::TruncatedMeanQuadrangle<double> &)) &ROL::TruncatedMeanQuadrangle<double>::operator=, "C++: ROL::TruncatedMeanQuadrangle<double>::operator=(const class ROL::TruncatedMeanQuadrangle<double> &) --> class ROL::TruncatedMeanQuadrangle<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("regret", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::regret, "C++: ROL::ExpectationQuad<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("error", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::error, "C++: ROL::ExpectationQuad<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::ExpectationQuad<double>::*)()) &ROL::ExpectationQuad<double>::check, "C++: ROL::ExpectationQuad<double>::check() --> void");
		cl.def("assign", (class ROL::ExpectationQuad<double> & (ROL::ExpectationQuad<double>::*)(const class ROL::ExpectationQuad<double> &)) &ROL::ExpectationQuad<double>::operator=, "C++: ROL::ExpectationQuad<double>::operator=(const class ROL::ExpectationQuad<double> &) --> class ROL::ExpectationQuad<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::MoreauYosidaCVaR file:ROL_MoreauYosidaCVaR.hpp line:105
		pybind11::class_<ROL::MoreauYosidaCVaR<double>, Teuchos::RCP<ROL::MoreauYosidaCVaR<double>>, PyCallBack_ROL_MoreauYosidaCVaR_double_t, ROL::ExpectationQuad<double>> cl(M("ROL"), "MoreauYosidaCVaR_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<double, double>(), pybind11::arg("prob"), pybind11::arg("eps") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_MoreauYosidaCVaR_double_t const &o){ return new PyCallBack_ROL_MoreauYosidaCVaR_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::MoreauYosidaCVaR<double> const &o){ return new ROL::MoreauYosidaCVaR<double>(o); } ) );
		cl.def("error", [](ROL::MoreauYosidaCVaR<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::MoreauYosidaCVaR<double>::*)(double, int)) &ROL::MoreauYosidaCVaR<double>::error, "C++: ROL::MoreauYosidaCVaR<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("regret", [](ROL::MoreauYosidaCVaR<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::MoreauYosidaCVaR<double>::*)(double, int)) &ROL::MoreauYosidaCVaR<double>::regret, "C++: ROL::MoreauYosidaCVaR<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::MoreauYosidaCVaR<double>::*)()) &ROL::MoreauYosidaCVaR<double>::check, "C++: ROL::MoreauYosidaCVaR<double>::check() --> void");
		cl.def("assign", (class ROL::MoreauYosidaCVaR<double> & (ROL::MoreauYosidaCVaR<double>::*)(const class ROL::MoreauYosidaCVaR<double> &)) &ROL::MoreauYosidaCVaR<double>::operator=, "C++: ROL::MoreauYosidaCVaR<double>::operator=(const class ROL::MoreauYosidaCVaR<double> &) --> class ROL::MoreauYosidaCVaR<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("regret", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::regret, "C++: ROL::ExpectationQuad<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("error", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::error, "C++: ROL::ExpectationQuad<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::ExpectationQuad<double>::*)()) &ROL::ExpectationQuad<double>::check, "C++: ROL::ExpectationQuad<double>::check() --> void");
		cl.def("assign", (class ROL::ExpectationQuad<double> & (ROL::ExpectationQuad<double>::*)(const class ROL::ExpectationQuad<double> &)) &ROL::ExpectationQuad<double>::operator=, "C++: ROL::ExpectationQuad<double>::operator=(const class ROL::ExpectationQuad<double> &) --> class ROL::ExpectationQuad<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::GenMoreauYosidaCVaR file:ROL_GenMoreauYosidaCVaR.hpp line:105
		pybind11::class_<ROL::GenMoreauYosidaCVaR<double>, Teuchos::RCP<ROL::GenMoreauYosidaCVaR<double>>, PyCallBack_ROL_GenMoreauYosidaCVaR_double_t, ROL::ExpectationQuad<double>> cl(M("ROL"), "GenMoreauYosidaCVaR_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<double, double>(), pybind11::arg("prob"), pybind11::arg("eps") );

		cl.def( pybind11::init<double, double, double>(), pybind11::arg("prob"), pybind11::arg("lam"), pybind11::arg("eps") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_GenMoreauYosidaCVaR_double_t const &o){ return new PyCallBack_ROL_GenMoreauYosidaCVaR_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::GenMoreauYosidaCVaR<double> const &o){ return new ROL::GenMoreauYosidaCVaR<double>(o); } ) );
		cl.def("error", [](ROL::GenMoreauYosidaCVaR<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::GenMoreauYosidaCVaR<double>::*)(double, int)) &ROL::GenMoreauYosidaCVaR<double>::error, "C++: ROL::GenMoreauYosidaCVaR<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("regret", [](ROL::GenMoreauYosidaCVaR<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::GenMoreauYosidaCVaR<double>::*)(double, int)) &ROL::GenMoreauYosidaCVaR<double>::regret, "C++: ROL::GenMoreauYosidaCVaR<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::GenMoreauYosidaCVaR<double>::*)()) &ROL::GenMoreauYosidaCVaR<double>::check, "C++: ROL::GenMoreauYosidaCVaR<double>::check() --> void");
		cl.def("assign", (class ROL::GenMoreauYosidaCVaR<double> & (ROL::GenMoreauYosidaCVaR<double>::*)(const class ROL::GenMoreauYosidaCVaR<double> &)) &ROL::GenMoreauYosidaCVaR<double>::operator=, "C++: ROL::GenMoreauYosidaCVaR<double>::operator=(const class ROL::GenMoreauYosidaCVaR<double> &) --> class ROL::GenMoreauYosidaCVaR<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("regret", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::regret, "C++: ROL::ExpectationQuad<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("error", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::error, "C++: ROL::ExpectationQuad<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::ExpectationQuad<double>::*)()) &ROL::ExpectationQuad<double>::check, "C++: ROL::ExpectationQuad<double>::check() --> void");
		cl.def("assign", (class ROL::ExpectationQuad<double> & (ROL::ExpectationQuad<double>::*)(const class ROL::ExpectationQuad<double> &)) &ROL::ExpectationQuad<double>::operator=, "C++: ROL::ExpectationQuad<double>::operator=(const class ROL::ExpectationQuad<double> &) --> class ROL::ExpectationQuad<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::LogExponentialQuadrangle file:ROL_LogExponentialQuadrangle.hpp line:81
		pybind11::class_<ROL::LogExponentialQuadrangle<double>, Teuchos::RCP<ROL::LogExponentialQuadrangle<double>>, PyCallBack_ROL_LogExponentialQuadrangle_double_t, ROL::ExpectationQuad<double>> cl(M("ROL"), "LogExponentialQuadrangle_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::LogExponentialQuadrangle<double>(); }, [](){ return new PyCallBack_ROL_LogExponentialQuadrangle_double_t(); } ), "doc");
		cl.def( pybind11::init<const double>(), pybind11::arg("coeff") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_LogExponentialQuadrangle_double_t const &o){ return new PyCallBack_ROL_LogExponentialQuadrangle_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::LogExponentialQuadrangle<double> const &o){ return new ROL::LogExponentialQuadrangle<double>(o); } ) );
		cl.def("error", [](ROL::LogExponentialQuadrangle<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::LogExponentialQuadrangle<double>::*)(double, int)) &ROL::LogExponentialQuadrangle<double>::error, "C++: ROL::LogExponentialQuadrangle<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("regret", [](ROL::LogExponentialQuadrangle<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::LogExponentialQuadrangle<double>::*)(double, int)) &ROL::LogExponentialQuadrangle<double>::regret, "C++: ROL::LogExponentialQuadrangle<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("assign", (class ROL::LogExponentialQuadrangle<double> & (ROL::LogExponentialQuadrangle<double>::*)(const class ROL::LogExponentialQuadrangle<double> &)) &ROL::LogExponentialQuadrangle<double>::operator=, "C++: ROL::LogExponentialQuadrangle<double>::operator=(const class ROL::LogExponentialQuadrangle<double> &) --> class ROL::LogExponentialQuadrangle<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("regret", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::regret, "C++: ROL::ExpectationQuad<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("error", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::error, "C++: ROL::ExpectationQuad<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::ExpectationQuad<double>::*)()) &ROL::ExpectationQuad<double>::check, "C++: ROL::ExpectationQuad<double>::check() --> void");
		cl.def("assign", (class ROL::ExpectationQuad<double> & (ROL::ExpectationQuad<double>::*)(const class ROL::ExpectationQuad<double> &)) &ROL::ExpectationQuad<double>::operator=, "C++: ROL::ExpectationQuad<double>::operator=(const class ROL::ExpectationQuad<double> &) --> class ROL::ExpectationQuad<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::MeanVarianceQuadrangle file:ROL_MeanVarianceQuadrangle.hpp line:78
		pybind11::class_<ROL::MeanVarianceQuadrangle<double>, Teuchos::RCP<ROL::MeanVarianceQuadrangle<double>>, PyCallBack_ROL_MeanVarianceQuadrangle_double_t, ROL::ExpectationQuad<double>> cl(M("ROL"), "MeanVarianceQuadrangle_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::MeanVarianceQuadrangle<double>(); }, [](){ return new PyCallBack_ROL_MeanVarianceQuadrangle_double_t(); } ), "doc");
		cl.def( pybind11::init<const double>(), pybind11::arg("coeff") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_MeanVarianceQuadrangle_double_t const &o){ return new PyCallBack_ROL_MeanVarianceQuadrangle_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::MeanVarianceQuadrangle<double> const &o){ return new ROL::MeanVarianceQuadrangle<double>(o); } ) );
		cl.def("error", [](ROL::MeanVarianceQuadrangle<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::MeanVarianceQuadrangle<double>::*)(double, int)) &ROL::MeanVarianceQuadrangle<double>::error, "C++: ROL::MeanVarianceQuadrangle<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("regret", [](ROL::MeanVarianceQuadrangle<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::MeanVarianceQuadrangle<double>::*)(double, int)) &ROL::MeanVarianceQuadrangle<double>::regret, "C++: ROL::MeanVarianceQuadrangle<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("assign", (class ROL::MeanVarianceQuadrangle<double> & (ROL::MeanVarianceQuadrangle<double>::*)(const class ROL::MeanVarianceQuadrangle<double> &)) &ROL::MeanVarianceQuadrangle<double>::operator=, "C++: ROL::MeanVarianceQuadrangle<double>::operator=(const class ROL::MeanVarianceQuadrangle<double> &) --> class ROL::MeanVarianceQuadrangle<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("regret", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::regret, "C++: ROL::ExpectationQuad<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("error", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::error, "C++: ROL::ExpectationQuad<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::ExpectationQuad<double>::*)()) &ROL::ExpectationQuad<double>::check, "C++: ROL::ExpectationQuad<double>::check() --> void");
		cl.def("assign", (class ROL::ExpectationQuad<double> & (ROL::ExpectationQuad<double>::*)(const class ROL::ExpectationQuad<double> &)) &ROL::ExpectationQuad<double>::operator=, "C++: ROL::ExpectationQuad<double>::operator=(const class ROL::ExpectationQuad<double> &) --> class ROL::ExpectationQuad<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::FDivergence file:ROL_FDivergence.hpp line:87
		pybind11::class_<ROL::FDivergence<double>, Teuchos::RCP<ROL::FDivergence<double>>, PyCallBack_ROL_FDivergence_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "FDivergence_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double>(), pybind11::arg("thresh") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def(pybind11::init<PyCallBack_ROL_FDivergence_double_t const &>());
		cl.def("Fprimal", [](ROL::FDivergence<double> const &o, double const & a0) -> double { return o.Fprimal(a0); }, "", pybind11::arg("x"));
		cl.def("Fprimal", (double (ROL::FDivergence<double>::*)(double, int) const) &ROL::FDivergence<double>::Fprimal, "C++: ROL::FDivergence<double>::Fprimal(double, int) const --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("Fdual", [](ROL::FDivergence<double> const &o, double const & a0) -> double { return o.Fdual(a0); }, "", pybind11::arg("x"));
		cl.def("Fdual", (double (ROL::FDivergence<double>::*)(double, int) const) &ROL::FDivergence<double>::Fdual, "C++: ROL::FDivergence<double>::Fdual(double, int) const --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", [](ROL::FDivergence<double> const &o) -> bool { return o.check(); }, "");
		cl.def("check", (bool (ROL::FDivergence<double>::*)(std::ostream &) const) &ROL::FDivergence<double>::check, "C++: ROL::FDivergence<double>::check(std::ostream &) const --> bool", pybind11::arg("outStream"));
		cl.def("initialize", (void (ROL::FDivergence<double>::*)(const class ROL::Vector<double> &)) &ROL::FDivergence<double>::initialize, "C++: ROL::FDivergence<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("updateValue", (void (ROL::FDivergence<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::FDivergence<double>::updateValue, "C++: ROL::FDivergence<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::FDivergence<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::FDivergence<double>::getValue, "C++: ROL::FDivergence<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::FDivergence<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::FDivergence<double>::updateGradient, "C++: ROL::FDivergence<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::FDivergence<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::FDivergence<double>::getGradient, "C++: ROL::FDivergence<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::FDivergence<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::FDivergence<double>::updateHessVec, "C++: ROL::FDivergence<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::FDivergence<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::FDivergence<double>::getHessVec, "C++: ROL::FDivergence<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::FDivergence<double> & (ROL::FDivergence<double>::*)(const class ROL::FDivergence<double> &)) &ROL::FDivergence<double>::operator=, "C++: ROL::FDivergence<double>::operator=(const class ROL::FDivergence<double> &) --> class ROL::FDivergence<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Chi2Divergence file:ROL_Chi2Divergence.hpp line:73
		pybind11::class_<ROL::Chi2Divergence<double>, Teuchos::RCP<ROL::Chi2Divergence<double>>, PyCallBack_ROL_Chi2Divergence_double_t, ROL::FDivergence<double>> cl(M("ROL"), "Chi2Divergence_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double>(), pybind11::arg("thresh") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Chi2Divergence_double_t const &o){ return new PyCallBack_ROL_Chi2Divergence_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Chi2Divergence<double> const &o){ return new ROL::Chi2Divergence<double>(o); } ) );
		cl.def("Fprimal", [](ROL::Chi2Divergence<double> const &o, double const & a0) -> double { return o.Fprimal(a0); }, "", pybind11::arg("x"));
		cl.def("Fprimal", (double (ROL::Chi2Divergence<double>::*)(double, int) const) &ROL::Chi2Divergence<double>::Fprimal, "C++: ROL::Chi2Divergence<double>::Fprimal(double, int) const --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("Fdual", [](ROL::Chi2Divergence<double> const &o, double const & a0) -> double { return o.Fdual(a0); }, "", pybind11::arg("x"));
		cl.def("Fdual", (double (ROL::Chi2Divergence<double>::*)(double, int) const) &ROL::Chi2Divergence<double>::Fdual, "C++: ROL::Chi2Divergence<double>::Fdual(double, int) const --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("assign", (class ROL::Chi2Divergence<double> & (ROL::Chi2Divergence<double>::*)(const class ROL::Chi2Divergence<double> &)) &ROL::Chi2Divergence<double>::operator=, "C++: ROL::Chi2Divergence<double>::operator=(const class ROL::Chi2Divergence<double> &) --> class ROL::Chi2Divergence<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("Fprimal", [](ROL::FDivergence<double> const &o, double const & a0) -> double { return o.Fprimal(a0); }, "", pybind11::arg("x"));
		cl.def("Fprimal", (double (ROL::FDivergence<double>::*)(double, int) const) &ROL::FDivergence<double>::Fprimal, "C++: ROL::FDivergence<double>::Fprimal(double, int) const --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("Fdual", [](ROL::FDivergence<double> const &o, double const & a0) -> double { return o.Fdual(a0); }, "", pybind11::arg("x"));
		cl.def("Fdual", (double (ROL::FDivergence<double>::*)(double, int) const) &ROL::FDivergence<double>::Fdual, "C++: ROL::FDivergence<double>::Fdual(double, int) const --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", [](ROL::FDivergence<double> const &o) -> bool { return o.check(); }, "");
		cl.def("check", (bool (ROL::FDivergence<double>::*)(std::ostream &) const) &ROL::FDivergence<double>::check, "C++: ROL::FDivergence<double>::check(std::ostream &) const --> bool", pybind11::arg("outStream"));
		cl.def("initialize", (void (ROL::FDivergence<double>::*)(const class ROL::Vector<double> &)) &ROL::FDivergence<double>::initialize, "C++: ROL::FDivergence<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("updateValue", (void (ROL::FDivergence<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::FDivergence<double>::updateValue, "C++: ROL::FDivergence<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::FDivergence<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::FDivergence<double>::getValue, "C++: ROL::FDivergence<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::FDivergence<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::FDivergence<double>::updateGradient, "C++: ROL::FDivergence<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::FDivergence<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::FDivergence<double>::getGradient, "C++: ROL::FDivergence<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::FDivergence<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::FDivergence<double>::updateHessVec, "C++: ROL::FDivergence<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::FDivergence<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::FDivergence<double>::getHessVec, "C++: ROL::FDivergence<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::FDivergence<double> & (ROL::FDivergence<double>::*)(const class ROL::FDivergence<double> &)) &ROL::FDivergence<double>::operator=, "C++: ROL::FDivergence<double>::operator=(const class ROL::FDivergence<double> &) --> class ROL::FDivergence<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::KLDivergence file:ROL_KLDivergence.hpp line:81
		pybind11::class_<ROL::KLDivergence<double>, Teuchos::RCP<ROL::KLDivergence<double>>, PyCallBack_ROL_KLDivergence_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "KLDivergence_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](){ return new ROL::KLDivergence<double>(); }, [](){ return new PyCallBack_ROL_KLDivergence_double_t(); } ), "doc");
		cl.def( pybind11::init<const double>(), pybind11::arg("eps") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_KLDivergence_double_t const &o){ return new PyCallBack_ROL_KLDivergence_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::KLDivergence<double> const &o){ return new ROL::KLDivergence<double>(o); } ) );
		cl.def("initialize", (void (ROL::KLDivergence<double>::*)(const class ROL::Vector<double> &)) &ROL::KLDivergence<double>::initialize, "C++: ROL::KLDivergence<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("updateValue", (void (ROL::KLDivergence<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::KLDivergence<double>::updateValue, "C++: ROL::KLDivergence<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::KLDivergence<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::KLDivergence<double>::getValue, "C++: ROL::KLDivergence<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::KLDivergence<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::KLDivergence<double>::updateGradient, "C++: ROL::KLDivergence<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::KLDivergence<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::KLDivergence<double>::getGradient, "C++: ROL::KLDivergence<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::KLDivergence<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::KLDivergence<double>::updateHessVec, "C++: ROL::KLDivergence<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::KLDivergence<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::KLDivergence<double>::getHessVec, "C++: ROL::KLDivergence<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::KLDivergence<double> & (ROL::KLDivergence<double>::*)(const class ROL::KLDivergence<double> &)) &ROL::KLDivergence<double>::operator=, "C++: ROL::KLDivergence<double>::operator=(const class ROL::KLDivergence<double> &) --> class ROL::KLDivergence<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	// ROL::ERiskMeasure file:ROL_RiskMeasureFactory.hpp line:84
	pybind11::enum_<ROL::ERiskMeasure>(M("ROL"), "ERiskMeasure", pybind11::arithmetic(), "", pybind11::module_local())
		.value("RISKMEASURE_CVAR", ROL::RISKMEASURE_CVAR)
		.value("RISKMEASURE_MOREAUYOSIDACVAR", ROL::RISKMEASURE_MOREAUYOSIDACVAR)
		.value("RISKMEASURE_GENMOREAUYOSIDACVAR", ROL::RISKMEASURE_GENMOREAUYOSIDACVAR)
		.value("RISKMEASURE_MIXEDCVAR", ROL::RISKMEASURE_MIXEDCVAR)
		.value("RISKMEASURE_SPECTRALRISK", ROL::RISKMEASURE_SPECTRALRISK)
		.value("RISKMEASURE_SECONDORDERCVAR", ROL::RISKMEASURE_SECONDORDERCVAR)
		.value("RISKMEASURE_CHEBYSHEVSPECTRAL", ROL::RISKMEASURE_CHEBYSHEVSPECTRAL)
		.value("RISKMEASURE_QUANTILERADIUS", ROL::RISKMEASURE_QUANTILERADIUS)
		.value("RISKMEASURE_HMCR", ROL::RISKMEASURE_HMCR)
		.value("RISKMEASURE_ENTROPICRISK", ROL::RISKMEASURE_ENTROPICRISK)
		.value("RISKMEASURE_COHERENTENTROPICRISK", ROL::RISKMEASURE_COHERENTENTROPICRISK)
		.value("RISKMEASURE_MEANSEMIDEVIATION", ROL::RISKMEASURE_MEANSEMIDEVIATION)
		.value("RISKMEASURE_MEANSEMIDEVIATIONFROMTARGET", ROL::RISKMEASURE_MEANSEMIDEVIATIONFROMTARGET)
		.value("RISKMEASURE_MEANDEVIATIONFROMTARGET", ROL::RISKMEASURE_MEANDEVIATIONFROMTARGET)
		.value("RISKMEASURE_MEANDEVIATION", ROL::RISKMEASURE_MEANDEVIATION)
		.value("RISKMEASURE_MEANVARIANCEFROMTARGET", ROL::RISKMEASURE_MEANVARIANCEFROMTARGET)
		.value("RISKMEASURE_MEANVARIANCE", ROL::RISKMEASURE_MEANVARIANCE)
		.value("RISKMEASURE_TRUNCATEDMEAN", ROL::RISKMEASURE_TRUNCATEDMEAN)
		.value("RISKMEASURE_LOGQUANTILE", ROL::RISKMEASURE_LOGQUANTILE)
		.value("RISKMEASURE_SMOOTHEDWORSTCASE", ROL::RISKMEASURE_SMOOTHEDWORSTCASE)
		.value("RISKMEASURE_LOGEXPONENTIAL", ROL::RISKMEASURE_LOGEXPONENTIAL)
		.value("RISKMEASURE_SAFETYMARGIN", ROL::RISKMEASURE_SAFETYMARGIN)
		.value("RISKMEASURE_CHI2DIVERGENCE", ROL::RISKMEASURE_CHI2DIVERGENCE)
		.value("RISKMEASURE_KLDIVERGENCE", ROL::RISKMEASURE_KLDIVERGENCE)
		.value("RISKMEASURE_LAST", ROL::RISKMEASURE_LAST)
		.export_values();

;

	// ROL::ERiskMeasureToString(enum ROL::ERiskMeasure) file:ROL_RiskMeasureFactory.hpp line:112
	M("ROL").def("ERiskMeasureToString", (std::string (*)(enum ROL::ERiskMeasure)) &ROL::ERiskMeasureToString, "C++: ROL::ERiskMeasureToString(enum ROL::ERiskMeasure) --> std::string", pybind11::arg("ed"));

	// ROL::isValidRiskMeasure(enum ROL::ERiskMeasure) file:ROL_RiskMeasureFactory.hpp line:171
	M("ROL").def("isValidRiskMeasure", (int (*)(enum ROL::ERiskMeasure)) &ROL::isValidRiskMeasure, "C++: ROL::isValidRiskMeasure(enum ROL::ERiskMeasure) --> int", pybind11::arg("ed"));

	// ROL::StringToERiskMeasure(std::string) file:ROL_RiskMeasureFactory.hpp line:218
	M("ROL").def("StringToERiskMeasure", (enum ROL::ERiskMeasure (*)(std::string)) &ROL::StringToERiskMeasure, "C++: ROL::StringToERiskMeasure(std::string) --> enum ROL::ERiskMeasure", pybind11::arg("s"));

	// ROL::RiskMeasureFactory(class Teuchos::ParameterList &) file:ROL_RiskMeasureFactory.hpp line:229
	M("ROL").def("RiskMeasureFactory", (class Teuchos::RCP<class ROL::RandVarFunctional<double> > (*)(class Teuchos::ParameterList &)) &ROL::RiskMeasureFactory<double>, "C++: ROL::RiskMeasureFactory(class Teuchos::ParameterList &) --> class Teuchos::RCP<class ROL::RandVarFunctional<double> >", pybind11::arg("parlist"));

	{ // ROL::ExpectationQuadDeviation file:ROL_ExpectationQuadDeviation.hpp line:89
		pybind11::class_<ROL::ExpectationQuadDeviation<double>, Teuchos::RCP<ROL::ExpectationQuadDeviation<double>>, PyCallBack_ROL_ExpectationQuadDeviation_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "ExpectationQuadDeviation_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::ExpectationQuad<double> > &>(), pybind11::arg("eq") );

		cl.def( pybind11::init( [](PyCallBack_ROL_ExpectationQuadDeviation_double_t const &o){ return new PyCallBack_ROL_ExpectationQuadDeviation_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::ExpectationQuadDeviation<double> const &o){ return new ROL::ExpectationQuadDeviation<double>(o); } ) );
		cl.def("checkRegret", (void (ROL::ExpectationQuadDeviation<double>::*)()) &ROL::ExpectationQuadDeviation<double>::checkRegret, "C++: ROL::ExpectationQuadDeviation<double>::checkRegret() --> void");
		cl.def("updateValue", (void (ROL::ExpectationQuadDeviation<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadDeviation<double>::updateValue, "C++: ROL::ExpectationQuadDeviation<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::ExpectationQuadDeviation<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadDeviation<double>::updateGradient, "C++: ROL::ExpectationQuadDeviation<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::ExpectationQuadDeviation<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadDeviation<double>::updateHessVec, "C++: ROL::ExpectationQuadDeviation<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::ExpectationQuadDeviation<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadDeviation<double>::getValue, "C++: ROL::ExpectationQuadDeviation<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::ExpectationQuadDeviation<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadDeviation<double>::getGradient, "C++: ROL::ExpectationQuadDeviation<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::ExpectationQuadDeviation<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadDeviation<double>::getHessVec, "C++: ROL::ExpectationQuadDeviation<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::ExpectationQuadDeviation<double> & (ROL::ExpectationQuadDeviation<double>::*)(const class ROL::ExpectationQuadDeviation<double> &)) &ROL::ExpectationQuadDeviation<double>::operator=, "C++: ROL::ExpectationQuadDeviation<double>::operator=(const class ROL::ExpectationQuadDeviation<double> &) --> class ROL::ExpectationQuadDeviation<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::QuantileQuadrangle file:ROL_QuantileQuadrangle.hpp line:97
		pybind11::class_<ROL::QuantileQuadrangle<double>, Teuchos::RCP<ROL::QuantileQuadrangle<double>>, PyCallBack_ROL_QuantileQuadrangle_double_t, ROL::ExpectationQuad<double>> cl(M("ROL"), "QuantileQuadrangle_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<double, double, class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("prob"), pybind11::arg("eps"), pybind11::arg("pf") );

		cl.def( pybind11::init<double, double, double, class Teuchos::RCP<class ROL::PlusFunction<double> > &>(), pybind11::arg("prob"), pybind11::arg("lam"), pybind11::arg("eps"), pybind11::arg("pf") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_QuantileQuadrangle_double_t const &o){ return new PyCallBack_ROL_QuantileQuadrangle_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::QuantileQuadrangle<double> const &o){ return new ROL::QuantileQuadrangle<double>(o); } ) );
		cl.def("error", [](ROL::QuantileQuadrangle<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::QuantileQuadrangle<double>::*)(double, int)) &ROL::QuantileQuadrangle<double>::error, "C++: ROL::QuantileQuadrangle<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("regret", [](ROL::QuantileQuadrangle<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::QuantileQuadrangle<double>::*)(double, int)) &ROL::QuantileQuadrangle<double>::regret, "C++: ROL::QuantileQuadrangle<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::QuantileQuadrangle<double>::*)()) &ROL::QuantileQuadrangle<double>::check, "C++: ROL::QuantileQuadrangle<double>::check() --> void");
		cl.def("assign", (class ROL::QuantileQuadrangle<double> & (ROL::QuantileQuadrangle<double>::*)(const class ROL::QuantileQuadrangle<double> &)) &ROL::QuantileQuadrangle<double>::operator=, "C++: ROL::QuantileQuadrangle<double>::operator=(const class ROL::QuantileQuadrangle<double> &) --> class ROL::QuantileQuadrangle<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("regret", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.regret(a0); }, "", pybind11::arg("x"));
		cl.def("regret", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::regret, "C++: ROL::ExpectationQuad<double>::regret(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("error", [](ROL::ExpectationQuad<double> &o, double const & a0) -> double { return o.error(a0); }, "", pybind11::arg("x"));
		cl.def("error", (double (ROL::ExpectationQuad<double>::*)(double, int)) &ROL::ExpectationQuad<double>::error, "C++: ROL::ExpectationQuad<double>::error(double, int) --> double", pybind11::arg("x"), pybind11::arg("deriv"));
		cl.def("check", (void (ROL::ExpectationQuad<double>::*)()) &ROL::ExpectationQuad<double>::check, "C++: ROL::ExpectationQuad<double>::check() --> void");
		cl.def("assign", (class ROL::ExpectationQuad<double> & (ROL::ExpectationQuad<double>::*)(const class ROL::ExpectationQuad<double> &)) &ROL::ExpectationQuad<double>::operator=, "C++: ROL::ExpectationQuad<double>::operator=(const class ROL::ExpectationQuad<double> &) --> class ROL::ExpectationQuad<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	// ROL::EDeviationMeasure file:ROL_DeviationMeasureFactory.hpp line:64
	pybind11::enum_<ROL::EDeviationMeasure>(M("ROL"), "EDeviationMeasure", pybind11::arithmetic(), "", pybind11::module_local())
		.value("DEVIATIONMEASURE_MEANVARIANCEQUADRANGLE", ROL::DEVIATIONMEASURE_MEANVARIANCEQUADRANGLE)
		.value("DEVIATIONMEASURE_TRUNCATEDMEANQUADRANGLE", ROL::DEVIATIONMEASURE_TRUNCATEDMEANQUADRANGLE)
		.value("DEVIATIONMEASURE_QUANTILEQUADRANGLE", ROL::DEVIATIONMEASURE_QUANTILEQUADRANGLE)
		.value("DEVIATIONMEASURE_MOREAUYOSIDACVAR", ROL::DEVIATIONMEASURE_MOREAUYOSIDACVAR)
		.value("DEVIATIONMEASURE_GENMOREAUYOSIDACVAR", ROL::DEVIATIONMEASURE_GENMOREAUYOSIDACVAR)
		.value("DEVIATIONMEASURE_LOGEXPONENTIALQUADRANGLE", ROL::DEVIATIONMEASURE_LOGEXPONENTIALQUADRANGLE)
		.value("DEVIATIONMEASURE_LOGQUANTILEQUADRANGLE", ROL::DEVIATIONMEASURE_LOGQUANTILEQUADRANGLE)
		.value("DEVIATIONMEASURE_SMOOTHEDWORSTCASEQUADRANGLE", ROL::DEVIATIONMEASURE_SMOOTHEDWORSTCASEQUADRANGLE)
		.value("DEVIATIONMEASURE_LAST", ROL::DEVIATIONMEASURE_LAST)
		.export_values();

;

	// ROL::EDeviationMeasureToString(enum ROL::EDeviationMeasure) file:ROL_DeviationMeasureFactory.hpp line:76
	M("ROL").def("EDeviationMeasureToString", (std::string (*)(enum ROL::EDeviationMeasure)) &ROL::EDeviationMeasureToString, "C++: ROL::EDeviationMeasureToString(enum ROL::EDeviationMeasure) --> std::string", pybind11::arg("ed"));

	// ROL::isValidDeviationMeasure(enum ROL::EDeviationMeasure) file:ROL_DeviationMeasureFactory.hpp line:103
	M("ROL").def("isValidDeviationMeasure", (int (*)(enum ROL::EDeviationMeasure)) &ROL::isValidDeviationMeasure, "C++: ROL::isValidDeviationMeasure(enum ROL::EDeviationMeasure) --> int", pybind11::arg("ed"));

	// ROL::StringToEDeviationMeasure(std::string) file:ROL_DeviationMeasureFactory.hpp line:134
	M("ROL").def("StringToEDeviationMeasure", (enum ROL::EDeviationMeasure (*)(std::string)) &ROL::StringToEDeviationMeasure, "C++: ROL::StringToEDeviationMeasure(std::string) --> enum ROL::EDeviationMeasure", pybind11::arg("s"));

	// ROL::DeviationMeasureFactory(class Teuchos::ParameterList &) file:ROL_DeviationMeasureFactory.hpp line:145
	M("ROL").def("DeviationMeasureFactory", (class Teuchos::RCP<class ROL::RandVarFunctional<double> > (*)(class Teuchos::ParameterList &)) &ROL::DeviationMeasureFactory<double>, "C++: ROL::DeviationMeasureFactory(class Teuchos::ParameterList &) --> class Teuchos::RCP<class ROL::RandVarFunctional<double> >", pybind11::arg("parlist"));

	{ // ROL::ExpectationQuadError file:ROL_ExpectationQuadError.hpp line:89
		pybind11::class_<ROL::ExpectationQuadError<double>, Teuchos::RCP<ROL::ExpectationQuadError<double>>, PyCallBack_ROL_ExpectationQuadError_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "ExpectationQuadError_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::ExpectationQuad<double> > &>(), pybind11::arg("eq") );

		cl.def( pybind11::init( [](PyCallBack_ROL_ExpectationQuadError_double_t const &o){ return new PyCallBack_ROL_ExpectationQuadError_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::ExpectationQuadError<double> const &o){ return new ROL::ExpectationQuadError<double>(o); } ) );
		cl.def("checkRegret", (void (ROL::ExpectationQuadError<double>::*)()) &ROL::ExpectationQuadError<double>::checkRegret, "C++: ROL::ExpectationQuadError<double>::checkRegret() --> void");
		cl.def("updateValue", (void (ROL::ExpectationQuadError<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadError<double>::updateValue, "C++: ROL::ExpectationQuadError<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::ExpectationQuadError<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadError<double>::updateGradient, "C++: ROL::ExpectationQuadError<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::ExpectationQuadError<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadError<double>::updateHessVec, "C++: ROL::ExpectationQuadError<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::ExpectationQuadError<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadError<double>::getValue, "C++: ROL::ExpectationQuadError<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::ExpectationQuadError<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadError<double>::getGradient, "C++: ROL::ExpectationQuadError<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::ExpectationQuadError<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadError<double>::getHessVec, "C++: ROL::ExpectationQuadError<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::ExpectationQuadError<double> & (ROL::ExpectationQuadError<double>::*)(const class ROL::ExpectationQuadError<double> &)) &ROL::ExpectationQuadError<double>::operator=, "C++: ROL::ExpectationQuadError<double>::operator=(const class ROL::ExpectationQuadError<double> &) --> class ROL::ExpectationQuadError<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	// ROL::EErrorMeasure file:ROL_ErrorMeasureFactory.hpp line:64
	pybind11::enum_<ROL::EErrorMeasure>(M("ROL"), "EErrorMeasure", pybind11::arithmetic(), "", pybind11::module_local())
		.value("ERRORMEASURE_MEANVARIANCEQUADRANGLE", ROL::ERRORMEASURE_MEANVARIANCEQUADRANGLE)
		.value("ERRORMEASURE_TRUNCATEDMEANQUADRANGLE", ROL::ERRORMEASURE_TRUNCATEDMEANQUADRANGLE)
		.value("ERRORMEASURE_QUANTILEQUADRANGLE", ROL::ERRORMEASURE_QUANTILEQUADRANGLE)
		.value("ERRORMEASURE_MOREAUYOSIDACVAR", ROL::ERRORMEASURE_MOREAUYOSIDACVAR)
		.value("ERRORMEASURE_GENMOREAUYOSIDACVAR", ROL::ERRORMEASURE_GENMOREAUYOSIDACVAR)
		.value("ERRORMEASURE_LOGEXPONENTIALQUADRANGLE", ROL::ERRORMEASURE_LOGEXPONENTIALQUADRANGLE)
		.value("ERRORMEASURE_LOGQUANTILEQUADRANGLE", ROL::ERRORMEASURE_LOGQUANTILEQUADRANGLE)
		.value("ERRORMEASURE_SMOOTHEDWORSTCASEQUADRANGLE", ROL::ERRORMEASURE_SMOOTHEDWORSTCASEQUADRANGLE)
		.value("ERRORMEASURE_LAST", ROL::ERRORMEASURE_LAST)
		.export_values();

;

	// ROL::EErrorMeasureToString(enum ROL::EErrorMeasure) file:ROL_ErrorMeasureFactory.hpp line:76
	M("ROL").def("EErrorMeasureToString", (std::string (*)(enum ROL::EErrorMeasure)) &ROL::EErrorMeasureToString, "C++: ROL::EErrorMeasureToString(enum ROL::EErrorMeasure) --> std::string", pybind11::arg("ed"));

	// ROL::isValidErrorMeasure(enum ROL::EErrorMeasure) file:ROL_ErrorMeasureFactory.hpp line:103
	M("ROL").def("isValidErrorMeasure", (int (*)(enum ROL::EErrorMeasure)) &ROL::isValidErrorMeasure, "C++: ROL::isValidErrorMeasure(enum ROL::EErrorMeasure) --> int", pybind11::arg("ed"));

	// ROL::StringToEErrorMeasure(std::string) file:ROL_ErrorMeasureFactory.hpp line:134
	M("ROL").def("StringToEErrorMeasure", (enum ROL::EErrorMeasure (*)(std::string)) &ROL::StringToEErrorMeasure, "C++: ROL::StringToEErrorMeasure(std::string) --> enum ROL::EErrorMeasure", pybind11::arg("s"));

	// ROL::ErrorMeasureFactory(class Teuchos::ParameterList &) file:ROL_ErrorMeasureFactory.hpp line:145
	M("ROL").def("ErrorMeasureFactory", (class Teuchos::RCP<class ROL::RandVarFunctional<double> > (*)(class Teuchos::ParameterList &)) &ROL::ErrorMeasureFactory<double>, "C++: ROL::ErrorMeasureFactory(class Teuchos::ParameterList &) --> class Teuchos::RCP<class ROL::RandVarFunctional<double> >", pybind11::arg("parlist"));

	{ // ROL::ExpectationQuadRegret file:ROL_ExpectationQuadRegret.hpp line:89
		pybind11::class_<ROL::ExpectationQuadRegret<double>, Teuchos::RCP<ROL::ExpectationQuadRegret<double>>, PyCallBack_ROL_ExpectationQuadRegret_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "ExpectationQuadRegret_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::ExpectationQuad<double> > &>(), pybind11::arg("eq") );

		cl.def( pybind11::init( [](PyCallBack_ROL_ExpectationQuadRegret_double_t const &o){ return new PyCallBack_ROL_ExpectationQuadRegret_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::ExpectationQuadRegret<double> const &o){ return new ROL::ExpectationQuadRegret<double>(o); } ) );
		cl.def("checkRegret", (void (ROL::ExpectationQuadRegret<double>::*)()) &ROL::ExpectationQuadRegret<double>::checkRegret, "C++: ROL::ExpectationQuadRegret<double>::checkRegret() --> void");
		cl.def("updateValue", (void (ROL::ExpectationQuadRegret<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadRegret<double>::updateValue, "C++: ROL::ExpectationQuadRegret<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::ExpectationQuadRegret<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadRegret<double>::updateGradient, "C++: ROL::ExpectationQuadRegret<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::ExpectationQuadRegret<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ExpectationQuadRegret<double>::updateHessVec, "C++: ROL::ExpectationQuadRegret<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::ExpectationQuadRegret<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadRegret<double>::getValue, "C++: ROL::ExpectationQuadRegret<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::ExpectationQuadRegret<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadRegret<double>::getGradient, "C++: ROL::ExpectationQuadRegret<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::ExpectationQuadRegret<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ExpectationQuadRegret<double>::getHessVec, "C++: ROL::ExpectationQuadRegret<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::ExpectationQuadRegret<double> & (ROL::ExpectationQuadRegret<double>::*)(const class ROL::ExpectationQuadRegret<double> &)) &ROL::ExpectationQuadRegret<double>::operator=, "C++: ROL::ExpectationQuadRegret<double>::operator=(const class ROL::ExpectationQuadRegret<double> &) --> class ROL::ExpectationQuadRegret<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	// ROL::ERegretMeasure file:ROL_RegretMeasureFactory.hpp line:61
	pybind11::enum_<ROL::ERegretMeasure>(M("ROL"), "ERegretMeasure", pybind11::arithmetic(), "", pybind11::module_local())
		.value("REGRETMEASURE_MEANABSOLUTELOSS", ROL::REGRETMEASURE_MEANABSOLUTELOSS)
		.value("REGRETMEASURE_MOREAUYOSIDAMEANABSOLUTELOSS", ROL::REGRETMEASURE_MOREAUYOSIDAMEANABSOLUTELOSS)
		.value("REGRETMEASURE_GENMOREAUYOSIDAMEANABSOLUTELOSS", ROL::REGRETMEASURE_GENMOREAUYOSIDAMEANABSOLUTELOSS)
		.value("REGRETMEASURE_EXPONENTIAL", ROL::REGRETMEASURE_EXPONENTIAL)
		.value("REGRETMEASURE_MEANL2", ROL::REGRETMEASURE_MEANL2)
		.value("REGRETMEASURE_TRUNCATEDMEAN", ROL::REGRETMEASURE_TRUNCATEDMEAN)
		.value("REGRETMEASURE_LOGQUANTILE", ROL::REGRETMEASURE_LOGQUANTILE)
		.value("REGRETMEASURE_SMOOTHEDWORSTCASE", ROL::REGRETMEASURE_SMOOTHEDWORSTCASE)
		.value("REGRETMEASURE_LAST", ROL::REGRETMEASURE_LAST)
		.export_values();

;

	// ROL::ERegretMeasureToString(enum ROL::ERegretMeasure) file:ROL_RegretMeasureFactory.hpp line:73
	M("ROL").def("ERegretMeasureToString", (std::string (*)(enum ROL::ERegretMeasure)) &ROL::ERegretMeasureToString, "C++: ROL::ERegretMeasureToString(enum ROL::ERegretMeasure) --> std::string", pybind11::arg("ed"));

	// ROL::isValidRegretMeasure(enum ROL::ERegretMeasure) file:ROL_RegretMeasureFactory.hpp line:100
	M("ROL").def("isValidRegretMeasure", (int (*)(enum ROL::ERegretMeasure)) &ROL::isValidRegretMeasure, "C++: ROL::isValidRegretMeasure(enum ROL::ERegretMeasure) --> int", pybind11::arg("ed"));

	// ROL::StringToERegretMeasure(std::string) file:ROL_RegretMeasureFactory.hpp line:131
	M("ROL").def("StringToERegretMeasure", (enum ROL::ERegretMeasure (*)(std::string)) &ROL::StringToERegretMeasure, "C++: ROL::StringToERegretMeasure(std::string) --> enum ROL::ERegretMeasure", pybind11::arg("s"));

	// ROL::RegretMeasureFactory(class Teuchos::ParameterList &) file:ROL_RegretMeasureFactory.hpp line:142
	M("ROL").def("RegretMeasureFactory", (class Teuchos::RCP<class ROL::RandVarFunctional<double> > (*)(class Teuchos::ParameterList &)) &ROL::RegretMeasureFactory<double>, "C++: ROL::RegretMeasureFactory(class Teuchos::ParameterList &) --> class Teuchos::RCP<class ROL::RandVarFunctional<double> >", pybind11::arg("parlist"));

	{ // ROL::BPOE file:ROL_BPOE.hpp line:66
		pybind11::class_<ROL::BPOE<double>, Teuchos::RCP<ROL::BPOE<double>>, PyCallBack_ROL_BPOE_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "BPOE_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](const double & a0){ return new ROL::BPOE<double>(a0); }, [](const double & a0){ return new PyCallBack_ROL_BPOE_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("threshold"), pybind11::arg("order") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_BPOE_double_t const &o){ return new PyCallBack_ROL_BPOE_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::BPOE<double> const &o){ return new ROL::BPOE<double>(o); } ) );
		cl.def("initialize", (void (ROL::BPOE<double>::*)(const class ROL::Vector<double> &)) &ROL::BPOE<double>::initialize, "C++: ROL::BPOE<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("updateValue", (void (ROL::BPOE<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::BPOE<double>::updateValue, "C++: ROL::BPOE<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::BPOE<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::BPOE<double>::getValue, "C++: ROL::BPOE<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::BPOE<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::BPOE<double>::updateGradient, "C++: ROL::BPOE<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::BPOE<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::BPOE<double>::getGradient, "C++: ROL::BPOE<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::BPOE<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::BPOE<double>::updateHessVec, "C++: ROL::BPOE<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::BPOE<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::BPOE<double>::getHessVec, "C++: ROL::BPOE<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::BPOE<double> & (ROL::BPOE<double>::*)(const class ROL::BPOE<double> &)) &ROL::BPOE<double>::operator=, "C++: ROL::BPOE<double>::operator=(const class ROL::BPOE<double> &) --> class ROL::BPOE<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::SmoothedPOE file:ROL_SmoothedPOE.hpp line:66
		pybind11::class_<ROL::SmoothedPOE<double>, Teuchos::RCP<ROL::SmoothedPOE<double>>, PyCallBack_ROL_SmoothedPOE_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "SmoothedPOE_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const double, const double>(), pybind11::arg("threshold"), pybind11::arg("eps") );

		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_SmoothedPOE_double_t const &o){ return new PyCallBack_ROL_SmoothedPOE_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::SmoothedPOE<double> const &o){ return new ROL::SmoothedPOE<double>(o); } ) );
		cl.def("updateValue", (void (ROL::SmoothedPOE<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SmoothedPOE<double>::updateValue, "C++: ROL::SmoothedPOE<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::SmoothedPOE<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SmoothedPOE<double>::getValue, "C++: ROL::SmoothedPOE<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::SmoothedPOE<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SmoothedPOE<double>::updateGradient, "C++: ROL::SmoothedPOE<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::SmoothedPOE<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SmoothedPOE<double>::getGradient, "C++: ROL::SmoothedPOE<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::SmoothedPOE<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::SmoothedPOE<double>::updateHessVec, "C++: ROL::SmoothedPOE<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::SmoothedPOE<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::SmoothedPOE<double>::getHessVec, "C++: ROL::SmoothedPOE<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::SmoothedPOE<double> & (ROL::SmoothedPOE<double>::*)(const class ROL::SmoothedPOE<double> &)) &ROL::SmoothedPOE<double>::operator=, "C++: ROL::SmoothedPOE<double>::operator=(const class ROL::SmoothedPOE<double> &) --> class ROL::SmoothedPOE<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	// ROL::EProbability file:ROL_ProbabilityFactory.hpp line:55
	pybind11::enum_<ROL::EProbability>(M("ROL"), "EProbability", pybind11::arithmetic(), "", pybind11::module_local())
		.value("PROBABILITY_BPOE", ROL::PROBABILITY_BPOE)
		.value("PROBABILITY_SMOOTHEDPOE", ROL::PROBABILITY_SMOOTHEDPOE)
		.value("PROBABILITY_LAST", ROL::PROBABILITY_LAST)
		.export_values();

;

	// ROL::EProbabilityToString(enum ROL::EProbability) file:ROL_ProbabilityFactory.hpp line:61
	M("ROL").def("EProbabilityToString", (std::string (*)(enum ROL::EProbability)) &ROL::EProbabilityToString, "C++: ROL::EProbabilityToString(enum ROL::EProbability) --> std::string", pybind11::arg("ed"));

	// ROL::isValidProbability(enum ROL::EProbability) file:ROL_ProbabilityFactory.hpp line:76
	M("ROL").def("isValidProbability", (int (*)(enum ROL::EProbability)) &ROL::isValidProbability, "C++: ROL::isValidProbability(enum ROL::EProbability) --> int", pybind11::arg("ed"));

	// ROL::StringToEProbability(std::string) file:ROL_ProbabilityFactory.hpp line:101
	M("ROL").def("StringToEProbability", (enum ROL::EProbability (*)(std::string)) &ROL::StringToEProbability, "C++: ROL::StringToEProbability(std::string) --> enum ROL::EProbability", pybind11::arg("s"));

	// ROL::ProbabilityFactory(class Teuchos::ParameterList &) file:ROL_ProbabilityFactory.hpp line:112
	M("ROL").def("ProbabilityFactory", (class Teuchos::RCP<class ROL::RandVarFunctional<double> > (*)(class Teuchos::ParameterList &)) &ROL::ProbabilityFactory<double>, "C++: ROL::ProbabilityFactory(class Teuchos::ParameterList &) --> class Teuchos::RCP<class ROL::RandVarFunctional<double> >", pybind11::arg("parlist"));

	// ROL::RandVarFunctionalFactory(class Teuchos::ParameterList &) file:ROL_RandVarFunctionalFactory.hpp line:56
	M("ROL").def("RandVarFunctionalFactory", (class Teuchos::RCP<class ROL::RandVarFunctional<double> > (*)(class Teuchos::ParameterList &)) &ROL::RandVarFunctionalFactory<double>, "C++: ROL::RandVarFunctionalFactory(class Teuchos::ParameterList &) --> class Teuchos::RCP<class ROL::RandVarFunctional<double> >", pybind11::arg("parlist"));

	{ // ROL::ConvexCombinationRiskMeasure file:ROL_ConvexCombinationRiskMeasure.hpp line:68
		pybind11::class_<ROL::ConvexCombinationRiskMeasure<double>, Teuchos::RCP<ROL::ConvexCombinationRiskMeasure<double>>, PyCallBack_ROL_ConvexCombinationRiskMeasure_double_t, ROL::RandVarFunctional<double>> cl(M("ROL"), "ConvexCombinationRiskMeasure_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<class Teuchos::ParameterList &>(), pybind11::arg("parlist") );

		cl.def( pybind11::init( [](PyCallBack_ROL_ConvexCombinationRiskMeasure_double_t const &o){ return new PyCallBack_ROL_ConvexCombinationRiskMeasure_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::ConvexCombinationRiskMeasure<double> const &o){ return new ROL::ConvexCombinationRiskMeasure<double>(o); } ) );
		cl.def("setSample", (void (ROL::ConvexCombinationRiskMeasure<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::ConvexCombinationRiskMeasure<double>::setSample, "C++: ROL::ConvexCombinationRiskMeasure<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("resetStorage", [](ROL::ConvexCombinationRiskMeasure<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::ConvexCombinationRiskMeasure<double>::*)(bool)) &ROL::ConvexCombinationRiskMeasure<double>::resetStorage, "C++: ROL::ConvexCombinationRiskMeasure<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::ConvexCombinationRiskMeasure<double>::*)(enum ROL::UpdateType)) &ROL::ConvexCombinationRiskMeasure<double>::resetStorage, "C++: ROL::ConvexCombinationRiskMeasure<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::ConvexCombinationRiskMeasure<double>::*)(const class ROL::Vector<double> &)) &ROL::ConvexCombinationRiskMeasure<double>::initialize, "C++: ROL::ConvexCombinationRiskMeasure<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("updateValue", (void (ROL::ConvexCombinationRiskMeasure<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ConvexCombinationRiskMeasure<double>::updateValue, "C++: ROL::ConvexCombinationRiskMeasure<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::ConvexCombinationRiskMeasure<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ConvexCombinationRiskMeasure<double>::getValue, "C++: ROL::ConvexCombinationRiskMeasure<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateGradient", (void (ROL::ConvexCombinationRiskMeasure<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ConvexCombinationRiskMeasure<double>::updateGradient, "C++: ROL::ConvexCombinationRiskMeasure<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getGradient", (void (ROL::ConvexCombinationRiskMeasure<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ConvexCombinationRiskMeasure<double>::getGradient, "C++: ROL::ConvexCombinationRiskMeasure<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("updateHessVec", (void (ROL::ConvexCombinationRiskMeasure<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::ConvexCombinationRiskMeasure<double>::updateHessVec, "C++: ROL::ConvexCombinationRiskMeasure<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getHessVec", (void (ROL::ConvexCombinationRiskMeasure<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::ConvexCombinationRiskMeasure<double>::getHessVec, "C++: ROL::ConvexCombinationRiskMeasure<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::ConvexCombinationRiskMeasure<double> & (ROL::ConvexCombinationRiskMeasure<double>::*)(const class ROL::ConvexCombinationRiskMeasure<double> &)) &ROL::ConvexCombinationRiskMeasure<double>::operator=, "C++: ROL::ConvexCombinationRiskMeasure<double>::operator=(const class ROL::ConvexCombinationRiskMeasure<double> &) --> class ROL::ConvexCombinationRiskMeasure<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("useStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useStorage, "C++: ROL::RandVarFunctional<double>::useStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("useHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::useHessVecStorage, "C++: ROL::RandVarFunctional<double>::useHessVecStorage(bool) --> void", pybind11::arg("storage"));
		cl.def("setStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setStorage, "C++: ROL::RandVarFunctional<double>::setStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("value_storage"), pybind11::arg("gradient_storage"));
		cl.def("setHessVecStorage", (void (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &)) &ROL::RandVarFunctional<double>::setHessVecStorage, "C++: ROL::RandVarFunctional<double>::setHessVecStorage(const class Teuchos::RCP<class ROL::ScalarController<double, class std::vector<double, class std::allocator<double> > > > &, const class Teuchos::RCP<class ROL::VectorController<double, class std::vector<double, class std::allocator<double> > > > &) --> void", pybind11::arg("gradvec_storage"), pybind11::arg("hessvec_storage"));
		cl.def("resetStorage", [](ROL::RandVarFunctional<double> &o) -> void { return o.resetStorage(); }, "");
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(bool)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(bool) --> void", pybind11::arg("flag"));
		cl.def("resetStorage", (void (ROL::RandVarFunctional<double>::*)(enum ROL::UpdateType)) &ROL::RandVarFunctional<double>::resetStorage, "C++: ROL::RandVarFunctional<double>::resetStorage(enum ROL::UpdateType) --> void", pybind11::arg("type"));
		cl.def("initialize", (void (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &)) &ROL::RandVarFunctional<double>::initialize, "C++: ROL::RandVarFunctional<double>::initialize(const class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("setSample", (void (ROL::RandVarFunctional<double>::*)(const class std::vector<double, class std::allocator<double> > &, const double)) &ROL::RandVarFunctional<double>::setSample, "C++: ROL::RandVarFunctional<double>::setSample(const class std::vector<double, class std::allocator<double> > &, const double) --> void", pybind11::arg("point"), pybind11::arg("weight"));
		cl.def("computeStatistic", (double (ROL::RandVarFunctional<double>::*)(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const) &ROL::RandVarFunctional<double>::computeStatistic, "C++: ROL::RandVarFunctional<double>::computeStatistic(const class Teuchos::RCP<const class std::vector<double, class std::allocator<double> > > &) const --> double", pybind11::arg("xstat"));
		cl.def("updateValue", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateValue, "C++: ROL::RandVarFunctional<double>::updateValue(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateGradient, "C++: ROL::RandVarFunctional<double>::updateGradient(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("updateHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &)) &ROL::RandVarFunctional<double>::updateHessVec, "C++: ROL::RandVarFunctional<double>::updateHessVec(class ROL::Objective<double> &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, double &) --> void", pybind11::arg("obj"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("tol"));
		cl.def("getValue", (double (ROL::RandVarFunctional<double>::*)(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getValue, "C++: ROL::RandVarFunctional<double>::getValue(const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> double", pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getGradient", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getGradient, "C++: ROL::RandVarFunctional<double>::getGradient(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("g"), pybind11::arg("gstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("getHessVec", (void (ROL::RandVarFunctional<double>::*)(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &)) &ROL::RandVarFunctional<double>::getHessVec, "C++: ROL::RandVarFunctional<double>::getHessVec(class ROL::Vector<double> &, class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, const class ROL::Vector<double> &, const class std::vector<double, class std::allocator<double> > &, class ROL::SampleGenerator<double> &) --> void", pybind11::arg("hv"), pybind11::arg("hvstat"), pybind11::arg("v"), pybind11::arg("vstat"), pybind11::arg("x"), pybind11::arg("xstat"), pybind11::arg("sampler"));
		cl.def("assign", (class ROL::RandVarFunctional<double> & (ROL::RandVarFunctional<double>::*)(const class ROL::RandVarFunctional<double> &)) &ROL::RandVarFunctional<double>::operator=, "C++: ROL::RandVarFunctional<double>::operator=(const class ROL::RandVarFunctional<double> &) --> class ROL::RandVarFunctional<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::StochasticObjective file:ROL_StochasticObjective.hpp line:56
		pybind11::class_<ROL::StochasticObjective<double>, Teuchos::RCP<ROL::StochasticObjective<double>>, PyCallBack_ROL_StochasticObjective_double_t, ROL::Objective<double>> cl(M("ROL"), "StochasticObjective_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a4){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3, a4); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a4){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3, a4); } ), "doc");
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a4, const bool & a5){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3, a4, a5); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a4, const bool & a5){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3, a4, a5); } ), "doc");
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a4, const bool & a5, const int & a6){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3, a4, a5, a6); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a4, const bool & a5, const int & a6){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3, a4, a5, a6); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const bool, const int, const int>(), pybind11::arg("obj"), pybind11::arg("rvf"), pybind11::arg("vsampler"), pybind11::arg("gsampler"), pybind11::arg("hsampler"), pybind11::arg("storage"), pybind11::arg("comp"), pybind11::arg("index") );

		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3); } ), "doc");
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const bool & a4){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3, a4); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const bool & a4){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3, a4); } ), "doc");
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const bool & a4, const int & a5){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3, a4, a5); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const bool & a4, const int & a5){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3, a4, a5); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const bool, const int, const int>(), pybind11::arg("obj"), pybind11::arg("rvf"), pybind11::arg("vsampler"), pybind11::arg("gsampler"), pybind11::arg("storage"), pybind11::arg("comp"), pybind11::arg("index") );

		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2){ return new ROL::StochasticObjective<double>(a0, a1, a2); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2); } ), "doc");
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const bool & a3){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const bool & a3){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3); } ), "doc");
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const bool & a3, const int & a4){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3, a4); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const bool & a3, const int & a4){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3, a4); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, const class Teuchos::RCP<class ROL::RandVarFunctional<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const bool, const int, const int>(), pybind11::arg("obj"), pybind11::arg("rvf"), pybind11::arg("sampler"), pybind11::arg("storage"), pybind11::arg("comp"), pybind11::arg("index") );

		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a4){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3, a4); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a4){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3, a4); } ), "doc");
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a4, const int & a5){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3, a4, a5); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a4, const int & a5){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3, a4, a5); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, class Teuchos::ParameterList &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const int, const int>(), pybind11::arg("obj"), pybind11::arg("parlist"), pybind11::arg("vsampler"), pybind11::arg("gsampler"), pybind11::arg("hsampler"), pybind11::arg("comp"), pybind11::arg("index") );

		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3); } ), "doc");
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const int & a4){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3, a4); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a3, const int & a4){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3, a4); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, class Teuchos::ParameterList &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const int, const int>(), pybind11::arg("obj"), pybind11::arg("parlist"), pybind11::arg("vsampler"), pybind11::arg("gsampler"), pybind11::arg("comp"), pybind11::arg("index") );

		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2){ return new ROL::StochasticObjective<double>(a0, a1, a2); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2); } ), "doc");
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const int & a3){ return new ROL::StochasticObjective<double>(a0, a1, a2, a3); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, class Teuchos::ParameterList & a1, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a2, const int & a3){ return new PyCallBack_ROL_StochasticObjective_double_t(a0, a1, a2, a3); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, class Teuchos::ParameterList &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, const int, const int>(), pybind11::arg("obj"), pybind11::arg("parlist"), pybind11::arg("sampler"), pybind11::arg("comp"), pybind11::arg("index") );

		cl.def( pybind11::init( [](PyCallBack_ROL_StochasticObjective_double_t const &o){ return new PyCallBack_ROL_StochasticObjective_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::StochasticObjective<double> const &o){ return new ROL::StochasticObjective<double>(o); } ) );
		cl.def("computeStatistic", (double (ROL::StochasticObjective<double>::*)(const class ROL::Vector<double> &) const) &ROL::StochasticObjective<double>::computeStatistic, "C++: ROL::StochasticObjective<double>::computeStatistic(const class ROL::Vector<double> &) const --> double", pybind11::arg("x"));
		cl.def("setIndex", (void (ROL::StochasticObjective<double>::*)(int)) &ROL::StochasticObjective<double>::setIndex, "C++: ROL::StochasticObjective<double>::setIndex(int) --> void", pybind11::arg("ind"));
		cl.def("update", [](ROL::StochasticObjective<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::StochasticObjective<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::StochasticObjective<double>::update, "C++: ROL::StochasticObjective<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::StochasticObjective<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::StochasticObjective<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::StochasticObjective<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::StochasticObjective<double>::update, "C++: ROL::StochasticObjective<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (double (ROL::StochasticObjective<double>::*)(const class ROL::Vector<double> &, double &)) &ROL::StochasticObjective<double>::value, "C++: ROL::StochasticObjective<double>::value(const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("gradient", (void (ROL::StochasticObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::StochasticObjective<double>::gradient, "C++: ROL::StochasticObjective<double>::gradient(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("hessVec", (void (ROL::StochasticObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::StochasticObjective<double>::hessVec, "C++: ROL::StochasticObjective<double>::hessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("precond", (void (ROL::StochasticObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::StochasticObjective<double>::precond, "C++: ROL::StochasticObjective<double>::precond(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::Objective<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::Objective<double>::update, "Update objective function. \n\n      This function updates the objective function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is the type of update requested.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Objective<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::Objective<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::Objective<double>::update, "Update objective function. \n\n      This function updates the objective function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is true if the iterate has changed.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Objective<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (double (ROL::Objective<double>::*)(const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::value, "C++: ROL::Objective<double>::value(const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("gradient", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::gradient, "Compute gradient.\n\n      This function returns the objective function gradient.\n      \n\n   is the gradient.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\n      The default implementation is a finite-difference approximation based on the function value.\n      This requires the definition of a basis \n\n for the optimization vectors x and\n      the definition of a basis \n\n for the dual optimization vectors (gradient vectors g).\n      The bases must be related through the Riesz map, i.e., \n\n,\n      and this must be reflected in the implementation of the ROL::Vector::dual() method.\n\nC++: ROL::Objective<double>::gradient(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("dirDeriv", (double (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::dirDeriv, "Compute directional derivative.\n\n      This function returns the directional derivative of the objective function in the \n direction.\n      \n\n   is the current iterate.\n      \n\n   is the direction.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::dirDeriv(const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("d"), pybind11::arg("tol"));
		cl.def("hessVec", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::hessVec, "Apply Hessian approximation to vector.\n\n      This function applies the Hessian of the objective function to the vector \n.\n      \n\n  is the the action of the Hessian on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::hessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("invHessVec", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::invHessVec, "Apply inverse Hessian approximation to vector.\n\n      This function applies the inverse Hessian of the objective function to the vector \n.\n      \n\n  is the action of the inverse Hessian on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::invHessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("precond", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::precond, "Apply preconditioner to vector.\n\n      This function applies a preconditioner for the Hessian of the objective function to the vector \n.\n      \n\n  is the action of the Hessian preconditioner on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::precond(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("prox", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double &)) &ROL::Objective<double>::prox, "C++: ROL::Objective<double>::prox(class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("t"), pybind11::arg("tol"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2); }, "", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool & a3) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3); }, "", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"));
		cl.def("checkHessSym", (class std::vector<double, class std::allocator<double> > (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Objective<double>::checkHessSym, "Hessian symmetry check.\n\n      This function checks the symmetry of the Hessian by comparing \n      \n\n\n\n\n\n      \n             is an optimization variable.\n      \n\n             is a direction vector.\n      \n\n             is a direction vector.\n      \n\n is a flag that turns on/off output.\n      \n\n     is the output stream.\n\nC++: ROL::Objective<double>::checkHessSym(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3); }, "", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const bool & a4) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3, a4); }, "", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"));
		cl.def("checkHessSym", (class std::vector<double, class std::allocator<double> > (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Objective<double>::checkHessSym, "Hessian symmetry check.\n\n      This function checks the symmetry of the Hessian by comparing \n      \n\n\n\n\n\n      \n             is an optimization variable.\n      \n\n            is used to create temporary Hessian-times-vector vectors.\n      \n\n             is a direction vector.\n      \n\n             is a direction vector.\n      \n\n is a flag that turns on/off output.\n      \n\n     is the output stream.\n\nC++: ROL::Objective<double>::checkHessSym(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("setParameter", (void (ROL::Objective<double>::*)(const class std::vector<double, class std::allocator<double> > &)) &ROL::Objective<double>::setParameter, "C++: ROL::Objective<double>::setParameter(const class std::vector<double, class std::allocator<double> > &) --> void", pybind11::arg("param"));
		cl.def("assign", (class ROL::Objective<double> & (ROL::Objective<double>::*)(const class ROL::Objective<double> &)) &ROL::Objective<double>::operator=, "C++: ROL::Objective<double>::operator=(const class ROL::Objective<double> &) --> class ROL::Objective<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::ConstraintFromObjective file:ROL_ConstraintFromObjective.hpp line:64
		pybind11::class_<ROL::ConstraintFromObjective<double>, Teuchos::RCP<ROL::ConstraintFromObjective<double>>, PyCallBack_ROL_ConstraintFromObjective_double_t, ROL::Constraint<double>> cl(M("ROL"), "ConstraintFromObjective_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0){ return new ROL::ConstraintFromObjective<double>(a0); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0){ return new PyCallBack_ROL_ConstraintFromObjective_double_t(a0); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, const double>(), pybind11::arg("obj"), pybind11::arg("offset") );

		cl.def( pybind11::init( [](PyCallBack_ROL_ConstraintFromObjective_double_t const &o){ return new PyCallBack_ROL_ConstraintFromObjective_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::ConstraintFromObjective<double> const &o){ return new ROL::ConstraintFromObjective<double>(o); } ) );
		cl.def("getObjective", (const class Teuchos::RCP<class ROL::Objective<double> > (ROL::ConstraintFromObjective<double>::*)() const) &ROL::ConstraintFromObjective<double>::getObjective, "C++: ROL::ConstraintFromObjective<double>::getObjective() const --> const class Teuchos::RCP<class ROL::Objective<double> >");
		cl.def("setParameter", (void (ROL::ConstraintFromObjective<double>::*)(const class std::vector<double, class std::allocator<double> > &)) &ROL::ConstraintFromObjective<double>::setParameter, "C++: ROL::ConstraintFromObjective<double>::setParameter(const class std::vector<double, class std::allocator<double> > &) --> void", pybind11::arg("param"));
		cl.def("update", [](ROL::ConstraintFromObjective<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::ConstraintFromObjective<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::ConstraintFromObjective<double>::update, "C++: ROL::ConstraintFromObjective<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::ConstraintFromObjective<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::ConstraintFromObjective<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::ConstraintFromObjective<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::ConstraintFromObjective<double>::update, "C++: ROL::ConstraintFromObjective<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (void (ROL::ConstraintFromObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::ConstraintFromObjective<double>::value, "C++: ROL::ConstraintFromObjective<double>::value(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("c"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyJacobian", (void (ROL::ConstraintFromObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::ConstraintFromObjective<double>::applyJacobian, "C++: ROL::ConstraintFromObjective<double>::applyJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("jv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointJacobian", (void (ROL::ConstraintFromObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::ConstraintFromObjective<double>::applyAdjointJacobian, "C++: ROL::ConstraintFromObjective<double>::applyAdjointJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ajv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointHessian", (void (ROL::ConstraintFromObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::ConstraintFromObjective<double>::applyAdjointHessian, "C++: ROL::ConstraintFromObjective<double>::applyAdjointHessian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ahuv"), pybind11::arg("u"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("update", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::Constraint<double>::update, "Update constraint function. \n\n      This function updates the constraint function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is the type of update requested.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Constraint<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::Constraint<double>::update, "Update constraint functions.  \n                x is the optimization variable, \n                flag = true if optimization variable is changed,\n                iter is the outer algorithm iterations count.\n\nC++: ROL::Constraint<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::value, "C++: ROL::Constraint<double>::value(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("c"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyJacobian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyJacobian, "Apply the constraint Jacobian at \n, \n,\n             to vector \n\n.\n\n             \n  is the result of applying the constraint Jacobian to  at  a constraint-space vector\n             \n\n   is an optimization-space vector\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is a finite-difference approximation.\n\n             ---\n\nC++: ROL::Constraint<double>::applyJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("jv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointJacobian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyAdjointJacobian, "Apply the adjoint of the the constraint Jacobian at \n, \n,\n             to vector \n\n.\n\n             \n is the result of applying the adjoint of the constraint Jacobian to  at  a dual optimization-space vector\n             \n\n   is a dual constraint-space vector\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is a finite-difference approximation.\n\n             ---\n\nC++: ROL::Constraint<double>::applyAdjointJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ajv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointJacobian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyAdjointJacobian, "Apply the adjoint of the the constraint Jacobian at \n, \n,\n             to vector \n\n.\n\n             \n is the result of applying the adjoint of the constraint Jacobian to  at  a dual optimization-space vector\n             \n\n   is a dual constraint-space vector\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n  is a vector used for temporary variables; a constraint-space vector\n             \n\n is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is a finite-difference approximation.\n\n             ---\n\nC++: ROL::Constraint<double>::applyAdjointJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ajv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualv"), pybind11::arg("tol"));
		cl.def("applyAdjointHessian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyAdjointHessian, "Apply the derivative of the adjoint of the constraint Jacobian at \n\n             to vector \n in direction \n,\n             according to \n\n.\n\n             \n is the result of applying the derivative of the adjoint of the constraint Jacobian at  to vector  in direction  a dual optimization-space vector\n             \n\n    is the direction vector; a dual constraint-space vector\n             \n\n    is an optimization-space vector\n             \n\n    is the constraint argument; an optimization-space vector\n             \n\n  is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n, and \n. \n             The default implementation is a finite-difference approximation based on the adjoint Jacobian.\n\n             ---\n\nC++: ROL::Constraint<double>::applyAdjointHessian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("huv"), pybind11::arg("u"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("solveAugmentedSystem", (class std::vector<double, class std::allocator<double> > (ROL::Constraint<double>::*)(class ROL::Vector<double> &, class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::solveAugmentedSystem, "Approximately solves the  augmented system \n             \n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n             where \n, \n,\n             \n\n, \n,\n             \n\n is an identity or Riesz\n             operator, and \n\n\n             is a zero operator.\n\n             \n  is the optimization-space component of the result\n             \n\n  is the dual constraint-space component of the result\n             \n\n  is the dual optimization-space component of the right-hand side\n             \n\n  is the constraint-space component of the right-hand side\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n is the nominal relative residual tolerance\n\n             On return, \n approximately\n             solves the augmented system, where the size of the residual is\n             governed by special stopping conditions. \n             The default implementation is the preconditioned generalized\n             minimal residual (GMRES) method, which enables the use of\n             nonsymmetric preconditioners.\n\n             ---\n\nC++: ROL::Constraint<double>::solveAugmentedSystem(class ROL::Vector<double> &, class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("v1"), pybind11::arg("v2"), pybind11::arg("b1"), pybind11::arg("b2"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyPreconditioner", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyPreconditioner, "Apply a constraint preconditioner at \n, \n,\n             to vector \n\n.  Ideally, this preconditioner satisfies the following relationship:\n             \n\n\n\n             where R is the appropriate Riesz map in \n.  It is used by the #solveAugmentedSystem method.\n\n             \n  is the result of applying the constraint preconditioner to  at  a dual constraint-space vector\n             \n\n   is a constraint-space vector\n             \n\n   is the preconditioner argument; an optimization-space vector\n             \n\n   is the preconditioner argument; a dual optimization-space vector, unused\n             \n\n is a tolerance for inexact evaluations\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is the Riesz map in \n\n.\n\n             ---\n\nC++: ROL::Constraint<double>::applyPreconditioner(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"), pybind11::arg("tol"));
		cl.def("activate", (void (ROL::Constraint<double>::*)()) &ROL::Constraint<double>::activate, "Turn on constraints \n\nC++: ROL::Constraint<double>::activate() --> void");
		cl.def("deactivate", (void (ROL::Constraint<double>::*)()) &ROL::Constraint<double>::deactivate, "Turn off constraints\n\nC++: ROL::Constraint<double>::deactivate() --> void");
		cl.def("isActivated", (bool (ROL::Constraint<double>::*)()) &ROL::Constraint<double>::isActivated, "Check if constraints are on\n\nC++: ROL::Constraint<double>::isActivated() --> bool");
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"));
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool & a3) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2, a3); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("printToStream"));
		cl.def("checkAdjointConsistencyJacobian", (double (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Constraint<double>::checkAdjointConsistencyJacobian, "C++: ROL::Constraint<double>::checkAdjointConsistencyJacobian(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> double", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualw"), pybind11::arg("dualv"));
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4, const bool & a5) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4, a5); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualw"), pybind11::arg("dualv"), pybind11::arg("printToStream"));
		cl.def("checkAdjointConsistencyJacobian", (double (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Constraint<double>::checkAdjointConsistencyJacobian, "C++: ROL::Constraint<double>::checkAdjointConsistencyJacobian(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> double", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualw"), pybind11::arg("dualv"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("setParameter", (void (ROL::Constraint<double>::*)(const class std::vector<double, class std::allocator<double> > &)) &ROL::Constraint<double>::setParameter, "C++: ROL::Constraint<double>::setParameter(const class std::vector<double, class std::allocator<double> > &) --> void", pybind11::arg("param"));
		cl.def("assign", (class ROL::Constraint<double> & (ROL::Constraint<double>::*)(const class ROL::Constraint<double> &)) &ROL::Constraint<double>::operator=, "C++: ROL::Constraint<double>::operator=(const class ROL::Constraint<double> &) --> class ROL::Constraint<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::StochasticConstraint file:ROL_StochasticConstraint.hpp line:54
		pybind11::class_<ROL::StochasticConstraint<double>, Teuchos::RCP<ROL::StochasticConstraint<double>>, PyCallBack_ROL_StochasticConstraint_double_t, ROL::Constraint<double>> cl(M("ROL"), "StochasticConstraint_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a1, class Teuchos::ParameterList & a2){ return new ROL::StochasticConstraint<double>(a0, a1, a2); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a1, class Teuchos::ParameterList & a2){ return new PyCallBack_ROL_StochasticConstraint_double_t(a0, a1, a2); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, class Teuchos::ParameterList &, const int>(), pybind11::arg("obj"), pybind11::arg("sampler"), pybind11::arg("parlist"), pybind11::arg("index") );

		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Constraint<double> > & a0, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a1, class Teuchos::ParameterList & a2){ return new ROL::StochasticConstraint<double>(a0, a1, a2); }, [](const class Teuchos::RCP<class ROL::Constraint<double> > & a0, const class Teuchos::RCP<class ROL::SampleGenerator<double> > & a1, class Teuchos::ParameterList & a2){ return new PyCallBack_ROL_StochasticConstraint_double_t(a0, a1, a2); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Constraint<double> > &, const class Teuchos::RCP<class ROL::SampleGenerator<double> > &, class Teuchos::ParameterList &, const int>(), pybind11::arg("con"), pybind11::arg("sampler"), pybind11::arg("parlist"), pybind11::arg("index") );

		cl.def( pybind11::init( [](PyCallBack_ROL_StochasticConstraint_double_t const &o){ return new PyCallBack_ROL_StochasticConstraint_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::StochasticConstraint<double> const &o){ return new ROL::StochasticConstraint<double>(o); } ) );
		cl.def("computeStatistic", (double (ROL::StochasticConstraint<double>::*)(const class ROL::Vector<double> &) const) &ROL::StochasticConstraint<double>::computeStatistic, "C++: ROL::StochasticConstraint<double>::computeStatistic(const class ROL::Vector<double> &) const --> double", pybind11::arg("x"));
		cl.def("setIndex", (void (ROL::StochasticConstraint<double>::*)(int)) &ROL::StochasticConstraint<double>::setIndex, "C++: ROL::StochasticConstraint<double>::setIndex(int) --> void", pybind11::arg("ind"));
		cl.def("update", [](ROL::StochasticConstraint<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::StochasticConstraint<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::StochasticConstraint<double>::update, "C++: ROL::StochasticConstraint<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::StochasticConstraint<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::StochasticConstraint<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::StochasticConstraint<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::StochasticConstraint<double>::update, "C++: ROL::StochasticConstraint<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (void (ROL::StochasticConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::StochasticConstraint<double>::value, "C++: ROL::StochasticConstraint<double>::value(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("c"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyJacobian", (void (ROL::StochasticConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::StochasticConstraint<double>::applyJacobian, "C++: ROL::StochasticConstraint<double>::applyJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("jv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointJacobian", (void (ROL::StochasticConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::StochasticConstraint<double>::applyAdjointJacobian, "C++: ROL::StochasticConstraint<double>::applyAdjointJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ajv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointHessian", (void (ROL::StochasticConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::StochasticConstraint<double>::applyAdjointHessian, "C++: ROL::StochasticConstraint<double>::applyAdjointHessian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ahuv"), pybind11::arg("u"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("assign", (class ROL::StochasticConstraint<double> & (ROL::StochasticConstraint<double>::*)(const class ROL::StochasticConstraint<double> &)) &ROL::StochasticConstraint<double>::operator=, "C++: ROL::StochasticConstraint<double>::operator=(const class ROL::StochasticConstraint<double> &) --> class ROL::StochasticConstraint<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("update", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::Constraint<double>::update, "Update constraint function. \n\n      This function updates the constraint function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is the type of update requested.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Constraint<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::Constraint<double>::update, "Update constraint functions.  \n                x is the optimization variable, \n                flag = true if optimization variable is changed,\n                iter is the outer algorithm iterations count.\n\nC++: ROL::Constraint<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::value, "C++: ROL::Constraint<double>::value(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("c"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyJacobian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyJacobian, "Apply the constraint Jacobian at \n, \n,\n             to vector \n\n.\n\n             \n  is the result of applying the constraint Jacobian to  at  a constraint-space vector\n             \n\n   is an optimization-space vector\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is a finite-difference approximation.\n\n             ---\n\nC++: ROL::Constraint<double>::applyJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("jv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointJacobian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyAdjointJacobian, "Apply the adjoint of the the constraint Jacobian at \n, \n,\n             to vector \n\n.\n\n             \n is the result of applying the adjoint of the constraint Jacobian to  at  a dual optimization-space vector\n             \n\n   is a dual constraint-space vector\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is a finite-difference approximation.\n\n             ---\n\nC++: ROL::Constraint<double>::applyAdjointJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ajv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointJacobian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyAdjointJacobian, "Apply the adjoint of the the constraint Jacobian at \n, \n,\n             to vector \n\n.\n\n             \n is the result of applying the adjoint of the constraint Jacobian to  at  a dual optimization-space vector\n             \n\n   is a dual constraint-space vector\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n  is a vector used for temporary variables; a constraint-space vector\n             \n\n is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is a finite-difference approximation.\n\n             ---\n\nC++: ROL::Constraint<double>::applyAdjointJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ajv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualv"), pybind11::arg("tol"));
		cl.def("applyAdjointHessian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyAdjointHessian, "Apply the derivative of the adjoint of the constraint Jacobian at \n\n             to vector \n in direction \n,\n             according to \n\n.\n\n             \n is the result of applying the derivative of the adjoint of the constraint Jacobian at  to vector  in direction  a dual optimization-space vector\n             \n\n    is the direction vector; a dual constraint-space vector\n             \n\n    is an optimization-space vector\n             \n\n    is the constraint argument; an optimization-space vector\n             \n\n  is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n, and \n. \n             The default implementation is a finite-difference approximation based on the adjoint Jacobian.\n\n             ---\n\nC++: ROL::Constraint<double>::applyAdjointHessian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("huv"), pybind11::arg("u"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("solveAugmentedSystem", (class std::vector<double, class std::allocator<double> > (ROL::Constraint<double>::*)(class ROL::Vector<double> &, class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::solveAugmentedSystem, "Approximately solves the  augmented system \n             \n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n             where \n, \n,\n             \n\n, \n,\n             \n\n is an identity or Riesz\n             operator, and \n\n\n             is a zero operator.\n\n             \n  is the optimization-space component of the result\n             \n\n  is the dual constraint-space component of the result\n             \n\n  is the dual optimization-space component of the right-hand side\n             \n\n  is the constraint-space component of the right-hand side\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n is the nominal relative residual tolerance\n\n             On return, \n approximately\n             solves the augmented system, where the size of the residual is\n             governed by special stopping conditions. \n             The default implementation is the preconditioned generalized\n             minimal residual (GMRES) method, which enables the use of\n             nonsymmetric preconditioners.\n\n             ---\n\nC++: ROL::Constraint<double>::solveAugmentedSystem(class ROL::Vector<double> &, class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("v1"), pybind11::arg("v2"), pybind11::arg("b1"), pybind11::arg("b2"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyPreconditioner", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyPreconditioner, "Apply a constraint preconditioner at \n, \n,\n             to vector \n\n.  Ideally, this preconditioner satisfies the following relationship:\n             \n\n\n\n             where R is the appropriate Riesz map in \n.  It is used by the #solveAugmentedSystem method.\n\n             \n  is the result of applying the constraint preconditioner to  at  a dual constraint-space vector\n             \n\n   is a constraint-space vector\n             \n\n   is the preconditioner argument; an optimization-space vector\n             \n\n   is the preconditioner argument; a dual optimization-space vector, unused\n             \n\n is a tolerance for inexact evaluations\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is the Riesz map in \n\n.\n\n             ---\n\nC++: ROL::Constraint<double>::applyPreconditioner(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"), pybind11::arg("tol"));
		cl.def("activate", (void (ROL::Constraint<double>::*)()) &ROL::Constraint<double>::activate, "Turn on constraints \n\nC++: ROL::Constraint<double>::activate() --> void");
		cl.def("deactivate", (void (ROL::Constraint<double>::*)()) &ROL::Constraint<double>::deactivate, "Turn off constraints\n\nC++: ROL::Constraint<double>::deactivate() --> void");
		cl.def("isActivated", (bool (ROL::Constraint<double>::*)()) &ROL::Constraint<double>::isActivated, "Check if constraints are on\n\nC++: ROL::Constraint<double>::isActivated() --> bool");
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"));
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool & a3) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2, a3); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("printToStream"));
		cl.def("checkAdjointConsistencyJacobian", (double (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Constraint<double>::checkAdjointConsistencyJacobian, "C++: ROL::Constraint<double>::checkAdjointConsistencyJacobian(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> double", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualw"), pybind11::arg("dualv"));
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4, const bool & a5) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4, a5); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualw"), pybind11::arg("dualv"), pybind11::arg("printToStream"));
		cl.def("checkAdjointConsistencyJacobian", (double (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Constraint<double>::checkAdjointConsistencyJacobian, "C++: ROL::Constraint<double>::checkAdjointConsistencyJacobian(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> double", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualw"), pybind11::arg("dualv"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("setParameter", (void (ROL::Constraint<double>::*)(const class std::vector<double, class std::allocator<double> > &)) &ROL::Constraint<double>::setParameter, "C++: ROL::Constraint<double>::setParameter(const class std::vector<double, class std::allocator<double> > &) --> void", pybind11::arg("param"));
		cl.def("assign", (class ROL::Constraint<double> & (ROL::Constraint<double>::*)(const class ROL::Constraint<double> &)) &ROL::Constraint<double>::operator=, "C++: ROL::Constraint<double>::operator=(const class ROL::Constraint<double> &) --> class ROL::Constraint<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::StdBoundConstraint file:ROL_StdBoundConstraint.hpp line:58
		pybind11::class_<ROL::StdBoundConstraint<double>, Teuchos::RCP<ROL::StdBoundConstraint<double>>, PyCallBack_ROL_StdBoundConstraint_double_t, ROL::BoundConstraint<double>> cl(M("ROL"), "StdBoundConstraint_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](class std::vector<double, class std::allocator<double> > & a0){ return new ROL::StdBoundConstraint<double>(a0); }, [](class std::vector<double, class std::allocator<double> > & a0){ return new PyCallBack_ROL_StdBoundConstraint_double_t(a0); } ), "doc");
		cl.def( pybind11::init( [](class std::vector<double, class std::allocator<double> > & a0, bool const & a1){ return new ROL::StdBoundConstraint<double>(a0, a1); }, [](class std::vector<double, class std::allocator<double> > & a0, bool const & a1){ return new PyCallBack_ROL_StdBoundConstraint_double_t(a0, a1); } ), "doc");
		cl.def( pybind11::init( [](class std::vector<double, class std::allocator<double> > & a0, bool const & a1, double const & a2){ return new ROL::StdBoundConstraint<double>(a0, a1, a2); }, [](class std::vector<double, class std::allocator<double> > & a0, bool const & a1, double const & a2){ return new PyCallBack_ROL_StdBoundConstraint_double_t(a0, a1, a2); } ), "doc");
		cl.def( pybind11::init<class std::vector<double, class std::allocator<double> > &, bool, double, const double>(), pybind11::arg("x"), pybind11::arg("isLower"), pybind11::arg("scale"), pybind11::arg("feasTol") );

		cl.def( pybind11::init( [](class std::vector<double, class std::allocator<double> > & a0, class std::vector<double, class std::allocator<double> > & a1){ return new ROL::StdBoundConstraint<double>(a0, a1); }, [](class std::vector<double, class std::allocator<double> > & a0, class std::vector<double, class std::allocator<double> > & a1){ return new PyCallBack_ROL_StdBoundConstraint_double_t(a0, a1); } ), "doc");
		cl.def( pybind11::init( [](class std::vector<double, class std::allocator<double> > & a0, class std::vector<double, class std::allocator<double> > & a1, double const & a2){ return new ROL::StdBoundConstraint<double>(a0, a1, a2); }, [](class std::vector<double, class std::allocator<double> > & a0, class std::vector<double, class std::allocator<double> > & a1, double const & a2){ return new PyCallBack_ROL_StdBoundConstraint_double_t(a0, a1, a2); } ), "doc");
		cl.def( pybind11::init<class std::vector<double, class std::allocator<double> > &, class std::vector<double, class std::allocator<double> > &, double, const double>(), pybind11::arg("l"), pybind11::arg("u"), pybind11::arg("scale"), pybind11::arg("feasTol") );

		cl.def( pybind11::init( [](PyCallBack_ROL_StdBoundConstraint_double_t const &o){ return new PyCallBack_ROL_StdBoundConstraint_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::StdBoundConstraint<double> const &o){ return new ROL::StdBoundConstraint<double>(o); } ) );
		cl.def("project", (void (ROL::StdBoundConstraint<double>::*)(class ROL::Vector<double> &)) &ROL::StdBoundConstraint<double>::project, "C++: ROL::StdBoundConstraint<double>::project(class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("projectInterior", (void (ROL::StdBoundConstraint<double>::*)(class ROL::Vector<double> &)) &ROL::StdBoundConstraint<double>::projectInterior, "C++: ROL::StdBoundConstraint<double>::projectInterior(class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("pruneUpperActive", [](ROL::StdBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneUpperActive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneUpperActive", (void (ROL::StdBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::StdBoundConstraint<double>::pruneUpperActive, "C++: ROL::StdBoundConstraint<double>::pruneUpperActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneUpperActive", [](ROL::StdBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneUpperActive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneUpperActive", [](ROL::StdBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneUpperActive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneUpperActive", (void (ROL::StdBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::StdBoundConstraint<double>::pruneUpperActive, "C++: ROL::StdBoundConstraint<double>::pruneUpperActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneLowerActive", [](ROL::StdBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneLowerActive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneLowerActive", [](ROL::StdBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneLowerActive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneLowerActive", (void (ROL::StdBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::StdBoundConstraint<double>::pruneLowerActive, "C++: ROL::StdBoundConstraint<double>::pruneLowerActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneLowerActive", [](ROL::StdBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneLowerActive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneLowerActive", (void (ROL::StdBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::StdBoundConstraint<double>::pruneLowerActive, "C++: ROL::StdBoundConstraint<double>::pruneLowerActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("isFeasible", (bool (ROL::StdBoundConstraint<double>::*)(const class ROL::Vector<double> &)) &ROL::StdBoundConstraint<double>::isFeasible, "C++: ROL::StdBoundConstraint<double>::isFeasible(const class ROL::Vector<double> &) --> bool", pybind11::arg("x"));
		cl.def("applyInverseScalingFunction", (void (ROL::StdBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const) &ROL::StdBoundConstraint<double>::applyInverseScalingFunction, "C++: ROL::StdBoundConstraint<double>::applyInverseScalingFunction(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const --> void", pybind11::arg("dv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"));
		cl.def("applyScalingFunctionJacobian", (void (ROL::StdBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const) &ROL::StdBoundConstraint<double>::applyScalingFunctionJacobian, "C++: ROL::StdBoundConstraint<double>::applyScalingFunctionJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const --> void", pybind11::arg("dv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"));
		cl.def("project", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &)) &ROL::BoundConstraint<double>::project, "C++: ROL::BoundConstraint<double>::project(class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("projectInterior", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &)) &ROL::BoundConstraint<double>::projectInterior, "C++: ROL::BoundConstraint<double>::projectInterior(class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("pruneUpperActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneUpperActive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneUpperActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneUpperActive, "C++: ROL::BoundConstraint<double>::pruneUpperActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneUpperActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneUpperActive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneUpperActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneUpperActive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneUpperActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneUpperActive, "C++: ROL::BoundConstraint<double>::pruneUpperActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneLowerActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneLowerActive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneLowerActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneLowerActive, "C++: ROL::BoundConstraint<double>::pruneLowerActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneLowerActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneLowerActive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneLowerActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneLowerActive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneLowerActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneLowerActive, "C++: ROL::BoundConstraint<double>::pruneLowerActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("getLowerBound", (const class Teuchos::RCP<const class ROL::Vector<double> > (ROL::BoundConstraint<double>::*)() const) &ROL::BoundConstraint<double>::getLowerBound, "C++: ROL::BoundConstraint<double>::getLowerBound() const --> const class Teuchos::RCP<const class ROL::Vector<double> >");
		cl.def("getUpperBound", (const class Teuchos::RCP<const class ROL::Vector<double> > (ROL::BoundConstraint<double>::*)() const) &ROL::BoundConstraint<double>::getUpperBound, "C++: ROL::BoundConstraint<double>::getUpperBound() const --> const class Teuchos::RCP<const class ROL::Vector<double> >");
		cl.def("isFeasible", (bool (ROL::BoundConstraint<double>::*)(const class ROL::Vector<double> &)) &ROL::BoundConstraint<double>::isFeasible, "C++: ROL::BoundConstraint<double>::isFeasible(const class ROL::Vector<double> &) --> bool", pybind11::arg("v"));
		cl.def("applyInverseScalingFunction", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const) &ROL::BoundConstraint<double>::applyInverseScalingFunction, "C++: ROL::BoundConstraint<double>::applyInverseScalingFunction(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const --> void", pybind11::arg("dv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"));
		cl.def("applyScalingFunctionJacobian", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const) &ROL::BoundConstraint<double>::applyScalingFunctionJacobian, "C++: ROL::BoundConstraint<double>::applyScalingFunctionJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const --> void", pybind11::arg("dv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"));
		cl.def("activateLower", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::activateLower, "C++: ROL::BoundConstraint<double>::activateLower() --> void");
		cl.def("activateUpper", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::activateUpper, "C++: ROL::BoundConstraint<double>::activateUpper() --> void");
		cl.def("activate", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::activate, "C++: ROL::BoundConstraint<double>::activate() --> void");
		cl.def("deactivateLower", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::deactivateLower, "C++: ROL::BoundConstraint<double>::deactivateLower() --> void");
		cl.def("deactivateUpper", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::deactivateUpper, "C++: ROL::BoundConstraint<double>::deactivateUpper() --> void");
		cl.def("deactivate", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::deactivate, "C++: ROL::BoundConstraint<double>::deactivate() --> void");
		cl.def("isLowerActivated", (bool (ROL::BoundConstraint<double>::*)() const) &ROL::BoundConstraint<double>::isLowerActivated, "C++: ROL::BoundConstraint<double>::isLowerActivated() const --> bool");
		cl.def("isUpperActivated", (bool (ROL::BoundConstraint<double>::*)() const) &ROL::BoundConstraint<double>::isUpperActivated, "C++: ROL::BoundConstraint<double>::isUpperActivated() const --> bool");
		cl.def("isActivated", (bool (ROL::BoundConstraint<double>::*)() const) &ROL::BoundConstraint<double>::isActivated, "C++: ROL::BoundConstraint<double>::isActivated() const --> bool");
		cl.def("pruneActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneActive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneActive, "C++: ROL::BoundConstraint<double>::pruneActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneActive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneActive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneActive, "C++: ROL::BoundConstraint<double>::pruneActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneLowerInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneLowerInactive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneLowerInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneLowerInactive, "C++: ROL::BoundConstraint<double>::pruneLowerInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneUpperInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneUpperInactive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneUpperInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneUpperInactive, "C++: ROL::BoundConstraint<double>::pruneUpperInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneLowerInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneLowerInactive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneLowerInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneLowerInactive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneLowerInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneLowerInactive, "C++: ROL::BoundConstraint<double>::pruneLowerInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneUpperInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneUpperInactive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneUpperInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneUpperInactive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneUpperInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneUpperInactive, "C++: ROL::BoundConstraint<double>::pruneUpperInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneInactive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneInactive, "C++: ROL::BoundConstraint<double>::pruneInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneInactive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneInactive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneInactive, "C++: ROL::BoundConstraint<double>::pruneInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("computeProjectedGradient", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &)) &ROL::BoundConstraint<double>::computeProjectedGradient, "C++: ROL::BoundConstraint<double>::computeProjectedGradient(class ROL::Vector<double> &, const class ROL::Vector<double> &) --> void", pybind11::arg("g"), pybind11::arg("x"));
		cl.def("computeProjectedStep", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &)) &ROL::BoundConstraint<double>::computeProjectedStep, "C++: ROL::BoundConstraint<double>::computeProjectedStep(class ROL::Vector<double> &, const class ROL::Vector<double> &) --> void", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("assign", (class ROL::BoundConstraint<double> & (ROL::BoundConstraint<double>::*)(const class ROL::BoundConstraint<double> &)) &ROL::BoundConstraint<double>::operator=, "C++: ROL::BoundConstraint<double>::operator=(const class ROL::BoundConstraint<double> &) --> class ROL::BoundConstraint<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::RiskBoundConstraint file:ROL_RiskBoundConstraint.hpp line:54
		pybind11::class_<ROL::RiskBoundConstraint<double>, Teuchos::RCP<ROL::RiskBoundConstraint<double>>, PyCallBack_ROL_RiskBoundConstraint_double_t, ROL::BoundConstraint<double>> cl(M("ROL"), "RiskBoundConstraint_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](class Teuchos::RCP<class Teuchos::ParameterList> & a0){ return new ROL::RiskBoundConstraint<double>(a0); }, [](class Teuchos::RCP<class Teuchos::ParameterList> & a0){ return new PyCallBack_ROL_RiskBoundConstraint_double_t(a0); } ), "doc");
		cl.def( pybind11::init<class Teuchos::RCP<class Teuchos::ParameterList> &, const class Teuchos::RCP<class ROL::BoundConstraint<double> > &>(), pybind11::arg("parlist"), pybind11::arg("bc") );

		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::BoundConstraint<double> > &>(), pybind11::arg("bc") );

		cl.def( pybind11::init( [](PyCallBack_ROL_RiskBoundConstraint_double_t const &o){ return new PyCallBack_ROL_RiskBoundConstraint_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::RiskBoundConstraint<double> const &o){ return new ROL::RiskBoundConstraint<double>(o); } ) );
		cl.def("project", (void (ROL::RiskBoundConstraint<double>::*)(class ROL::Vector<double> &)) &ROL::RiskBoundConstraint<double>::project, "C++: ROL::RiskBoundConstraint<double>::project(class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("projectInterior", (void (ROL::RiskBoundConstraint<double>::*)(class ROL::Vector<double> &)) &ROL::RiskBoundConstraint<double>::projectInterior, "C++: ROL::RiskBoundConstraint<double>::projectInterior(class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("pruneUpperActive", [](ROL::RiskBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneUpperActive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneUpperActive", (void (ROL::RiskBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::RiskBoundConstraint<double>::pruneUpperActive, "C++: ROL::RiskBoundConstraint<double>::pruneUpperActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneUpperActive", [](ROL::RiskBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneUpperActive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneUpperActive", [](ROL::RiskBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneUpperActive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneUpperActive", (void (ROL::RiskBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::RiskBoundConstraint<double>::pruneUpperActive, "C++: ROL::RiskBoundConstraint<double>::pruneUpperActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneLowerActive", [](ROL::RiskBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneLowerActive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneLowerActive", (void (ROL::RiskBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::RiskBoundConstraint<double>::pruneLowerActive, "C++: ROL::RiskBoundConstraint<double>::pruneLowerActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneLowerActive", [](ROL::RiskBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneLowerActive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneLowerActive", [](ROL::RiskBoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneLowerActive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneLowerActive", (void (ROL::RiskBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::RiskBoundConstraint<double>::pruneLowerActive, "C++: ROL::RiskBoundConstraint<double>::pruneLowerActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("getLowerBound", (const class Teuchos::RCP<const class ROL::Vector<double> > (ROL::RiskBoundConstraint<double>::*)() const) &ROL::RiskBoundConstraint<double>::getLowerBound, "C++: ROL::RiskBoundConstraint<double>::getLowerBound() const --> const class Teuchos::RCP<const class ROL::Vector<double> >");
		cl.def("getUpperBound", (const class Teuchos::RCP<const class ROL::Vector<double> > (ROL::RiskBoundConstraint<double>::*)() const) &ROL::RiskBoundConstraint<double>::getUpperBound, "C++: ROL::RiskBoundConstraint<double>::getUpperBound() const --> const class Teuchos::RCP<const class ROL::Vector<double> >");
		cl.def("isFeasible", (bool (ROL::RiskBoundConstraint<double>::*)(const class ROL::Vector<double> &)) &ROL::RiskBoundConstraint<double>::isFeasible, "C++: ROL::RiskBoundConstraint<double>::isFeasible(const class ROL::Vector<double> &) --> bool", pybind11::arg("v"));
		cl.def("applyInverseScalingFunction", (void (ROL::RiskBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const) &ROL::RiskBoundConstraint<double>::applyInverseScalingFunction, "C++: ROL::RiskBoundConstraint<double>::applyInverseScalingFunction(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const --> void", pybind11::arg("dv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"));
		cl.def("applyScalingFunctionJacobian", (void (ROL::RiskBoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const) &ROL::RiskBoundConstraint<double>::applyScalingFunctionJacobian, "C++: ROL::RiskBoundConstraint<double>::applyScalingFunctionJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const --> void", pybind11::arg("dv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"));
		cl.def("assign", (class ROL::RiskBoundConstraint<double> & (ROL::RiskBoundConstraint<double>::*)(const class ROL::RiskBoundConstraint<double> &)) &ROL::RiskBoundConstraint<double>::operator=, "C++: ROL::RiskBoundConstraint<double>::operator=(const class ROL::RiskBoundConstraint<double> &) --> class ROL::RiskBoundConstraint<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
		cl.def("project", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &)) &ROL::BoundConstraint<double>::project, "C++: ROL::BoundConstraint<double>::project(class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("projectInterior", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &)) &ROL::BoundConstraint<double>::projectInterior, "C++: ROL::BoundConstraint<double>::projectInterior(class ROL::Vector<double> &) --> void", pybind11::arg("x"));
		cl.def("pruneUpperActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneUpperActive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneUpperActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneUpperActive, "C++: ROL::BoundConstraint<double>::pruneUpperActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneUpperActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneUpperActive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneUpperActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneUpperActive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneUpperActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneUpperActive, "C++: ROL::BoundConstraint<double>::pruneUpperActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneLowerActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneLowerActive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneLowerActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneLowerActive, "C++: ROL::BoundConstraint<double>::pruneLowerActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneLowerActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneLowerActive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneLowerActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneLowerActive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneLowerActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneLowerActive, "C++: ROL::BoundConstraint<double>::pruneLowerActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("getLowerBound", (const class Teuchos::RCP<const class ROL::Vector<double> > (ROL::BoundConstraint<double>::*)() const) &ROL::BoundConstraint<double>::getLowerBound, "C++: ROL::BoundConstraint<double>::getLowerBound() const --> const class Teuchos::RCP<const class ROL::Vector<double> >");
		cl.def("getUpperBound", (const class Teuchos::RCP<const class ROL::Vector<double> > (ROL::BoundConstraint<double>::*)() const) &ROL::BoundConstraint<double>::getUpperBound, "C++: ROL::BoundConstraint<double>::getUpperBound() const --> const class Teuchos::RCP<const class ROL::Vector<double> >");
		cl.def("isFeasible", (bool (ROL::BoundConstraint<double>::*)(const class ROL::Vector<double> &)) &ROL::BoundConstraint<double>::isFeasible, "C++: ROL::BoundConstraint<double>::isFeasible(const class ROL::Vector<double> &) --> bool", pybind11::arg("v"));
		cl.def("applyInverseScalingFunction", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const) &ROL::BoundConstraint<double>::applyInverseScalingFunction, "C++: ROL::BoundConstraint<double>::applyInverseScalingFunction(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const --> void", pybind11::arg("dv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"));
		cl.def("applyScalingFunctionJacobian", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const) &ROL::BoundConstraint<double>::applyScalingFunctionJacobian, "C++: ROL::BoundConstraint<double>::applyScalingFunctionJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &) const --> void", pybind11::arg("dv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"));
		cl.def("activateLower", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::activateLower, "C++: ROL::BoundConstraint<double>::activateLower() --> void");
		cl.def("activateUpper", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::activateUpper, "C++: ROL::BoundConstraint<double>::activateUpper() --> void");
		cl.def("activate", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::activate, "C++: ROL::BoundConstraint<double>::activate() --> void");
		cl.def("deactivateLower", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::deactivateLower, "C++: ROL::BoundConstraint<double>::deactivateLower() --> void");
		cl.def("deactivateUpper", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::deactivateUpper, "C++: ROL::BoundConstraint<double>::deactivateUpper() --> void");
		cl.def("deactivate", (void (ROL::BoundConstraint<double>::*)()) &ROL::BoundConstraint<double>::deactivate, "C++: ROL::BoundConstraint<double>::deactivate() --> void");
		cl.def("isLowerActivated", (bool (ROL::BoundConstraint<double>::*)() const) &ROL::BoundConstraint<double>::isLowerActivated, "C++: ROL::BoundConstraint<double>::isLowerActivated() const --> bool");
		cl.def("isUpperActivated", (bool (ROL::BoundConstraint<double>::*)() const) &ROL::BoundConstraint<double>::isUpperActivated, "C++: ROL::BoundConstraint<double>::isUpperActivated() const --> bool");
		cl.def("isActivated", (bool (ROL::BoundConstraint<double>::*)() const) &ROL::BoundConstraint<double>::isActivated, "C++: ROL::BoundConstraint<double>::isActivated() const --> bool");
		cl.def("pruneActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneActive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneActive, "C++: ROL::BoundConstraint<double>::pruneActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneActive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneActive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneActive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneActive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneActive, "C++: ROL::BoundConstraint<double>::pruneActive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneLowerInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneLowerInactive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneLowerInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneLowerInactive, "C++: ROL::BoundConstraint<double>::pruneLowerInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneUpperInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneUpperInactive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneUpperInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneUpperInactive, "C++: ROL::BoundConstraint<double>::pruneUpperInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneLowerInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneLowerInactive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneLowerInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneLowerInactive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneLowerInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneLowerInactive, "C++: ROL::BoundConstraint<double>::pruneLowerInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneUpperInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneUpperInactive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneUpperInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneUpperInactive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneUpperInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneUpperInactive, "C++: ROL::BoundConstraint<double>::pruneUpperInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("pruneInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1) -> void { return o.pruneInactive(a0, a1); }, "", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("pruneInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double)) &ROL::BoundConstraint<double>::pruneInactive, "C++: ROL::BoundConstraint<double>::pruneInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, double) --> void", pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("eps"));
		cl.def("pruneInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> void { return o.pruneInactive(a0, a1, a2); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"));
		cl.def("pruneInactive", [](ROL::BoundConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, double const & a3) -> void { return o.pruneInactive(a0, a1, a2, a3); }, "", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"));
		cl.def("pruneInactive", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double)) &ROL::BoundConstraint<double>::pruneInactive, "C++: ROL::BoundConstraint<double>::pruneInactive(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double) --> void", pybind11::arg("v"), pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("xeps"), pybind11::arg("geps"));
		cl.def("computeProjectedGradient", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &)) &ROL::BoundConstraint<double>::computeProjectedGradient, "C++: ROL::BoundConstraint<double>::computeProjectedGradient(class ROL::Vector<double> &, const class ROL::Vector<double> &) --> void", pybind11::arg("g"), pybind11::arg("x"));
		cl.def("computeProjectedStep", (void (ROL::BoundConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &)) &ROL::BoundConstraint<double>::computeProjectedStep, "C++: ROL::BoundConstraint<double>::computeProjectedStep(class ROL::Vector<double> &, const class ROL::Vector<double> &) --> void", pybind11::arg("v"), pybind11::arg("x"));
		cl.def("assign", (class ROL::BoundConstraint<double> & (ROL::BoundConstraint<double>::*)(const class ROL::BoundConstraint<double> &)) &ROL::BoundConstraint<double>::operator=, "C++: ROL::BoundConstraint<double>::operator=(const class ROL::BoundConstraint<double> &) --> class ROL::BoundConstraint<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::RiskLessObjective file:ROL_RiskLessObjective.hpp line:53
		pybind11::class_<ROL::RiskLessObjective<double>, Teuchos::RCP<ROL::RiskLessObjective<double>>, PyCallBack_ROL_RiskLessObjective_double_t, ROL::Objective<double>> cl(M("ROL"), "RiskLessObjective_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &>(), pybind11::arg("obj") );

		cl.def( pybind11::init( [](PyCallBack_ROL_RiskLessObjective_double_t const &o){ return new PyCallBack_ROL_RiskLessObjective_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::RiskLessObjective<double> const &o){ return new ROL::RiskLessObjective<double>(o); } ) );
		cl.def("update", [](ROL::RiskLessObjective<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::RiskLessObjective<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::RiskLessObjective<double>::update, "C++: ROL::RiskLessObjective<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::RiskLessObjective<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::RiskLessObjective<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::RiskLessObjective<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::RiskLessObjective<double>::update, "C++: ROL::RiskLessObjective<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (double (ROL::RiskLessObjective<double>::*)(const class ROL::Vector<double> &, double &)) &ROL::RiskLessObjective<double>::value, "C++: ROL::RiskLessObjective<double>::value(const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("gradient", (void (ROL::RiskLessObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::RiskLessObjective<double>::gradient, "C++: ROL::RiskLessObjective<double>::gradient(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("hessVec", (void (ROL::RiskLessObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::RiskLessObjective<double>::hessVec, "C++: ROL::RiskLessObjective<double>::hessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("precond", (void (ROL::RiskLessObjective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::RiskLessObjective<double>::precond, "C++: ROL::RiskLessObjective<double>::precond(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("setParameter", (void (ROL::RiskLessObjective<double>::*)(const class std::vector<double, class std::allocator<double> > &)) &ROL::RiskLessObjective<double>::setParameter, "C++: ROL::RiskLessObjective<double>::setParameter(const class std::vector<double, class std::allocator<double> > &) --> void", pybind11::arg("param"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::Objective<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::Objective<double>::update, "Update objective function. \n\n      This function updates the objective function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is the type of update requested.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Objective<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::Objective<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::Objective<double>::update, "Update objective function. \n\n      This function updates the objective function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is true if the iterate has changed.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Objective<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (double (ROL::Objective<double>::*)(const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::value, "C++: ROL::Objective<double>::value(const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("gradient", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::gradient, "Compute gradient.\n\n      This function returns the objective function gradient.\n      \n\n   is the gradient.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\n      The default implementation is a finite-difference approximation based on the function value.\n      This requires the definition of a basis \n\n for the optimization vectors x and\n      the definition of a basis \n\n for the dual optimization vectors (gradient vectors g).\n      The bases must be related through the Riesz map, i.e., \n\n,\n      and this must be reflected in the implementation of the ROL::Vector::dual() method.\n\nC++: ROL::Objective<double>::gradient(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("g"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("dirDeriv", (double (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::dirDeriv, "Compute directional derivative.\n\n      This function returns the directional derivative of the objective function in the \n direction.\n      \n\n   is the current iterate.\n      \n\n   is the direction.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::dirDeriv(const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> double", pybind11::arg("x"), pybind11::arg("d"), pybind11::arg("tol"));
		cl.def("hessVec", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::hessVec, "Apply Hessian approximation to vector.\n\n      This function applies the Hessian of the objective function to the vector \n.\n      \n\n  is the the action of the Hessian on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::hessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("invHessVec", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::invHessVec, "Apply inverse Hessian approximation to vector.\n\n      This function applies the inverse Hessian of the objective function to the vector \n.\n      \n\n  is the action of the inverse Hessian on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::invHessVec(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("precond", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Objective<double>::precond, "Apply preconditioner to vector.\n\n      This function applies a preconditioner for the Hessian of the objective function to the vector \n.\n      \n\n  is the action of the Hessian preconditioner on \n.\n      \n\n   is the direction vector.\n      \n\n   is the current iterate.\n      \n\n is a tolerance for inexact objective function computation.\n\nC++: ROL::Objective<double>::precond(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("prox", (void (ROL::Objective<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double &)) &ROL::Objective<double>::prox, "C++: ROL::Objective<double>::prox(class ROL::Vector<double> &, const class ROL::Vector<double> &, double, double &) --> void", pybind11::arg("Pv"), pybind11::arg("v"), pybind11::arg("t"), pybind11::arg("tol"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2); }, "", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool & a3) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3); }, "", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"));
		cl.def("checkHessSym", (class std::vector<double, class std::allocator<double> > (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Objective<double>::checkHessSym, "Hessian symmetry check.\n\n      This function checks the symmetry of the Hessian by comparing \n      \n\n\n\n\n\n      \n             is an optimization variable.\n      \n\n             is a direction vector.\n      \n\n             is a direction vector.\n      \n\n is a flag that turns on/off output.\n      \n\n     is the output stream.\n\nC++: ROL::Objective<double>::checkHessSym(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("x"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3); }, "", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"));
		cl.def("checkHessSym", [](ROL::Objective<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const bool & a4) -> std::vector<double, class std::allocator<double> > { return o.checkHessSym(a0, a1, a2, a3, a4); }, "", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"));
		cl.def("checkHessSym", (class std::vector<double, class std::allocator<double> > (ROL::Objective<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Objective<double>::checkHessSym, "Hessian symmetry check.\n\n      This function checks the symmetry of the Hessian by comparing \n      \n\n\n\n\n\n      \n             is an optimization variable.\n      \n\n            is used to create temporary Hessian-times-vector vectors.\n      \n\n             is a direction vector.\n      \n\n             is a direction vector.\n      \n\n is a flag that turns on/off output.\n      \n\n     is the output stream.\n\nC++: ROL::Objective<double>::checkHessSym(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("x"), pybind11::arg("hv"), pybind11::arg("v"), pybind11::arg("w"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("setParameter", (void (ROL::Objective<double>::*)(const class std::vector<double, class std::allocator<double> > &)) &ROL::Objective<double>::setParameter, "C++: ROL::Objective<double>::setParameter(const class std::vector<double, class std::allocator<double> > &) --> void", pybind11::arg("param"));
		cl.def("assign", (class ROL::Objective<double> & (ROL::Objective<double>::*)(const class ROL::Objective<double> &)) &ROL::Objective<double>::operator=, "C++: ROL::Objective<double>::operator=(const class ROL::Objective<double> &) --> class ROL::Objective<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::RiskLessConstraint file:ROL_RiskLessConstraint.hpp line:53
		pybind11::class_<ROL::RiskLessConstraint<double>, Teuchos::RCP<ROL::RiskLessConstraint<double>>, PyCallBack_ROL_RiskLessConstraint_double_t, ROL::Constraint<double>> cl(M("ROL"), "RiskLessConstraint_double_t", "", pybind11::module_local());
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Constraint<double> > &>(), pybind11::arg("con") );

		cl.def( pybind11::init( [](PyCallBack_ROL_RiskLessConstraint_double_t const &o){ return new PyCallBack_ROL_RiskLessConstraint_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::RiskLessConstraint<double> const &o){ return new ROL::RiskLessConstraint<double>(o); } ) );
		cl.def("applyAdjointJacobian", [](ROL::RiskLessConstraint<double> &o, class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, double & a4) -> void { return o.applyAdjointJacobian(a0, a1, a2, a3, a4); }, "", pybind11::arg("ajv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualv"), pybind11::arg("tol"));
		cl.def("update", [](ROL::RiskLessConstraint<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::RiskLessConstraint<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::RiskLessConstraint<double>::update, "C++: ROL::RiskLessConstraint<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::RiskLessConstraint<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::RiskLessConstraint<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::RiskLessConstraint<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::RiskLessConstraint<double>::update, "C++: ROL::RiskLessConstraint<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (void (ROL::RiskLessConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::RiskLessConstraint<double>::value, "C++: ROL::RiskLessConstraint<double>::value(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("c"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyJacobian", (void (ROL::RiskLessConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::RiskLessConstraint<double>::applyJacobian, "C++: ROL::RiskLessConstraint<double>::applyJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("jv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointJacobian", (void (ROL::RiskLessConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::RiskLessConstraint<double>::applyAdjointJacobian, "C++: ROL::RiskLessConstraint<double>::applyAdjointJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ajv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointHessian", (void (ROL::RiskLessConstraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::RiskLessConstraint<double>::applyAdjointHessian, "C++: ROL::RiskLessConstraint<double>::applyAdjointHessian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ahuv"), pybind11::arg("u"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("setParameter", (void (ROL::RiskLessConstraint<double>::*)(const class std::vector<double, class std::allocator<double> > &)) &ROL::RiskLessConstraint<double>::setParameter, "C++: ROL::RiskLessConstraint<double>::setParameter(const class std::vector<double, class std::allocator<double> > &) --> void", pybind11::arg("param"));
		cl.def("update", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, enum ROL::UpdateType const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("type"));
		cl.def("update", (void (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, enum ROL::UpdateType, int)) &ROL::Constraint<double>::update, "Update constraint function. \n\n      This function updates the constraint function at new iterations. \n      \n\n      is the new iterate. \n      \n\n   is the type of update requested.\n      \n\n   is the outer algorithm iterations count.\n\nC++: ROL::Constraint<double>::update(const class ROL::Vector<double> &, enum ROL::UpdateType, int) --> void", pybind11::arg("x"), pybind11::arg("type"), pybind11::arg("iter"));
		cl.def("update", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0) -> void { return o.update(a0); }, "", pybind11::arg("x"));
		cl.def("update", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, bool const & a1) -> void { return o.update(a0, a1); }, "", pybind11::arg("x"), pybind11::arg("flag"));
		cl.def("update", (void (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, bool, int)) &ROL::Constraint<double>::update, "Update constraint functions.  \n                x is the optimization variable, \n                flag = true if optimization variable is changed,\n                iter is the outer algorithm iterations count.\n\nC++: ROL::Constraint<double>::update(const class ROL::Vector<double> &, bool, int) --> void", pybind11::arg("x"), pybind11::arg("flag"), pybind11::arg("iter"));
		cl.def("value", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::value, "C++: ROL::Constraint<double>::value(class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("c"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyJacobian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyJacobian, "Apply the constraint Jacobian at \n, \n,\n             to vector \n\n.\n\n             \n  is the result of applying the constraint Jacobian to  at  a constraint-space vector\n             \n\n   is an optimization-space vector\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is a finite-difference approximation.\n\n             ---\n\nC++: ROL::Constraint<double>::applyJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("jv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointJacobian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyAdjointJacobian, "Apply the adjoint of the the constraint Jacobian at \n, \n,\n             to vector \n\n.\n\n             \n is the result of applying the adjoint of the constraint Jacobian to  at  a dual optimization-space vector\n             \n\n   is a dual constraint-space vector\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is a finite-difference approximation.\n\n             ---\n\nC++: ROL::Constraint<double>::applyAdjointJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ajv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyAdjointJacobian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyAdjointJacobian, "Apply the adjoint of the the constraint Jacobian at \n, \n,\n             to vector \n\n.\n\n             \n is the result of applying the adjoint of the constraint Jacobian to  at  a dual optimization-space vector\n             \n\n   is a dual constraint-space vector\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n  is a vector used for temporary variables; a constraint-space vector\n             \n\n is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is a finite-difference approximation.\n\n             ---\n\nC++: ROL::Constraint<double>::applyAdjointJacobian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("ajv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualv"), pybind11::arg("tol"));
		cl.def("applyAdjointHessian", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyAdjointHessian, "Apply the derivative of the adjoint of the constraint Jacobian at \n\n             to vector \n in direction \n,\n             according to \n\n.\n\n             \n is the result of applying the derivative of the adjoint of the constraint Jacobian at  to vector  in direction  a dual optimization-space vector\n             \n\n    is the direction vector; a dual constraint-space vector\n             \n\n    is an optimization-space vector\n             \n\n    is the constraint argument; an optimization-space vector\n             \n\n  is a tolerance for inexact evaluations; currently unused\n\n             On return, \n, where\n             \n\n, \n, and \n. \n             The default implementation is a finite-difference approximation based on the adjoint Jacobian.\n\n             ---\n\nC++: ROL::Constraint<double>::applyAdjointHessian(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("huv"), pybind11::arg("u"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("solveAugmentedSystem", (class std::vector<double, class std::allocator<double> > (ROL::Constraint<double>::*)(class ROL::Vector<double> &, class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::solveAugmentedSystem, "Approximately solves the  augmented system \n             \n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n             where \n, \n,\n             \n\n, \n,\n             \n\n is an identity or Riesz\n             operator, and \n\n\n             is a zero operator.\n\n             \n  is the optimization-space component of the result\n             \n\n  is the dual constraint-space component of the result\n             \n\n  is the dual optimization-space component of the right-hand side\n             \n\n  is the constraint-space component of the right-hand side\n             \n\n   is the constraint argument; an optimization-space vector\n             \n\n is the nominal relative residual tolerance\n\n             On return, \n approximately\n             solves the augmented system, where the size of the residual is\n             governed by special stopping conditions. \n             The default implementation is the preconditioned generalized\n             minimal residual (GMRES) method, which enables the use of\n             nonsymmetric preconditioners.\n\n             ---\n\nC++: ROL::Constraint<double>::solveAugmentedSystem(class ROL::Vector<double> &, class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> class std::vector<double, class std::allocator<double> >", pybind11::arg("v1"), pybind11::arg("v2"), pybind11::arg("b1"), pybind11::arg("b2"), pybind11::arg("x"), pybind11::arg("tol"));
		cl.def("applyPreconditioner", (void (ROL::Constraint<double>::*)(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &)) &ROL::Constraint<double>::applyPreconditioner, "Apply a constraint preconditioner at \n, \n,\n             to vector \n\n.  Ideally, this preconditioner satisfies the following relationship:\n             \n\n\n\n             where R is the appropriate Riesz map in \n.  It is used by the #solveAugmentedSystem method.\n\n             \n  is the result of applying the constraint preconditioner to  at  a dual constraint-space vector\n             \n\n   is a constraint-space vector\n             \n\n   is the preconditioner argument; an optimization-space vector\n             \n\n   is the preconditioner argument; a dual optimization-space vector, unused\n             \n\n is a tolerance for inexact evaluations\n\n             On return, \n, where\n             \n\n, \n. \n             The default implementation is the Riesz map in \n\n.\n\n             ---\n\nC++: ROL::Constraint<double>::applyPreconditioner(class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, double &) --> void", pybind11::arg("pv"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("g"), pybind11::arg("tol"));
		cl.def("activate", (void (ROL::Constraint<double>::*)()) &ROL::Constraint<double>::activate, "Turn on constraints \n\nC++: ROL::Constraint<double>::activate() --> void");
		cl.def("deactivate", (void (ROL::Constraint<double>::*)()) &ROL::Constraint<double>::deactivate, "Turn off constraints\n\nC++: ROL::Constraint<double>::deactivate() --> void");
		cl.def("isActivated", (bool (ROL::Constraint<double>::*)()) &ROL::Constraint<double>::isActivated, "Check if constraints are on\n\nC++: ROL::Constraint<double>::isActivated() --> bool");
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"));
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const bool & a3) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2, a3); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("printToStream"));
		cl.def("checkAdjointConsistencyJacobian", (double (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Constraint<double>::checkAdjointConsistencyJacobian, "C++: ROL::Constraint<double>::checkAdjointConsistencyJacobian(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> double", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualw"), pybind11::arg("dualv"));
		cl.def("checkAdjointConsistencyJacobian", [](ROL::Constraint<double> &o, const class ROL::Vector<double> & a0, const class ROL::Vector<double> & a1, const class ROL::Vector<double> & a2, const class ROL::Vector<double> & a3, const class ROL::Vector<double> & a4, const bool & a5) -> double { return o.checkAdjointConsistencyJacobian(a0, a1, a2, a3, a4, a5); }, "", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualw"), pybind11::arg("dualv"), pybind11::arg("printToStream"));
		cl.def("checkAdjointConsistencyJacobian", (double (ROL::Constraint<double>::*)(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &)) &ROL::Constraint<double>::checkAdjointConsistencyJacobian, "C++: ROL::Constraint<double>::checkAdjointConsistencyJacobian(const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const class ROL::Vector<double> &, const bool, std::ostream &) --> double", pybind11::arg("w"), pybind11::arg("v"), pybind11::arg("x"), pybind11::arg("dualw"), pybind11::arg("dualv"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("setParameter", (void (ROL::Constraint<double>::*)(const class std::vector<double, class std::allocator<double> > &)) &ROL::Constraint<double>::setParameter, "C++: ROL::Constraint<double>::setParameter(const class std::vector<double, class std::allocator<double> > &) --> void", pybind11::arg("param"));
		cl.def("assign", (class ROL::Constraint<double> & (ROL::Constraint<double>::*)(const class ROL::Constraint<double> &)) &ROL::Constraint<double>::operator=, "C++: ROL::Constraint<double>::operator=(const class ROL::Constraint<double> &) --> class ROL::Constraint<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
	{ // ROL::Problem file: line:49
		pybind11::class_<ROL::Problem<double>, Teuchos::RCP<ROL::Problem<double>>, PyCallBack_ROL_Problem_double_t> cl(M("ROL"), "Problem_double_t", "", pybind11::module_local());
		cl.def( pybind11::init( [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::Vector<double> > & a1){ return new ROL::Problem<double>(a0, a1); }, [](const class Teuchos::RCP<class ROL::Objective<double> > & a0, const class Teuchos::RCP<class ROL::Vector<double> > & a1){ return new PyCallBack_ROL_Problem_double_t(a0, a1); } ), "doc");
		cl.def( pybind11::init<const class Teuchos::RCP<class ROL::Objective<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &>(), pybind11::arg("obj"), pybind11::arg("x"), pybind11::arg("g") );

		cl.def( pybind11::init( [](PyCallBack_ROL_Problem_double_t const &o){ return new PyCallBack_ROL_Problem_double_t(o); } ) );
		cl.def( pybind11::init( [](ROL::Problem<double> const &o){ return new ROL::Problem<double>(o); } ) );
		cl.def("addBoundConstraint", (void (ROL::Problem<double>::*)(const class Teuchos::RCP<class ROL::BoundConstraint<double> > &)) &ROL::Problem<double>::addBoundConstraint, "Add a bound constraint.\n\n      \n  bound constraint object\n\nC++: ROL::Problem<double>::addBoundConstraint(const class Teuchos::RCP<class ROL::BoundConstraint<double> > &) --> void", pybind11::arg("bnd"));
		cl.def("removeBoundConstraint", (void (ROL::Problem<double>::*)()) &ROL::Problem<double>::removeBoundConstraint, "Remove an existing bound constraint.\n\nC++: ROL::Problem<double>::removeBoundConstraint() --> void");
		cl.def("addConstraint", [](ROL::Problem<double> &o, std::string const & a0, const class Teuchos::RCP<class ROL::Constraint<double> > & a1, const class Teuchos::RCP<class ROL::Vector<double> > & a2) -> void { return o.addConstraint(a0, a1, a2); }, "", pybind11::arg("name"), pybind11::arg("econ"), pybind11::arg("emul"));
		cl.def("addConstraint", [](ROL::Problem<double> &o, std::string const & a0, const class Teuchos::RCP<class ROL::Constraint<double> > & a1, const class Teuchos::RCP<class ROL::Vector<double> > & a2, const class Teuchos::RCP<class ROL::Vector<double> > & a3) -> void { return o.addConstraint(a0, a1, a2, a3); }, "", pybind11::arg("name"), pybind11::arg("econ"), pybind11::arg("emul"), pybind11::arg("eres"));
		cl.def("addConstraint", (void (ROL::Problem<double>::*)(std::string, const class Teuchos::RCP<class ROL::Constraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, bool)) &ROL::Problem<double>::addConstraint, "Add an equality constraint.\n\n      \n   the unique constraint identifier\n      \n\n   constraint object\n      \n\n   dual constraint space vector\n      \n\n   primal constraint space vector\n      \n\n  whether or not to clear constraint container\n\nC++: ROL::Problem<double>::addConstraint(std::string, const class Teuchos::RCP<class ROL::Constraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, bool) --> void", pybind11::arg("name"), pybind11::arg("econ"), pybind11::arg("emul"), pybind11::arg("eres"), pybind11::arg("reset"));
		cl.def("addConstraint", [](ROL::Problem<double> &o, std::string const & a0, const class Teuchos::RCP<class ROL::Constraint<double> > & a1, const class Teuchos::RCP<class ROL::Vector<double> > & a2, const class Teuchos::RCP<class ROL::BoundConstraint<double> > & a3) -> void { return o.addConstraint(a0, a1, a2, a3); }, "", pybind11::arg("name"), pybind11::arg("icon"), pybind11::arg("imul"), pybind11::arg("ibnd"));
		cl.def("addConstraint", [](ROL::Problem<double> &o, std::string const & a0, const class Teuchos::RCP<class ROL::Constraint<double> > & a1, const class Teuchos::RCP<class ROL::Vector<double> > & a2, const class Teuchos::RCP<class ROL::BoundConstraint<double> > & a3, const class Teuchos::RCP<class ROL::Vector<double> > & a4) -> void { return o.addConstraint(a0, a1, a2, a3, a4); }, "", pybind11::arg("name"), pybind11::arg("icon"), pybind11::arg("imul"), pybind11::arg("ibnd"), pybind11::arg("ires"));
		cl.def("addConstraint", (void (ROL::Problem<double>::*)(std::string, const class Teuchos::RCP<class ROL::Constraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, const class Teuchos::RCP<class ROL::BoundConstraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, bool)) &ROL::Problem<double>::addConstraint, "Add an inequality constraint.\n\n      \n   the unique constraint identifier\n      \n\n   constraint object\n      \n\n   dual constraint space vector\n      \n\n   bound constraint\n      \n\n   primal constraint space vector\n      \n\n  whether or not to clear constraint container\n\nC++: ROL::Problem<double>::addConstraint(std::string, const class Teuchos::RCP<class ROL::Constraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, const class Teuchos::RCP<class ROL::BoundConstraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, bool) --> void", pybind11::arg("name"), pybind11::arg("icon"), pybind11::arg("imul"), pybind11::arg("ibnd"), pybind11::arg("ires"), pybind11::arg("reset"));
		cl.def("removeConstraint", (void (ROL::Problem<double>::*)(std::string)) &ROL::Problem<double>::removeConstraint, "Remove an existing constraint.\n\n      \n  the unique constraint identifier\n\nC++: ROL::Problem<double>::removeConstraint(std::string) --> void", pybind11::arg("name"));
		cl.def("addLinearConstraint", [](ROL::Problem<double> &o, std::string const & a0, const class Teuchos::RCP<class ROL::Constraint<double> > & a1, const class Teuchos::RCP<class ROL::Vector<double> > & a2) -> void { return o.addLinearConstraint(a0, a1, a2); }, "", pybind11::arg("name"), pybind11::arg("linear_econ"), pybind11::arg("linear_emul"));
		cl.def("addLinearConstraint", [](ROL::Problem<double> &o, std::string const & a0, const class Teuchos::RCP<class ROL::Constraint<double> > & a1, const class Teuchos::RCP<class ROL::Vector<double> > & a2, const class Teuchos::RCP<class ROL::Vector<double> > & a3) -> void { return o.addLinearConstraint(a0, a1, a2, a3); }, "", pybind11::arg("name"), pybind11::arg("linear_econ"), pybind11::arg("linear_emul"), pybind11::arg("linear_eres"));
		cl.def("addLinearConstraint", (void (ROL::Problem<double>::*)(std::string, const class Teuchos::RCP<class ROL::Constraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, bool)) &ROL::Problem<double>::addLinearConstraint, "Add a linear equality constraint.\n\n      \n         the unique constraint identifier\n      \n\n  constraint object\n      \n\n  dual constraint space vector\n      \n\n  primal constraint space vector\n      \n\n        whether or not to clear linear constraint container\n\nC++: ROL::Problem<double>::addLinearConstraint(std::string, const class Teuchos::RCP<class ROL::Constraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, bool) --> void", pybind11::arg("name"), pybind11::arg("linear_econ"), pybind11::arg("linear_emul"), pybind11::arg("linear_eres"), pybind11::arg("reset"));
		cl.def("addLinearConstraint", [](ROL::Problem<double> &o, std::string const & a0, const class Teuchos::RCP<class ROL::Constraint<double> > & a1, const class Teuchos::RCP<class ROL::Vector<double> > & a2, const class Teuchos::RCP<class ROL::BoundConstraint<double> > & a3) -> void { return o.addLinearConstraint(a0, a1, a2, a3); }, "", pybind11::arg("name"), pybind11::arg("linear_icon"), pybind11::arg("linear_imul"), pybind11::arg("linear_ibnd"));
		cl.def("addLinearConstraint", [](ROL::Problem<double> &o, std::string const & a0, const class Teuchos::RCP<class ROL::Constraint<double> > & a1, const class Teuchos::RCP<class ROL::Vector<double> > & a2, const class Teuchos::RCP<class ROL::BoundConstraint<double> > & a3, const class Teuchos::RCP<class ROL::Vector<double> > & a4) -> void { return o.addLinearConstraint(a0, a1, a2, a3, a4); }, "", pybind11::arg("name"), pybind11::arg("linear_icon"), pybind11::arg("linear_imul"), pybind11::arg("linear_ibnd"), pybind11::arg("linear_ires"));
		cl.def("addLinearConstraint", (void (ROL::Problem<double>::*)(std::string, const class Teuchos::RCP<class ROL::Constraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, const class Teuchos::RCP<class ROL::BoundConstraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, bool)) &ROL::Problem<double>::addLinearConstraint, "Add a linear inequality constraint.\n\n      \n         the unique constraint identifier\n      \n\n  constraint object\n      \n\n  dual constraint space vector\n      \n\n  bound constraint\n      \n\n  primal constraint space vector\n      \n\n        whether or not to clear linear constraint container\n\nC++: ROL::Problem<double>::addLinearConstraint(std::string, const class Teuchos::RCP<class ROL::Constraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, const class Teuchos::RCP<class ROL::BoundConstraint<double> > &, const class Teuchos::RCP<class ROL::Vector<double> > &, bool) --> void", pybind11::arg("name"), pybind11::arg("linear_icon"), pybind11::arg("linear_imul"), pybind11::arg("linear_ibnd"), pybind11::arg("linear_ires"), pybind11::arg("reset"));
		cl.def("removeLinearConstraint", (void (ROL::Problem<double>::*)(std::string)) &ROL::Problem<double>::removeLinearConstraint, "Remove an existing linear constraint.\n\n      \n  the unique constraint identifier\n\nC++: ROL::Problem<double>::removeLinearConstraint(std::string) --> void", pybind11::arg("name"));
		cl.def("setProjectionAlgorithm", (void (ROL::Problem<double>::*)(class Teuchos::ParameterList &)) &ROL::Problem<double>::setProjectionAlgorithm, "Set polyhedral projection algorithm.\n\n      \n  polyhedral projection algorithm\n\nC++: ROL::Problem<double>::setProjectionAlgorithm(class Teuchos::ParameterList &) --> void", pybind11::arg("list"));
		cl.def("getObjective", (const class Teuchos::RCP<class ROL::Objective<double> > & (ROL::Problem<double>::*)()) &ROL::Problem<double>::getObjective, "Get the objective function.\n\nC++: ROL::Problem<double>::getObjective() --> const class Teuchos::RCP<class ROL::Objective<double> > &", pybind11::return_value_policy::automatic);
		cl.def("getPrimalOptimizationVector", (const class Teuchos::RCP<class ROL::Vector<double> > & (ROL::Problem<double>::*)()) &ROL::Problem<double>::getPrimalOptimizationVector, "Get the primal optimization space vector.\n\nC++: ROL::Problem<double>::getPrimalOptimizationVector() --> const class Teuchos::RCP<class ROL::Vector<double> > &", pybind11::return_value_policy::automatic);
		cl.def("getDualOptimizationVector", (const class Teuchos::RCP<class ROL::Vector<double> > & (ROL::Problem<double>::*)()) &ROL::Problem<double>::getDualOptimizationVector, "Get the dual optimization space vector.\n\nC++: ROL::Problem<double>::getDualOptimizationVector() --> const class Teuchos::RCP<class ROL::Vector<double> > &", pybind11::return_value_policy::automatic);
		cl.def("getBoundConstraint", (const class Teuchos::RCP<class ROL::BoundConstraint<double> > & (ROL::Problem<double>::*)()) &ROL::Problem<double>::getBoundConstraint, "Get the bound constraint.\n\nC++: ROL::Problem<double>::getBoundConstraint() --> const class Teuchos::RCP<class ROL::BoundConstraint<double> > &", pybind11::return_value_policy::automatic);
		cl.def("getConstraint", (const class Teuchos::RCP<class ROL::Constraint<double> > & (ROL::Problem<double>::*)()) &ROL::Problem<double>::getConstraint, "Get the equality constraint.\n\nC++: ROL::Problem<double>::getConstraint() --> const class Teuchos::RCP<class ROL::Constraint<double> > &", pybind11::return_value_policy::automatic);
		cl.def("getMultiplierVector", (const class Teuchos::RCP<class ROL::Vector<double> > & (ROL::Problem<double>::*)()) &ROL::Problem<double>::getMultiplierVector, "Get the dual constraint space vector.\n\nC++: ROL::Problem<double>::getMultiplierVector() --> const class Teuchos::RCP<class ROL::Vector<double> > &", pybind11::return_value_policy::automatic);
		cl.def("getResidualVector", (const class Teuchos::RCP<class ROL::Vector<double> > & (ROL::Problem<double>::*)()) &ROL::Problem<double>::getResidualVector, "Get the primal constraint space vector.\n\nC++: ROL::Problem<double>::getResidualVector() --> const class Teuchos::RCP<class ROL::Vector<double> > &", pybind11::return_value_policy::automatic);
		cl.def("getPolyhedralProjection", (const class Teuchos::RCP<class ROL::PolyhedralProjection<double> > & (ROL::Problem<double>::*)()) &ROL::Problem<double>::getPolyhedralProjection, "Get the polyhedral projection object.  This is a null pointer if\n             no linear constraints and/or bounds are present.\n\nC++: ROL::Problem<double>::getPolyhedralProjection() --> const class Teuchos::RCP<class ROL::PolyhedralProjection<double> > &", pybind11::return_value_policy::automatic);
		cl.def("getProblemType", (enum ROL::EProblem (ROL::Problem<double>::*)()) &ROL::Problem<double>::getProblemType, "Get the optimization problem type (U, B, E, or G).\n\nC++: ROL::Problem<double>::getProblemType() --> enum ROL::EProblem");
		cl.def("checkLinearity", [](ROL::Problem<double> const &o) -> double { return o.checkLinearity(); }, "");
		cl.def("checkLinearity", [](ROL::Problem<double> const &o, bool const & a0) -> double { return o.checkLinearity(a0); }, "", pybind11::arg("printToStream"));
		cl.def("checkLinearity", (double (ROL::Problem<double>::*)(bool, std::ostream &) const) &ROL::Problem<double>::checkLinearity, "Check if user-supplied linear constraints are affine.\n\n      This function computes the error\n      \n\n\n\n      for each user-supplied linear constraint and returns the maximum.\n      \n\n   determines whether to print to the supplied std::ostream\n      \n\n       user supplied std::ostream\n\nC++: ROL::Problem<double>::checkLinearity(bool, std::ostream &) const --> double", pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("checkVectors", [](ROL::Problem<double> const &o) -> void { return o.checkVectors(); }, "");
		cl.def("checkVectors", [](ROL::Problem<double> const &o, bool const & a0) -> void { return o.checkVectors(a0); }, "", pybind11::arg("printToStream"));
		cl.def("checkVectors", (void (ROL::Problem<double>::*)(bool, std::ostream &) const) &ROL::Problem<double>::checkVectors, "Run vector checks for user-supplied vectors.\n\n      \n   determines whether to print to the supplied std::ostream\n      \n\n       user supplied std::ostream\n\nC++: ROL::Problem<double>::checkVectors(bool, std::ostream &) const --> void", pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("checkDerivatives", [](ROL::Problem<double> const &o) -> void { return o.checkDerivatives(); }, "");
		cl.def("checkDerivatives", [](ROL::Problem<double> const &o, bool const & a0) -> void { return o.checkDerivatives(a0); }, "", pybind11::arg("printToStream"));
		cl.def("checkDerivatives", [](ROL::Problem<double> const &o, bool const & a0, std::ostream & a1) -> void { return o.checkDerivatives(a0, a1); }, "", pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("checkDerivatives", [](ROL::Problem<double> const &o, bool const & a0, std::ostream & a1, const class Teuchos::RCP<class ROL::Vector<double> > & a2) -> void { return o.checkDerivatives(a0, a1, a2); }, "", pybind11::arg("printToStream"), pybind11::arg("outStream"), pybind11::arg("x0"));
		cl.def("checkDerivatives", (void (ROL::Problem<double>::*)(bool, std::ostream &, const class Teuchos::RCP<class ROL::Vector<double> > &, double) const) &ROL::Problem<double>::checkDerivatives, "Run derivative checks for user-supplied objective function and constraints.\n\n      \n   determines whether to print to the supplied std::ostream\n      \n\n       user supplied std::ostream\n\nC++: ROL::Problem<double>::checkDerivatives(bool, std::ostream &, const class Teuchos::RCP<class ROL::Vector<double> > &, double) const --> void", pybind11::arg("printToStream"), pybind11::arg("outStream"), pybind11::arg("x0"), pybind11::arg("scale"));
		cl.def("check", [](ROL::Problem<double> const &o) -> void { return o.check(); }, "");
		cl.def("check", [](ROL::Problem<double> const &o, bool const & a0) -> void { return o.check(a0); }, "", pybind11::arg("printToStream"));
		cl.def("check", [](ROL::Problem<double> const &o, bool const & a0, std::ostream & a1) -> void { return o.check(a0, a1); }, "", pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("check", [](ROL::Problem<double> const &o, bool const & a0, std::ostream & a1, const class Teuchos::RCP<class ROL::Vector<double> > & a2) -> void { return o.check(a0, a1, a2); }, "", pybind11::arg("printToStream"), pybind11::arg("outStream"), pybind11::arg("x0"));
		cl.def("check", (void (ROL::Problem<double>::*)(bool, std::ostream &, const class Teuchos::RCP<class ROL::Vector<double> > &, double) const) &ROL::Problem<double>::check, "Run vector, linearity and derivative checks for user-supplied\n             vectors, objective function and constraints.\n\n      \n   determines whether to print to the supplied std::ostream\n      \n\n       user supplied std::ostream\n\nC++: ROL::Problem<double>::check(bool, std::ostream &, const class Teuchos::RCP<class ROL::Vector<double> > &, double) const --> void", pybind11::arg("printToStream"), pybind11::arg("outStream"), pybind11::arg("x0"), pybind11::arg("scale"));
		cl.def("finalize", [](ROL::Problem<double> &o) -> void { return o.finalize(); }, "");
		cl.def("finalize", [](ROL::Problem<double> &o, bool const & a0) -> void { return o.finalize(a0); }, "", pybind11::arg("lumpConstraints"));
		cl.def("finalize", [](ROL::Problem<double> &o, bool const & a0, bool const & a1) -> void { return o.finalize(a0, a1); }, "", pybind11::arg("lumpConstraints"), pybind11::arg("printToStream"));
		cl.def("finalize", (void (ROL::Problem<double>::*)(bool, bool, std::ostream &)) &ROL::Problem<double>::finalize, "Tranform user-supplied constraints to consist of only bounds\n             and equalities.  Optimization problem cannot be modified after\n             finalize has been called without calling the edit function.\n\n      \n combine both linear and nonlinear constraints\n      \n\n   determines whether to print to the supplied std::ostream\n      \n\n       user supplied std::ostream\n\nC++: ROL::Problem<double>::finalize(bool, bool, std::ostream &) --> void", pybind11::arg("lumpConstraints"), pybind11::arg("printToStream"), pybind11::arg("outStream"));
		cl.def("isFinalized", (bool (ROL::Problem<double>::*)() const) &ROL::Problem<double>::isFinalized, "Indicate whether or no finalize has been called.\n\nC++: ROL::Problem<double>::isFinalized() const --> bool");
		cl.def("edit", (void (ROL::Problem<double>::*)()) &ROL::Problem<double>::edit, "Resume editting optimization problem after finalize has been called.\n\nC++: ROL::Problem<double>::edit() --> void");
		cl.def("finalizeIteration", (void (ROL::Problem<double>::*)()) &ROL::Problem<double>::finalizeIteration, "Transform the optimization variables to the native\n             parameterization after an optimization algorithm has finished.\n\nC++: ROL::Problem<double>::finalizeIteration() --> void");
		cl.def("assign", (class ROL::Problem<double> & (ROL::Problem<double>::*)(const class ROL::Problem<double> &)) &ROL::Problem<double>::operator=, "C++: ROL::Problem<double>::operator=(const class ROL::Problem<double> &) --> class ROL::Problem<double> &", pybind11::return_value_policy::automatic, pybind11::arg(""));
	}
}
